//-----------------------------------------------------------------------------
// Copyright  2008 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// program	color_steps
// file		color_steps.c
//
// purpose	Convert a number of colors and first/last color codes into a
//		list of color codes.
//
// syntax	color_steps  number  firstcolor  lastcolor
//
// compile	gcc -std=gnu99 -O2 -o color_steps color_steps.c -lm
// note		-lh is not really needed since only macros are used.
//-----------------------------------------------------------------------------

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libh/arith.h>
#include <libh/string.h>

int
main (
    int		argc
    ,
    char * *	argv
    ,
    char * *	envp
    )
{
    double g,r1,g1,b1,r2,g2,b2,s,t;
    unsigned int c,n;

    g = ( argc > 4 ) ? strtod( argv[4], NULL ) : PIX_GAMMA_PC;

    c = ( argc > 1 ) ? strtoul( argv[1], NULL, 0 ) : 5;
    if ( c <= 0 ) c = 5;

    n = ( argc > 2 ) ? strtoul( argv[2], NULL, 0 ) : 0x000000;
    r1 = pix_code_to_linear( g, 255 & ( n >> 16 ), 256 );
    g1 = pix_code_to_linear( g, 255 & ( n >>  8 ), 256 );
    b1 = pix_code_to_linear( g, 255 &   n        , 256 );

    n = ( argc > 3 ) ? strtoul( argv[3], NULL, 0 ) : 0xffffff;
    r2 = pix_code_to_linear( g, 255 & ( n >> 16 ), 256 );
    g2 = pix_code_to_linear( g, 255 & ( n >>  8 ), 256 );
    b2 = pix_code_to_linear( g, 255 &   n        , 256 );

    s = (double) ( c - 1 );
    t = 0.0;
    for ( n = 0; n < c; ++ n ) {
	printf( "%5u %02x%02x%02x\n", n,
		pix_linear_to_code( g, rescale0( r1, r2, s, t ), 256 ),
		pix_linear_to_code( g, rescale0( g1, g2, s, t ), 256 ),
		pix_linear_to_code( g, rescale0( b1, b2, s, t ), 256 ) );
	t += 1.0;
    }

    return 0;
}
