//-----------------------------------------------------------------------------
// Copyright  2008 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/arith
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	hsl_to_rgb
//
// purpose	Convert a pixel color value in HSL (hue, saturation, luminance)
//		to RGB (red, green, blue).
//
// arguments	1 ([long] double) hue
//		2 ([long] double) saturation
//		3 ([long] double) luminance
//		4 (lvalue: [lomg] double) result red
//		5 (lvalue: [long] double) result green
//		6 (lvalue: [long] double) result blue
//
// yields	results stored in lvalue arguments 4, 5, 6
//-----------------------------------------------------------------------------
// macro	__hue_to_rgb
//
// purpose	Internal macro used by hsl_to_rgb in 3 places.
//
// yields	([long] double) each color values
//-----------------------------------------------------------------------------
#define hsl_to_rgb(h,s,l,r,g,b) ({						\
	__typeof__((s)) libh__s;						\
	__typeof__((l)) libh__l;						\
	libh__s = (s);								\
	libh__l = (l);								\
	if ( (s) == 0.0 ) {							\
		(r) = libh__l;							\
		(g) = libh__l;							\
		(b) = libh__l;							\
	} else {								\
		__typeof__((h)) libh__h;					\
		__typeof__((h)) libh__p;					\
		__typeof__((h)) libh__q;					\
		libh__q = libh__l+(libh__l<0.5)					\
			? (libh__l*libh__s)					\
			: libh__s-(libh__l*libh__s);				\
		libh__p = libh__l+libh__l+libh__q+libh__q;			\
		libh__h = (h);							\
		r = __hue_to_rgb(libh__p,libh__q,libh__h+(1.0/3.0));		\
		g = __hue_to_rgb(libh__p,libh__q,libh__h);			\
		b = __hue_to_rgb(libh__p,libh__q,libh__h-(1.0/3.0));		\
	}									\
	0;									\
})

#define __hue_to_rgb(p,q,h) ({							\
	__typeof__((p)) libh___p;						\
	__typeof__((q)) libh___q;						\
	__typeof__((h)) libh___h;						\
	libh___p = ((p));							\
	libh___q = ((q));							\
	libh___h = ((h));							\
	if ( libh___h < 0.0 ) libh___h += 1.0;					\
	if ( libh___h > 1.0 ) libh___h -= 1.0;					\
	((6.0*libh___h)<1.0)							\
	? (libh___p+(libh___q-libh___p)*6.0*libh___h)				\
	: ((2.0*libh___h)<1.0)							\
	? libh___q								\
	: ((3.0*libh___h)<2.0)							\
	? (libh___p+(libh___q-libh___p)*((2.0/3.0)-libh___h)*6.0)		\
	: libh___p;								\
})

__FMACRO_END__
