//-----------------------------------------------------------------------------
// Copyright  2008 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/arith
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	hsv_to_rgb
//
// purpose	Convert a pixel color value in HSL (hue, saturation, value)
//		to RGB (red, green, blue).
//
// arguments	1 ([long] double) hue
//		2 ([long] double) saturation
//		3 ([long] double) value
//		4 (lvalue: [lomg] double) result red
//		5 (lvalue: [long] double) result green
//		6 (lvalue: [long] double) result blue
//
// yields	results stored in lvalue arguments 4, 5, 6
//-----------------------------------------------------------------------------
#define hsv_to_rgb(h,s,v,r,g,b) ({						\
	__typeof__((h)) libh__h;						\
	__typeof__((s)) libh__s;						\
	__typeof__((v)) libh__v;						\
	libh__s = (s);								\
	libh__v = (v);								\
	if ( libh__s == 0.0 ) {							\
		(r) = libh__v;							\
		(g) = libh__v;							\
		(b) = libh__v;							\
	} else {								\
		__typeof__((h)) libh__h;					\
		__typeof__((h)) libh__i;					\
		__typeof__((v)) libh__x;					\
		__typeof__((v)) libh__y;					\
		__typeof__((v)) libh__z;					\
		libh__h = (h)*6.0;						\
		libh__i = floor(libh__h);					\
		libh__h -= libh__i;						\
		libh__x = libh__v*(1.0-libh__s);				\
		libh__y = libh__v*(1.0-libh__s*libh__h);			\
		libh__z = libh__v*(1.0-libh__s*(1.0-libh__h));			\
		if      (libh__i==0.0) {(r)=libh__v;(g)=libh__z;(b)=libh__x;}	\
		else if (libh__i==1.0) {(r)=libh__y;(g)=libh__v;(b)=libh__x;}	\
		else if (libh__i==2.0) {(r)=libh__x;(g)=libh__v;(b)=libh__z;}	\
		else if (libh__i==3.0) {(r)=libh__x;(g)=libh__y;(b)=libh__v;}	\
		else if (libh__i==4.0) {(r)=libh__z;(g)=libh__x;(b)=libh__v;}	\
		else                   {(r)=libh__v;(g)=libh__x;(b)=libh__y;}	\
	}									\
	0;									\
})

__FMACRO_END__
