//-----------------------------------------------------------------------------
// Copyright  2008 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/arith
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	pix_code_to_linear
//
// purpose	Convert an integer (int) pixel intensity value in the range
//		0 .. N-1 to a gamma adjusted floating point (double pixel
//		intensity value in the range 0.0 .. 1.0.
//
// arguments	1 (double) gamma correction factor to use
//		2 (int) pixel coded integer value 0 .. N-1
//		3 (int) pixel coded integer scale N
//
// returns	(double) pixel intensity in the range 0.0 .. 1.0
//-----------------------------------------------------------------------------
#define pix_code_to_linear(g,cv,cs) ({						\
	int	libh__c;							\
	int	libh__s;							\
	libh__c = (cv);								\
	libh__s = (cs);								\
	( libh__s <= 1 ) ? 0.0 :						\
	( libh__c >= libh__s ) ? 1.0 :						\
	( libh__c <= 0 ) ? 0.0 :						\
	({	double	libh__t;						\
		libh__t = exp( log( (double) libh__c / (double) ( libh__s - 1 ) ) * (g) );	\
		( libh__t <= 0.0 ) ? 0.0 : ( libh__t >= 1.0 ) ? 1.0 : libh__t;	\
	});									\
})

//-----------------------------------------------------------------------------
// macro	pix_code_to_linear_scaled
//
// purpose	Convert an integer (int) pixel intensity value in the range
//		0 .. N-1 to a gamma adjusted floating point (double pixel
//		intensity value in the range 0.0 .. S.
//
// arguments	1 (double) gamma correction factor to use
//		2 (double) pixel linear scale S
//		3 (int) pixel coded integer value 0 .. N-1
//		4 (int) pixel coded integer scale N
//
// returns	(double) pixel intensity in the range 0.0 .. S
//-----------------------------------------------------------------------------
#define pix_code_to_linear_scaled(g,ls,cv,cs) ({				\
	double	libh__l;							\
	int	libh__c;							\
	int	libh__s;							\
	libh__l = (ls);								\
	libh__c = (cv);								\
	libh__s = (cs);								\
	( libh__s <= 1 ) ? 0.0 :						\
	( libh__c >= libh__s ) ? libh__l :					\
	( libh__c <= 0 ) ? 0.0 :						\
	({	double	libh__t;						\
		libh__t = exp( log( (double) libh__c / (double) ( libh__s - 1 ) ) * (g) );	\
		libh__l * ( ( libh__t <= 0.0 ) ? 0.0 : ( libh__t >= 1.0 ) ? 1.0 : libh__t );	\
	});									\
})

__FMACRO_END__
