//-----------------------------------------------------------------------------
// Copyright  2008 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/arith
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	pix_linear_to_code
//
// purpose	Convert a floating point (double) pixel intensity value in
//		the range 0.0 .. 1.0 to a gamma corrected integer (int) pixel
//		intensity value in the range 0 .. N-1.
//
// arguments	1 (double) gamma correction factor to use
//		2 (double) linear pixel intensity from 0.0 to 1.0
//		3 (int) coded integer scale N
//
// returns	(int) pixel intensity in the range 0 to N-1
//-----------------------------------------------------------------------------
#define pix_linear_to_code(g,lv,cs) ({						\
	double libh__l;								\
	double libh__g;								\
	int libh__c;								\
	int libh__t;								\
	libh__g = (g);								\
	libh__c = (cs);								\
	( libh__c <= 1 ) ? 0 : ({						\
		libh__l = (lv);							\
		( libh__l == 0.0 ) ? 0 : ({					\
			libh__t=(int)((exp(log(libh__l)/libh__g)*((double)(libh__c-1)))+0.5);	\
			( libh__t < 0 ) ? 0 : ( libh__t >= libh__c ) ? libh__c - 1 : libh__t;	\
		});								\
	});									\
})

//-----------------------------------------------------------------------------
// macro	pix_linear_scaled_to_code
//
// purpose	Convert a scaled floating point (double) pixel intensity value
//		in the range 0.0 .. S to a gamma corrected integer (int) pixel
//		intensity value in the range 0 .. N-1.
//
// arguments	1 (double) gamma correction factor to use
//		2 (double) linear pixel intensity from 0.0 to S
//		3 (double) linear scale S
//		4 (int) coded integer scale N
//
// returns	(int) pixel intensity in the range 0 to N-1
//-----------------------------------------------------------------------------
#define pix_linear_scaled_to_code(g,lv,ls,cs) ({				\
	double libh__l;								\
	double libh__g;								\
	int libh__c;								\
	int libh__t;								\
	libh__g = (g);								\
	libh__c = (cs);								\
	( libh__c <= 1 ) ? 0 : ({						\
		libh__l = (lv)/(ls);						\
		( libh__l == 0.0 ) ? 0 : ({					\
			libh__t=(int)((exp(log(libh__l)/libh__g)*((double)(libh__c-1)))+0.5);	\
			( libh__t < 0 ) ? 0 : ( libh__t >= libh__c ) ? libh__c - 1 : libh__t;	\
		});								\
	});									\
})

__FMACRO_END__
