//-----------------------------------------------------------------------------
// Copyright  2008 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/arith
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	rgb_to_hsv
//
// purpose	Convert a pixel color value in RGB (red, green, blue) to HSV
//		(hue, saturation, value)
//
// arguments	1 ([long] double) red
//		2 ([long] double) green
//		3 ([long] double) blue
//		4 (lvalue: [lomg] double) result hue
//		5 (lvalue: [long] double) result saturation
//		6 (lvalue: [long] double) result value
//
// yields	results stored in lvalue arguments 4, 5, 6
//-----------------------------------------------------------------------------
#define rgb_to_hsv(r,g,b,h,s,v) ({						\
	__typeof__((r)) libh__r;						\
	__typeof__((g)) libh__g;						\
	__typeof__((b)) libh__b;						\
	double libh__y;								\
	double libh__x;								\
	double libh__d;								\
	double libh__l;								\
	libh__r = (r);								\
	libh__g = (g);								\
	libh__b = (b);								\
	libh__y = libh__r;							\
	if ( libh__y > libh__g ) libh__y = libh__g;				\
	if ( libh__y > libh__b ) libh__y = libh__b;				\
	libh__x = libh__r;							\
	if ( libh__x < libh__g ) libh__x = libh__g;				\
	if ( libh__x < libh__b ) libh__x = libh__b;				\
	(v) = libh__x;								\
	libh__d = libh__x - libh__y;						\
	if ( libh__x == 0.0 ) {							\
		(s) = 0.0;							\
		(h) = 0.0;							\
	} else {								\
		__typeof__((r)) libh__r2;					\
		__typeof__((g)) libh__g2;					\
		__typeof__((b)) libh__b2;					\
		__typeof__((h)) libh__h;					\
		(s) = libh__d / libh__x;					\
		libh__d *= 6.0;							\
		libh__r2 = ( libh__x - libh__r ) / libh__d + 0.5;		\
		libh__g2 = ( libh__x - libh__g ) / libh__d + 0.5;		\
		libh__b2 = ( libh__x - libh__b ) / libh__d + 0.5;		\
		libh__h = ( libh__r == libh__x )				\
			? ( libh__b2 - libh__g2 )				\
			: ( libh__g == libh__x )				\
			? ( (1.0/3.0) + libh__r2 - libh__b2 )			\
			: ( (2.0/3.0) + libh__g2 - libh__r2 );			\
		if ( libh__h < 0.0 ) libh__h += 1.0;				\
		if ( libh__h > 1.0 ) libh__h -= 1.0;				\
		(h) = libh__h;							\
	}									\
	0;									\
})

__FMACRO_END__
