//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/callsign
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include <ctype.h>
#include <stdio.h>
#include <string.h>

#include "callsign_lib.h"

//-----------------------------------------------------------------------------
// data		country_call_array
//
// purpose	Map a radio callsign prefix assigned to a country by the ITU
//		to the country code assigned to that country by the ISO.  This
//		table can also be used to find all callsign prefixes assigned
//		to a country by linear search.
//
// elements	1 (char *) country code or "-" for unassigned
//		2 (char *) first prefix in callsign range
//		3 (char *) last prefix in callsign range
//
// note		Table is sorted by first callsign prefix.
//-----------------------------------------------------------------------------
#define COUNTRY_CALL_COUNT (((sizeof country_call_array)/(3*sizeof(char*)))-1)

static struct country_call	country_call_array	[] = {
	{	"-",	"0",	"1"	},
	{	"uk",	"2",	"2"	},
	{	"-",	"30",	"39"	},
	{	"mc",	"3a",	"3a"	},
	{	"mu",	"3b",	"3b"	},
	{	"gq",	"3c",	"3c"	},
	{	"sz",	"3da",	"3dm"	},
	{	"fj",	"3dn",	"3dz"	},
	{	"pa",	"3e",	"3f"	},
	{	"cl",	"3g",	"3g"	},
	{	"cn",	"3h",	"3u"	},
	{	"tn",	"3v",	"3v"	},
	{	"vn",	"3w",	"3w"	},
	{	"gn",	"3x",	"3x"	},
	{	"no",	"3y",	"3y"	},
	{	"pl",	"3z",	"3z"	},
	{	"-",	"40",	"49"	},
	{	"mx",	"4a",	"4c"	},
	{	"ph",	"4d",	"4i"	},
	{	"az",	"4j",	"4k"	},
	{	"ge",	"4l",	"4l"	},
	{	"ve",	"4m",	"4m"	},
	{	"yu",	"4n",	"4o"	},
	{	"lk",	"4p",	"4s"	},
	{	"pe",	"4t",	"4t"	},
	{	"un",	"4u",	"4u"	},
	{	"ht",	"4v",	"4v"	},
	{	"un",	"4w",	"4w"	},
	{	"il",	"4x",	"4x"	},
	{	"icao",	"4y",	"4y"	},
	{	"il",	"4z",	"4z"	},
	{	"-",	"50",	"59"	},
	{	"ly",	"5a",	"5a"	},
	{	"cy",	"5b",	"5b"	},
	{	"ma",	"5c",	"5g"	},
	{	"tz",	"5h",	"5i"	},
	{	"co",	"5j",	"5k"	},
	{	"lr",	"5l",	"5m"	},
	{	"ng",	"5n",	"5o"	},
	{	"dk",	"5p",	"5q"	},
	{	"mg",	"5r",	"5s"	},
	{	"mr",	"5t",	"5t"	},
	{	"ne",	"5u",	"5u"	},
	{	"tg",	"5v",	"5v"	},
	{	"ws",	"5w",	"5w"	},
	{	"ug",	"5x",	"5x"	},
	{	"ke",	"5y",	"5z"	},
	{	"-",	"60",	"69"	},
	{	"eg",	"6a",	"6b"	},
	{	"sy",	"6c",	"6c"	},
	{	"mx",	"6d",	"6j"	},
	{	"kr",	"6k",	"6n"	},
	{	"so",	"6o",	"6o"	},
	{	"pk",	"6p",	"6s"	},
	{	"sd",	"6t",	"6u"	},
	{	"sn",	"6v",	"6w"	},
	{	"mg",	"6x",	"6x"	},
	{	"jm",	"6y",	"6y"	},
	{	"lr",	"6z",	"6z"	},
	{	"-",	"70",	"79"	},
	{	"id",	"7a",	"7i"	},
	{	"jp",	"7j",	"7n"	},
	{	"ye",	"7o",	"7o"	},
	{	"ls",	"7p",	"7p"	},
	{	"mw",	"7q",	"7q"	},
	{	"dz",	"7r",	"7r"	},
	{	"se",	"7s",	"7s"	},
	{	"dz",	"7t",	"7y"	},
	{	"sa",	"7z",	"7z"	},
	{	"-",	"80",	"89"	},
	{	"id",	"8a",	"8i"	},
	{	"jp",	"8j",	"8n"	},
	{	"bw",	"8o",	"8o"	},
	{	"bb",	"8p",	"8p"	},
	{	"mv",	"8q",	"8q"	},
	{	"gy",	"8r",	"8r"	},
	{	"se",	"8s",	"8s"	},
	{	"in",	"8t",	"8y"	},
	{	"sa",	"8z",	"8z"	},
	{	"-",	"90",	"99"	},
	{	"hr",	"9a",	"9a"	},
	{	"ir",	"9b",	"9d"	},
	{	"et",	"9e",	"9f"	},
	{	"gh",	"9g",	"9g"	},
	{	"mt",	"9h",	"9h"	},
	{	"zm",	"9i",	"9j"	},
	{	"kw",	"9k",	"9k"	},
	{	"sl",	"9l",	"9l"	},
	{	"my",	"9m",	"9m"	},
	{	"np",	"9n",	"9n"	},
	{	"cd",	"9o",	"9t"	},
	{	"bi",	"9u",	"9u"	},
	{	"sg",	"9v",	"9v"	},
	{	"my",	"9w",	"9w"	},
	{	"rw",	"9x",	"9x"	},
	{	"tt",	"9y",	"9z"	},
	{	"-",	"a0",	"a1"	},
	{	"bw",	"a2",	"a2"	},
	{	"to",	"a3",	"a3"	},
	{	"om",	"a4",	"a4"	},
	{	"bt",	"a5",	"a5"	},
	{	"ae",	"a6",	"a6"	},
	{	"qa",	"a7",	"a7"	},
	{	"lr",	"a8",	"a8"	},
	{	"bh",	"a9",	"a9"	},
	{	"us",	"aa",	"al"	},
	{	"es",	"am",	"ao"	},
	{	"pk",	"ap",	"as"	},
	{	"in",	"at",	"aw"	},
	{	"au",	"ax",	"ax"	},
	{	"ar",	"ay",	"az"	},
	{	"cn",	"b",	"b"	},
	{	"-",	"c0",	"c1"	},
	{	"nr",	"c2",	"c2"	},
	{	"ad",	"c3",	"c3"	},
	{	"cy",	"c4",	"c4"	},
	{	"gm",	"c5",	"c5"	},
	{	"bs",	"c6",	"c6"	},
	{	"wmo",	"c7",	"c7"	},
	{	"mz",	"c8",	"c9"	},
	{	"cl",	"ca",	"ce"	},
	{	"ca",	"cf",	"ck"	},
	{	"cu",	"cl",	"cm"	},
	{	"ma",	"cn",	"cn"	},
	{	"cu",	"co",	"co"	},
	{	"bo",	"cp",	"cp"	},
	{	"pt",	"cq",	"cu"	},
	{	"uy",	"cv",	"cx"	},
	{	"ca",	"cy",	"cz"	},
	{	"-",	"d0",	"d1"	},
	{	"ao",	"d2",	"d3"	},
	{	"cv",	"d4",	"d4"	},
	{	"lr",	"d5",	"d5"	},
	{	"km",	"d6",	"d6"	},
	{	"kr",	"d7",	"d9"	},
	{	"de",	"da",	"dr"	},
	{	"kr",	"ds",	"dt"	},
	{	"ph",	"du",	"dz"	},
	{	"-",	"e0",	"e1"	},
	{	"th",	"e2",	"e2"	},
	{	"er",	"e3",	"e3"	},
	{	"ps",	"e4",	"e4"	},
	{	"-",	"e5",	"e9"	},
	{	"es",	"ea",	"eh"	},
	{	"ie",	"ei",	"ej"	},
	{	"am",	"ek",	"ek"	},
	{	"lr",	"el",	"el"	},
	{	"ua",	"em",	"eo"	},
	{	"ir",	"ep",	"eq"	},
	{	"md",	"er",	"er"	},
	{	"ee",	"es",	"es"	},
	{	"et",	"et",	"et"	},
	{	"by",	"eu",	"ew"	},
	{	"kg",	"ex",	"ex"	},
	{	"tj",	"ey",	"ey"	},
	{	"tm",	"ez",	"ez"	},
	{	"fr",	"f",	"f"	},
	{	"uk",	"g",	"g"	},
	{	"-",	"h0",	"h1"	},
	{	"cy",	"h2",	"h2"	},
	{	"pa",	"h3",	"h3"	},
	{	"sb",	"h4",	"h4"	},
	{	"-",	"h5",	"h5"	},
	{	"ni",	"h6",	"h7"	},
	{	"pa",	"h8",	"h9"	},
	{	"hu",	"ha",	"ha"	},
	{	"ch",	"hb",	"hb"	},
	{	"ec",	"hc",	"hd"	},
	{	"ch",	"he",	"he"	},
	{	"pl",	"hf",	"hf"	},
	{	"hu",	"hg",	"hg"	},
	{	"ht",	"hh",	"hh"	},
	{	"do",	"hi",	"hi"	},
	{	"co",	"hj",	"hk"	},
	{	"kr",	"hl",	"hl"	},
	{	"kp",	"hm",	"hm"	},
	{	"iq",	"hn",	"hn"	},
	{	"pa",	"ho",	"hp"	},
	{	"hn",	"hq",	"hr"	},
	{	"th",	"hs",	"hs"	},
	{	"ni",	"ht",	"ht"	},
	{	"sv",	"hu",	"hu"	},
	{	"va",	"hv",	"hv"	},
	{	"fr",	"hw",	"hy"	},
	{	"sa",	"hz",	"hz"	},
	{	"it",	"i",	"i"	},
	{	"-",	"j0",	"j1"	},
	{	"dj",	"j2",	"j2"	},
	{	"gd",	"j3",	"j3"	},
	{	"gr",	"j4",	"j4"	},
	{	"gw",	"j5",	"j5"	},
	{	"lc",	"j6",	"j6"	},
	{	"dm",	"j7",	"j7"	},
	{	"vc",	"j8",	"j8"	},
	{	"-",	"j9",	"j9"	},
	{	"jp",	"ja",	"js"	},
	{	"mn",	"jt",	"jv"	},
	{	"no",	"jw",	"jx"	},
	{	"jo",	"jy",	"jy"	},
	{	"id",	"jz",	"jz"	},
	{	"us",	"k",	"k"	},
	{	"-",	"l0",	"l1"	},
	{	"ar",	"l2",	"l9"	},
	{	"no",	"la",	"ln"	},
	{	"ar",	"lo",	"lw"	},
	{	"lu",	"lx",	"lx"	},
	{	"lt",	"ly",	"ly"	},
	{	"bg",	"lz",	"lz"	},
	{	"uk",	"m",	"m"	},
	{	"us",	"n",	"n"	},
	{	"-",	"o0",	"o9"	},
	{	"pe",	"oa",	"oc"	},
	{	"lb",	"od",	"od"	},
	{	"at",	"oe",	"oe"	},
	{	"fi",	"of",	"oj"	},
	{	"cz",	"ok",	"ol"	},
	{	"sk",	"om",	"om"	},
	{	"be",	"on",	"ot"	},
	{	"dk",	"ou",	"oz"	},
	{	"-",	"p0",	"p1"	},
	{	"pg",	"p2",	"p2"	},
	{	"cy",	"p3",	"p3"	},
	{	"aw",	"p4",	"p4"	},
	{	"kp",	"p5",	"p9"	},
	{	"nl",	"pa",	"pi"	},
	{	"an",	"pj",	"pj"	},
	{	"id",	"pk",	"po"	},
	{	"br",	"pp",	"py"	},
	{	"sr",	"pz",	"pz"	},
	{	"-",	"q",	"q"	},
	{	"ru",	"r",	"r"	},
	{	"bd",	"s2",	"s3"	},
	{	"-",	"s4",	"s4"	},
	{	"si",	"s5",	"s5"	},
	{	"sg",	"s6",	"s6"	},
	{	"sc",	"s7",	"s7"	},
	{	"za",	"s8",	"s8"	},
	{	"st",	"s9",	"s9"	},
	{	"se",	"sa",	"sm"	},
	{	"pl",	"sn",	"sr"	},
	{	"eg",	"ssa",	"ssm"	},
	{	"sd",	"ssn",	"stz"	},
	{	"eg",	"su",	"su"	},
	{	"gr",	"sv",	"sz"	},
	{	"-",	"t0",	"t1"	},
	{	"tv",	"t2",	"t2"	},
	{	"ki",	"t3",	"t3"	},
	{	"cu",	"t4",	"t4"	},
	{	"so",	"t5",	"t5"	},
	{	"af",	"t6",	"t6"	},
	{	"sm",	"t7",	"t7"	},
	{	"pw",	"t8",	"t8"	},
	{	"ba",	"t9",	"t9"	},
	{	"tr",	"ta",	"tc"	},
	{	"gt",	"td",	"td"	},
	{	"cr",	"te",	"te"	},
	{	"is",	"tf",	"tf"	},
	{	"gt",	"tg",	"tg"	},
	{	"fr",	"th",	"th"	},
	{	"cr",	"ti",	"ti"	},
	{	"cm",	"tj",	"tj"	},
	{	"fr",	"tk",	"tk"	},
	{	"cf",	"tl",	"tl"	},
	{	"fr",	"tm",	"tm"	},
	{	"cg",	"tn",	"tn"	},
	{	"fr",	"to",	"tq"	},
	{	"ga",	"tr",	"tr"	},
	{	"tn",	"ts",	"ts"	},
	{	"td",	"tt",	"tt"	},
	{	"ci",	"tu",	"tu"	},
	{	"fr",	"tv",	"tx"	},
	{	"bj",	"ty",	"ty"	},
	{	"ml",	"tz",	"tz"	},
	{	"ru",	"u0",	"ui"	},
	{	"uz",	"uj",	"um"	},
	{	"kz",	"un",	"uq"	},
	{	"ua",	"ur",	"uz"	},
	{	"-",	"v0",	"v1"	},
	{	"ag",	"v2",	"v2"	},
	{	"bz",	"v3",	"v3"	},
	{	"kn",	"v4",	"v4"	},
	{	"na",	"v5",	"v5"	},
	{	"fm",	"v6",	"v6"	},
	{	"mh",	"v7",	"v7"	},
	{	"bn",	"v8",	"v8"	},
	{	"-",	"v9",	"v9"	},
	{	"ca",	"va",	"vg"	},
	{	"au",	"vh",	"vn"	},
	{	"ca",	"vo",	"vo"	},
	{	"uk",	"vp",	"vq"	},
	{	"hk",	"vr",	"vr"	},
	{	"uk",	"vs",	"vs"	},
	{	"in",	"vt",	"vw"	},
	{	"ca",	"vx",	"vy"	},
	{	"au",	"vz",	"vz"	},
	{	"us",	"w",	"w"	},
	{	"-",	"x0",	"x9"	},
	{	"mx",	"xa",	"xi"	},
	{	"ca",	"xj",	"xo"	},
	{	"dk",	"xp",	"xp"	},
	{	"cl",	"xq",	"xr"	},
	{	"cn",	"xs",	"xs"	},
	{	"bf",	"xt",	"xt"	},
	{	"kh",	"xu",	"xu"	},
	{	"vn",	"xv",	"xv"	},
	{	"la",	"xw",	"xw"	},
	{	"pt",	"xx",	"xx"	},
	{	"mm",	"xy",	"xz"	},
	{	"-",	"y0",	"y1"	},
	{	"de",	"y2",	"y9"	},
	{	"af",	"ya",	"ya"	},
	{	"id",	"yb",	"yh"	},
	{	"iq",	"yi",	"yi"	},
	{	"vu",	"yj",	"yj"	},
	{	"sy",	"yk",	"yk"	},
	{	"lv",	"yl",	"yl"	},
	{	"tr",	"ym",	"ym"	},
	{	"ni",	"yn",	"yn"	},
	{	"ro",	"yo",	"yr"	},
	{	"sv",	"ys",	"ys"	},
	{	"yu",	"yt",	"yu"	},
	{	"ve",	"yv",	"yy"	},
	{	"yu",	"yz",	"yz"	},
	{	"-",	"z0",	"z1"	},
	{	"zw",	"z2",	"z2"	},
	{	"mk",	"z3",	"z3"	},
	{	"-",	"z4",	"z9"	},
	{	"al",	"za",	"za"	},
	{	"uk",	"zb",	"zj"	},
	{	"nz",	"zk",	"zm"	},
	{	"uk",	"zn",	"zo"	},
	{	"py",	"zp",	"zp"	},
	{	"uk",	"zq",	"zq"	},
	{	"za",	"zr",	"zu"	},
	{	"br",	"zv",	"zz"	},
	{	NULL,	NULL,	NULL	}
};

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	country_call_table
//
// purpose	Return a pointer to the callsign country table.
//
// arguments	-none-
//
// returns	(const struct country_call *) pointer to the callsign country
//		table
//
// note		A future version of this function will read the callsign to
//		country mapping table in from a shared resource file instead
//		of having it hard coded here.
//-----------------------------------------------------------------------------
const struct country_call *
country_call_table ()
__PROTO_END__
{
    return country_call_array;
}

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	country_call_table_count
//
// purpose	Return a count of the number of entries in the callsign
//		country table.
//
// arguments	-none-
//
// returns	(int) count of the number of entries in the table
//-----------------------------------------------------------------------------
int
country_call_table_count ()
__PROTO_END__
{
    return ( ( sizeof country_call_array ) / ( 3 * sizeof (char *) ) ) - 1;
}

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	country_by_call
//
// purpose	Given a radio callsign, return the country code and callsign
//		range it fits in.
//
// argument	1 (const char *) callsign string
//
// returns	(const struct country_call *) NULL : error, range not found
//		(const struct country_call *) pointer to country call entry
//
// note		Do not modify the referenced strings (const char *).
//-----------------------------------------------------------------------------
const struct country_call *
country_by_call (
    const char *	arg_call_p
    )
__PROTO_END__
{
    const struct country_call *		table_p		;

    for ( table_p = country_call_table(); table_p->code; ++ table_p ) {
	{
	    const char *	ap	;
	    const char *	tp	;
	    unsigned int	ac	;
	    unsigned int	tc	;

	    ap = arg_call_p;
	    tp = table_p->call_first;
	    for (;;) {
		ac = * ap;
		tc = * tp;
		if ( ! ac || ! tc ) break;
		ac = tolower( ac );
		if ( ac != tc ) break;
		++ tp;
		++ ap;
	    }
	    if ( tc ) {
		if ( ! ac ) continue;
		if ( ac < tc ) continue;
	    }
	}
	{
	    const char *	ap	;
	    const char *	tp	;
	    unsigned int	ac	;
	    unsigned int	tc	;

	    ap = arg_call_p;
	    tp = table_p->call_last;
	    for (;;) {
		ac = * ap;
		tc = * tp;
		if ( ! ac || ! tc ) break;
		ac = tolower( ac );
		if ( ac != tc ) break;
		++ tp;
		++ ap;
	    }
	    if ( tc ) {
		if ( ! ac ) continue;
		if ( ac > tc ) continue;
	    }
	}
	return table_p;
    }
    return NULL;
}
