//-----------------------------------------------------------------------------
// Copyright  2006 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/ctools
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "ctools_lib.h"

#include <stdarg.h>
#include <stdlib.h>

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	env_str_dupa
//
// purpose	Select the first defined environment variable from a given list
//		and return a pointer to an allocated copy of the value using
//		the alloca() function in the context of the function that uses
//		this macro.
//
// arguments	1 (const char *) default value
//		2..N (const char *) list of variable names, ending in NULL
//
// returns	(char *) pointer to duplicated value string
//
// WARNING	All the restrictions for alloca() apply to env_str_dupa().
//-----------------------------------------------------------------------------
#define env_str_dupa(d,n...) ({							\
	const char * libh__str;							\
	char * libh__new;							\
	size_t libh__len;							\
	libh__str = ( env_str( n , ( (const char*) (NULL) ) ) );		\
	if ( ! libh__str || ! * libh__str ) libh__str = ( d );			\
	libh__len = strlen( libh__str ) + 1;					\
	libh__new = alloca( libh__len );					\
	memcpy( libh__new, libh__str, libh__len );				\
	libh__new;								\
})

__FMACRO_END__
