//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/ctools
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "ctools_lib.h"

#include <stdarg.h>

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	pick_ptr
//
// purpose	Pick a pointer from the given arguments based on the value of
//		the first argument.  The first choice from argument 2 is
//		chosen with an index value of 0.
//
// arguments	1 (int) choice index value
//		2..N (void *) values to choose one to be returned
//
// returns	(void *) the chosen value
//
// warning	An index value that has no corresponding value will have an
//		undefined result and probably disasterous consequences.
//		THIS CONDITION CANNOT BE PROPERLY TESTED FOR.
//-----------------------------------------------------------------------------
#define pick_char_p(i,a...) ((char *)pick_ptr((i),a))
#define pick_schar_p(i,a...) ((signed char *)pick_ptr((i),a))
#define pick_uchar_p(i,a...) ((unsigned char *)pick_ptr((i),a))
#define pick_short_p(i,a...) ((short *)pick_ptr((i),a))
#define pick_ushort_p(i,a...) ((unsigned short *)pick_ptr((i),a))
#define pick_int_p(i,a...) ((int *)pick_ptr((i),a))
#define pick_uint_p(i,a...) ((unsigned int *)pick_ptr((i),a))
#define pick_long_p(i,a...) ((long *)pick_ptr((i),a))
#define pick_ulong_p(i,a...) ((unsigned long *)pick_ptr((i),a))
#define pick_longlong_p(i,a...) ((long long *)pick_ptr((i),a))
#define pick_ulonglong_p(i,a...) ((unsigned long long *)pick_ptr((i),a))
#define pick_float_p(i,a...) ((float *)pick_ptr((i),a))
#define pick_double_p(i,a...) ((double *)pick_ptr((i),a))
#define pick_long_double_p(i,a...) ((long double *)pick_ptr((i),a))
#define pick_void_p(i,a...) ((void *)pick_ptr((i),a))

void *
pick_ptr (
    int			arg_choice
    ,
    ...
    )
__PROTO_END__
{
    va_list		var_args	;
    void *		value		;

    va_start( var_args, arg_choice );
    do { value = va_arg( var_args, void * ); } while ( -- arg_choice >= 0 );
    va_end( var_args );
    return value;
}
