//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/ctools
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "ctools_lib.h"

#include <stdarg.h>

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	pick_ull
//
// purpose	Pick an unsigned long long from the given arguments based on
//		the value of the first argument.  The first choice from
//		argument 2 is chosen with an index value of 0.
//
// arguments	1 (int) choice index value
//		2..N (unsigned long long) values to choose one to be returned
//
// returns	(unsigned long long) the chosen value
//
// warning	An index value that has no corresponding value will have an
//		undefined result and probably disasterous consequences.
//		THIS CONDITION CANNOT BE PROPERLY TESTED FOR.
//-----------------------------------------------------------------------------
unsigned long long
pick_ull (
    int			arg_choice
    ,
    ...
    )
__PROTO_END__
{
    va_list		var_args	;
    unsigned long long	value		;

    va_start( var_args, arg_choice );
    do { value = va_arg( var_args, unsigned long long ); } while ( -- arg_choice >= 0 );
    va_end( var_args );
    return value;
}
