//-----------------------------------------------------------------------------
// Copyright  2006 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/ftr
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#define _GNU_SOURCE

#include "ftr_lib.h"

#include <fcntl.h>
#include <limits.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	ftr_end
//
// purpose	End an instance of file tree recursion.  This also sets the
//		process current working directory to the same as when the
//		ftr_new() function was called.
//
// arguments	1 (FTR) reference to which file tree recursion.
//
// returns	(int)  < 0 : error
//		(int) == 0 : success
//-----------------------------------------------------------------------------
int
ftr_end (
    FTR			arg_ftr
    )
__PROTO_END__
{
    FTR			this_ftr	;

    struct ftr_dir *	this_dir	;
    struct ftr_dir *	next_dir	;


    if ( ! arg_ftr ) return -1;

    //------------------------------
    // Loop through all queued FTRs.
    //------------------------------
    while ( arg_ftr ) {

	this_ftr = arg_ftr;
	arg_ftr = this_ftr->next_ftr;
	this_ftr->next_ftr = NULL;

	//---------------------------------------------------
	// Close all directory file descriptors being held.
	//---------------------------------------------------
	if ( this_ftr->rdir_full != -1 ) {
	    close( this_ftr->rdir_full );
	    this_ftr->rdir_full = -1;
	}
	if ( this_ftr->rdir_part != -1 ) {
	    close( this_ftr->rdir_part );
	    this_ftr->rdir_part = -1;
	}
	if ( this_ftr->rdir_last != -1 ) {
	    close( this_ftr->rdir_last );
	    this_ftr->rdir_last = -1;
	}

	//---------------------
	// Free the name space.
	//---------------------
	if ( this_ftr->name_root ) {
	    free( this_ftr->name_root );
	    this_ftr->name_root = NULL;
	    this_ftr->name_full = NULL;
	    this_ftr->name_part = NULL;
	    this_ftr->name_last = NULL;
	    this_ftr->name_end  = NULL;
	}

	//-------------------------------------------
	// Free any leftover directory descent stack.
	//-------------------------------------------
	this_dir = this_ftr->curr_dir;
	while ( this_dir ) {
	    close( this_dir->save_rdir_last );
	    avl_empty( & this_dir->name_tree, free );
	    next_dir = this_dir->parent_dir;
	    free( this_dir );
	    this_dir = next_dir;
	}
	this_ftr->curr_dir = NULL;

	//------------------------
	// Free the struct itself.
	//------------------------
	free( this_ftr );
    }

    return 0;
}
