//-----------------------------------------------------------------------------
// Copyright  2006 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/helloworld
// homepage	http://phil.ipal.org/freeware/libh/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libh/ftr.h>
#include <libh/string.h>

//-----------------------------------------------------------------------------
// program	ftr-deep
//
// purpose	Show increasingly deeper file system objects.
//
// syntax	ftr-deep  [ filenames ... ]
//
// returns	(int) >= 0 : success
//		(int) == EOF : error
//-----------------------------------------------------------------------------
int
main (
    int		argc
    ,
    char * *	argv
    ,
    char * *	envp
    )
{
    unsigned long long	count_objects	;
    unsigned long long	count_returns	;

    unsigned long long	total_objects	;
    unsigned long long	total_returns	;

    FTR		this_ftr	;
    char *	argp		;
    char *	pgm		;
    int		argi		;
    int		ftype		;
    int		deepest		;
    int		total		;


    total_objects = 0;
    total_returns = 0;
    total = 0;
    deepest = -1;
    pgm = str_tail_one( argv[0], '/' );
    argp = ( argi = 1 ) < argc ? argv[ argi ] : NULL;
    do {
	count_objects = 0;
	count_returns = 0;
	if ( ! ( this_ftr = ftr_new( argp ) ) ) {
            if ( argp )	fprintf( stderr, "%s: ftr_new( \"%s\" ) fails: %s\n", pgm, argp, strerror( errno ) );
            else	fprintf( stderr, "%s: ftr_new( NULL ) fails: %s\n", pgm, strerror( errno ) );
	    continue;
	}
	while ( ( ftype = ftr_get( this_ftr ) ) > 0 ) {
	    ++ count_returns;
	    if ( ftype == 'a' ) continue; // skip ascending directories
	    ++ count_objects;
	    if ( ftr_depth_curr( this_ftr ) <= deepest ) continue;
	    deepest = ftr_depth_curr( this_ftr );
	    printf( "%5u %c %s\n", ftr_depth_curr( this_ftr ), ftype, ftr_name_full( this_ftr ) );
	    fflush( stdout );
	}
	printf( "%20llu objects - %20llu returns\n", count_objects, count_returns );
	fflush( stdout );
	ftr_end( this_ftr );
	total_objects += count_objects;
	total_returns += count_returns;
	++ total;
    } while ( ++ argi < argc && ( argp = argv[ argi ] ) );
    if ( total > 1 ) {
	printf( "%20llu objects - %20llu returns - TOTAL\n", total_objects, total_returns );
    }
    return 0;
}
