//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/http
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	phil at ipal dot org
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

#include "http_lib.h"

#include <stdarg.h>

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	http_error_exit
//
// purpose	Handle a fatal error condition and exit the process.
//
// arguments	1 (int) exit status code to use
//		2 (const char *) HTTP error response string
//		3 (const char *) error message text
//
// returns	No return
//-----------------------------------------------------------------------------
#define http_error_exit(e,s,m...) ({(http_error)((s),m,(const char*)(NULL));exit((e));})

__FMACRO_END__

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	http_error__exit
//
// purpose	Handle a fatal error condition and exit the process quickly.
//
// arguments	1 (int) exit status code to use
//		2 (const char *) HTTP error response string
//		3 (const char *) error message text
//
// returns	No return
//-----------------------------------------------------------------------------
#define http_error__exit(e,s,m...) ({(http_error)((s),m,(const char*)(NULL));_exit((e));})

__FMACRO_END__

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	http_error_return
//
// purpose	Handle a fatal error condition and exit the process.
//
// arguments	1 (?) return value
//		2 (const char *) HTTP error response string
//		3 (const char *) error message text
//
// returns	Whatever is provided in argument 1.
//-----------------------------------------------------------------------------
#define http_error_return(r,s,m...) ({(http_error)((s),m,(const char*)(NULL));return (r);})

__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	http_error
//
// purpose	Handle a fatal error condition.
//
// arguments	1 (const char *) HTTP error response string
//		2..N (const char *) error message text
//
// returns	(int) -1 : error (imagine that)
//		(int)  0 : OK
//-----------------------------------------------------------------------------
#define http_error(s,m...) ((http_error)((s),m,(const char*)(NULL)))

int
(http_error) (
    const char *	arg_error_res
    ,
    ...
    )
__PROTO_END__
{
    va_list		var_args	;
    const char *	error_msg	;

    //-- If HTTP is already done, return an error.
    if ( http_done ) return -1;

    //-- Output response code and headers, with disabled cache.
    http_discard();
    http_response_str( arg_error_res );
    http_no_cache();
    http_header( 0, "Content-Type", "text/html" );
    http_commit_put();

    //-- Output a default simple basic error message page.
    fputs( "<html>\n<head>\n", stdout );
    if ( arg_error_res ) printf( "<title>%s</title>\n", arg_error_res );
    fputs( "</head>\n", stdout );
    fputs( "<body bgcolor=white text=black>\n<h1>Error: ", stdout );

    //-- Output variable argument message text.
    va_start( var_args, arg_error_res );
    while ( ( error_msg = va_arg( var_args, const char * ) ) ) {
	fputs( error_msg, stdout );
    }
    va_end( var_args );

    //-- Finish message text and flush buffers.
    if ( errno > 0 ) printf( "<h1>system code = %d (%s)</h1>\n", errno, strerror(errno) );
    fputs( "</body>\n</html>\n", stdout );
    fflush( stdout );

    return 0;
}
