//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/io
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "io_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	close_all_above
//
// purpose	Mass close file descriptors above a specified value.
//
// arguments	1 (int) highest file descriptor to not close
//
// returns	(int) number of descriptors closed successfully
//-----------------------------------------------------------------------------
int
close_all_above (
    int		arg_fd
    )
__PROTO_END__
{
    int         num_fd  ;
    int         max_fd  ;

    //------------------------------------------------------------
    // Set a sanity check on the highest possible file descriptor.
    //------------------------------------------------------------
    max_fd = 65535;

#ifdef OPEN_MAX
    if ( max_fd > OPEN_MAX ) max_fd = OPEN_MAX;
#endif

#ifdef _SC_OPEN_MAX
    num_fd = sysconf( _SC_OPEN_MAX );
    if ( max_fd > num_fd ) max_fd = num_fd;
#endif

    //----------------------------------------------------------
    // Close everything from the highest downward to the lowest
    // while counting the number actually closed to be returned.
    //----------------------------------------------------------
    num_fd = 0;
    while ( -- max_fd > arg_fd ) {
	if ( close( max_fd ) == 0 ) ++ num_fd;
    }
    return num_fd;
}
