//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
//-----------------------------------------------------------------------------
// package	libh/io
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include <stdio.h>

#include "io_lib.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	eput_byte
//
// purpose	Output a byte, escaped as needed, to stderr.
//
// arguments	1 (int) point to beginning of a byte
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define eput_byte(b) fput_byte((b),(stderr))

__FMACRO_END__

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	put_byte
//
// purpose	Output a byte, escaped as needed, to stdout.
//
// arguments	1 (int) point to beginning of a byte
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define put_byte(b) fput_byte((b),(stdout))

__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	fput_byte
//
// purpose	Output a byte, escaped as needed, to the specified open file.
//
// arguments	1 (int) point to beginning of a byte
//		2 (FILE *) file to output to
//
// returns	(int) total characters output (extra for escaped)
//-----------------------------------------------------------------------------
int
fput_byte (
    int			arg_byte
    ,
    FILE *		arg_file
    )
    __PROTO_END__
{
    int es;

    es = 0;
    if ( arg_byte == '\\' )	es = '\\';
    else if ( arg_byte == '\a' )	es = 'a';
    else if ( arg_byte == '\b' )	es = 'b';
    else if ( arg_byte == '\f' )	es = 'f';
    else if ( arg_byte == '\n' )	es = 'n';
    else if ( arg_byte == '\r' )	es = 'r';
    else if ( arg_byte == '\t' )	es = 't';
    else if ( arg_byte == '\v' )	es = 'v';
    if ( es ) {
	fputc( '\\', arg_file );
	fputc( es, arg_file );
	return 2;
    }
    if ( 32 <= arg_byte && arg_byte <= 126 ) {
	fputc( arg_byte, arg_file );
	return 1;
    }
    fputc( '\\', arg_file );
    fputc( arg_byte >> 6 & 7, arg_file );
    fputc( arg_byte >> 3 & 7, arg_file );
    fputc( arg_byte & 7, arg_file );
    return 4;
}
