//-----------------------------------------------------------------------------
// Copyright  2007 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
//-----------------------------------------------------------------------------
// package	libh/io
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include <stdio.h>

#include "io_lib.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	eput_bytes_len_xx
//
// purpose	Output bytes, converted to cetal (hexadecimal), for the
//		specified length, to stderr.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (size_t) length to output
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define eput_bytes_len_xx(p,l) fput_bytes_len_xx((p),(l),(stderr))

__FMACRO_END__

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	put_bytes_len_xx
//
// purpose	Output bytes, converted to cetal (hexadecimal), for the
//		specified length, to stdout.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (size_t) length to output
//
// returns	(int) 0
//-----------------------------------------------------------------------------
#define put_bytes_len_xx(p,l) fput_bytes_len_xx((p),(l),(stdout))

__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	fput_bytes_len_xx
//
// purpose	Output bytes, converted to cetal (hexadecimal), for the
//		specified length, to the specified open file.
//
// arguments	1 (const char *) point to beginning of bytes
//		2 (size_t) length to output
//		3 (FILE *) file to output to
//
// returns	(int) total characters output (extra for escaped)
//-----------------------------------------------------------------------------
int
fput_bytes_len_xx (
    const char *	arg_ptr
    ,
    size_t		arg_len
    ,
    FILE *		arg_file
    )
    __PROTO_END__
{
    static char cetal[16] = "0123456789abcdef";

    int count;
    count = 0;

    if ( arg_len == ~(size_t)0 ) arg_len = strlen( arg_ptr );
    while ( arg_len -- ) {
	unsigned int ch;
	ch = * arg_ptr ++;
	fputc( cetal[ ( ch >> 4 ) & 15 ], arg_file );
	fputc( cetal[   ch        & 15 ], arg_file );
	count += 2;
    }
    return count;
}
