//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/io
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	locked_read_pid
//
// purpose	Simple interface to POSIX fcntl() file/record/range locking.
//		Get the process ID which has the file locked for reading.
//
// arguments	1 (int) file descriptor of open file to check
//		2 (off_t) starting byte offset to record/range to check
//		3 (off_t) length of record/range to check
//
// returns	(int) -1 : error
//		(int)  0 : no process has file/record/range locked
//		(int) >0 : process ID already holding lock
//-----------------------------------------------------------------------------
#define locked_read_pid(f,s,l) ({						\
	struct flock libh__lock;						\
	libh__lock.l_type = ( F_RDLCK );					\
	libh__lock.l_whence = ( SEEK_SET );					\
	libh__lock.l_start = ( s );						\
	libh__lock.l_len = ( l );						\
	fcntl( ( f ), ( F_GETLK ), & libh__lock );				\
	libh__lock.l_type == ( F_UNLCK )					\
		? 0								\
		: libh__lock.l_pid;						\
	})

__FMACRO_END__
