//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/io
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include <unistd.h>

#include "io_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	read_block
//
// purpose	Read a full block of data, repeating until done.
//
// arguments	1 (int) file descriptor
//		2 (void *) data pointer
//		3 (size_t) data length
//
// returns	(ssize_t)  < 0 : error, see errno
//		(ssize_t) == 0 : end of file
//		(ssize_t)  > 0 : bytes read
//-----------------------------------------------------------------------------
ssize_t
read_block (
    int			arg_fd
    ,
    void *		arg_ptr
    ,
    size_t		arg_len
    )
__PROTO_END__
{
    void *		ptr	;
    size_t		len	;
    ssize_t		result	;

    ptr = arg_ptr;
    len = arg_len;
    if ( len == 0 ) return 0;

    do {
	result = read( arg_fd, ptr, len );
	if ( result <= 0 ) break;
	ptr += result;
	len -= result;
    } while ( len );

    return ptr > arg_ptr ? ptr - arg_ptr : result;
}
