//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/io
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		stderr_to_unique_file.c
//
// purpose	Source for related functions
//-----------------------------------------------------------------------------
#include "io_lib.h"


__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	stderr_to_unique_file
//
// purpose	Change the C standard stderr stream to write to a uniquely
//		created file according to the given pattern.
//
// arguments	1 (const char *) filename format pattern
//		2 (int) open flags
//		3 (int) open mode
//
// returns	(int) -1 : error
//		(int)  0 : OK
//-----------------------------------------------------------------------------
int
stderr_to_unique_file (
    const char *	arg_pattern
    ,
    int			arg_flags
    ,
    int			arg_mode
    )
__PROTO_END__
{
    int		fd	;

    if ( ( fd = open_unique( arg_pattern, arg_flags, arg_mode ) ) < 0 ) {
	return -1;
    }
    if ( fd != 2 ) {
	fflush( stderr );
	close( 2 );
	dup2( fd, 2 );
    }
    return 0;
}
