//-----------------------------------------------------------------------------
// Copyright  2012 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/keepstr
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "keepstr_lib.h"
#include "libh/keepstr.h"

#include <stdarg.h>
#include <stdio.h>

__PROTO_BEGIN__
//---------------------------------------------------------------------------------------------------
// function	keepstr_merge
//
// purpose	Merge a set of strings into one string, and look up the merged string in the pool.
//		Add it to the common shared string pool of absent.
//		Return a pointer to the merged string in the pool.
//
// note		The caller no longer needs to use the original string used for the search.
//
// arguments	1-N (const char *) string arguments
//
// returns	(const char *) == NULL : error (probably out of memory)
//		(const char *) != NULL : pointer to new option scanning object
//---------------------------------------------------------------------------------------------------
#define keepstr_merge(a...) ((keepstr_merge)(a,NULL))
const char *
(keepstr_merge) (
    const char *	arg_str
    ,
    ...
    )
__PROTO_END__
{
    va_list		var_arg	;
    const char *	src_p	;
    char *		dst_p	;
    size_t		len	;

    //-- Figure out how much temporary space is needed for the merged string.
    len = 0;
    va_start( var_arg, arg_str );
    for ( src_p = arg_str; src_p; ( src_p = va_arg( var_arg, const char * ) ) ) {
	len += strlen( src_p );
    }
    va_end( var_arg );

    //-- Allocate temporary space for the merged string (not terminated).
    dst_p = alloca( len );
    if ( ! dst_p ) return NULL;

    //-- Fill in the merged string contents (not terminated).
    va_start( var_arg, arg_str );
    for( src_p= arg_str; src_p; ( src_p = va_arg( var_arg, const char * ) ) ) {
	while ( * src_p ) * dst_p ++ = * src_p ++;
    }
    va_end( var_arg );

    //-- Lookup the merged string in the keepstr pool.
    return keepstr_pl( dst_p, len );
}
