//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/arith
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libh/io.h>
#include <libh/list.h>


//-----------------------------------------------------------------------------
// program	reverse
//
// purpose	Test some list macros with a program that reads in lines,
//		stores them in a linked list, and outputs them in reverse
//		order
//
// syntax	reverse < data
//-----------------------------------------------------------------------------
int
main (
    int		argc
    ,
    char * *	argv
    ,
    char * *	envp
    )
{
    struct line_node {
	struct line_node *	next		; // used by list macros
	struct line_node *	prev		; // used by list macros
	char			line	[1]	; // dynamically expanded
    };

    struct line_node *		line_list	; // list anchor
    struct line_node *		this_node	; // node pointer

    size_t		line_len		;
    char		line_buffer	[4096]	;


    list_init( line_list );

    //-------------------------
    // Collect lines into list.
    //-------------------------
    while ( get_line_file( line_buffer, sizeof (line_buffer), NULL ) >= 0 ) {
	line_len = strlen( line_buffer );
	this_node = malloc( sizeof (struct line_node) + line_len );
	if ( ! this_node ) {
	    fprintf( stderr, "Out of memory\n" );
	    return 1;
	}
	memcpy( this_node->line, line_buffer, line_len + 1 );
	list_insert_head( line_list, this_node );
    }

    //------------------------
    // Dump lines out of list.
    //------------------------
    list_flush( line_list, one_line ) {
	puts( one_line->line );
	free( one_line );
    }

    //-------------------------
    // Make sure list is empty.
    //-------------------------
    if ( ! list_is_empty( line_list ) ) {
	fprintf( stderr, "List is not empty\n" );
	return 1;
    }

    return 0;
}
