#!/usr/bin/env bash

#-----------------------------------------------------------------------------
# Return true if the first word matches any one of the following words.
#-----------------------------------------------------------------------------
function oneof {
    local a
    local s
    s="${1}"
    shift
    for a in "$@"; do
        if [[ "${a}" = "${s}" ]]; then
            return 0
        fi
    done
    return 1;
}

##############################################################################
function banner {
    cat <<EOF
//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

EOF
}

##############################################################################
function prefix {
    banner
    cat <<EOF
#include "map_${typearr}.h"

EOF
}

##############################################################################
function map_header {
    banner

    cat <<EOF
#include "map_lib.h"

EOF

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
//-----------------------------------------------------------------------------
// struct       map_link_${typearr}
// struct       map_node_${typearr}
//
// purpose      Define structures needed to keep the AVL trees used to hold
//              a mapping of this particular key type.
//
// note		The key array size is determined when node is created.
//-----------------------------------------------------------------------------
struct map_link_${typearr} {
    avl_link		link    ;
    size_t		keylen	;
    ${typecode}		key	[1];
};

struct map_node_${typearr} {
    struct map_mixed		data    ;
    struct map_link_${typearr}	link    ;
};

//-----------------------------------------------------------------------------
// function	map_${typearr}_length (internal, inline)
//
// purpose	Determine the length of a string of this key or data type.
//
// arguments	1 (const ${typecode} *) string pointer
//
// returns	(size_t) length of string
//-----------------------------------------------------------------------------
inline static
size_t
map_${typearr}_length (
    const ${typecode} *	arg_str
    )
{
    size_t	len	;
    len = 0;
    while ( * arg_str ++ != (${typecode}) 0 ) ++ len;
    return len;
}

//-----------------------------------------------------------------------------
// function	map_${typearr}_copy_array (internal, inline)
//
// purpose	Copy a string of this key or data type, and terminate it.
//
// arguments	1 (${typecode} *) destination pointer
//		2 (const ${typecode} *) source pointer
//		3 (size_t) length to copy
//
// returns	(void) nothing
//-----------------------------------------------------------------------------
inline static
void
map_${typearr}_copy_array (
    ${typecode} *		arg_dst
    ,
    const ${typecode} *	arg_src
    ,
    size_t		arg_len
    )
{
    while ( arg_len -- ) * arg_dst ++ = * arg_src ++;
    * arg_dst = (${typecode}) 0;
    return;
}

EOF
    fi
}

##############################################################################
function map_compare {
    prefix
    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_compare (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This function is the core fast comparison.
//
// arguments	1 (struct map_link_${typearr} *)
//		2 (struct map_link_${typearr} *)
//		3 (AVL *) AVL link pointer (ignored)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_${typearr}_compare (
    void *		arg_one
    ,
    void *		arg_two
    ,
    AVL *		arg_avl
    )
__PROTO_END__
{
    ${typecode} *		str1	;
    ${typecode} *		str2	;
    size_t		rem1	;
    size_t		rem2	;

    str1 = (${typecode} *)( ( (struct map_link_${typearr} *) arg_one )->key );
    rem1 = ( (struct map_link_${typearr} *) arg_one )->keylen;

    str2 = (${typecode} *)( ( (struct map_link_${typearr} *) arg_two )->key );
    rem2 = ( (struct map_link_${typearr} *) arg_two )->keylen;

    //-----------------------------
    // Begin the main compare loop.
    //-----------------------------
    for (;;) {

        //-- If either string is exhausted then return now.
        if ( rem1 == 0 ) return ( rem2 == 0 ) ? 0 : -1;
        if ( rem2 == 0 ) return 1;

	//-- If unequal then return the difference.
	if ( * str1 != * str2 ) return ( * str1 < * str2 ) ? -1 : 1;

	//-- Next.
	++ str1;
	++ str2;
	-- rem1;
	-- rem2;
    }
    //-- Never get here.
}

EOF
    fi
}

##############################################################################
function map_compare_str {
    prefix
    if oneof "${typearr}" ca sca uca; then
	cat <<EOF
__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_compare_str (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not intended for direct reference by application programs.
//		It is used only to establish the comparison method for the
//		AVL tree used to hold this mapping.
//
// features	This compare function provides for optional features to do the
//		comparison in varied ways.  These include:
//
//		lower:	Compare alphabetic characters as if converted to lower
//			case (even if compared to non-alphabetic).
//
//		upper:	Compare alphabetic characters as if converted to upper
//			case (even if compared to non-alphabetic).
//
//		digits:	Compare sequences of 1 or more digits by prefixing the
//			shorter sequence with 0 digits to make the comparison
//			work on the numeric value.
//
//		spaces:	Compare sequences of 1 or more non-graphical characters
//			as being equal to a single binary 0 character.
//
//		paths:	Compare file path directory separator characters as
//			being lower in collating order than other characters.
//
//		Options are set in the map_${typearr}_cmp_func_opt function.
//
// arguments	1 (struct map_link_${typearr} *)
//		2 (struct map_link_${typearr} *)
//		3 (AVL *) AVL link pointer
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_${typearr}_compare_str (
    void *		arg_one
    ,
    void *		arg_two
    ,
    AVL *		arg_avl
    )
__PROTO_END__
{
    ${typecode} *		str1	;
    ${typecode} *		str2	;
    size_t		rem1	;
    size_t		rem2	;

#ifdef MAP_OPT_LOWER
    int			opt_lower	;
#endif
#ifdef MAP_OPT_UPPER
    int			opt_upper	;
#endif
#ifdef MAP_OPT_DIGITS
    int			opt_digits	;
#endif
#ifdef MAP_OPT_SPACES
    int			opt_spaces	;
#endif
#ifdef MAP_OPT_PATHS
    int			opt_paths	;
#endif

    str1 = (${typecode} *)( ( (struct map_link_${typearr} *) arg_one )->key );
    rem1 = ( (struct map_link_${typearr} *) arg_one )->keylen;

    str2 = (${typecode} *)( ( (struct map_link_${typearr} *) arg_two )->key );
    rem2 = ( (struct map_link_${typearr} *) arg_two )->keylen;

#ifdef MAP_OPT_LOWER
    opt_lower	= map_from_avl( arg_avl )->options & MAP_OPT_LOWER;
#endif
#ifdef MAP_OPT_UPPER
    opt_upper	= map_from_avl( arg_avl )->options & MAP_OPT_UPPER;
#endif
#ifdef MAP_OPT_DIGITS
    opt_digits	= map_from_avl( arg_avl )->options & MAP_OPT_DIGITS;
#endif
#ifdef MAP_OPT_SPACES
    opt_spaces	= map_from_avl( arg_avl )->options & MAP_OPT_SPACES;
#endif
#ifdef MAP_OPT_PATHS
    opt_paths	= map_from_avl( arg_avl )->options & MAP_OPT_PATHS;
#endif

    //-----------------------------
    // Begin the main compare loop.
    //-----------------------------
    for (;;) {

        //-- If either string is exhausted then return now.
        if ( rem1 == 0 ) return ( rem2 == 0 ) ? 0 : -1;
        if ( rem2 == 0 ) return 1;

#ifdef MAP_OPT_DIGITS
        //-- If both characters are digits then handle as numeric substrings.
        if ( opt_digits && isdigit( * str1 ) && isdigit( * str2 ) ) {
            const ${typecode} *	ptr1    ;
            const ${typecode} *	ptr2    ;
            size_t              len1    ;
            size_t              len2    ;

            //-- Find the end of each numeric substring to know which is longer.
            ptr1 = str1;
            while ( -- rem1 && isdigit( * ptr1 ) ) ++ ptr1;
            len1 = ptr1 - str1;
            ptr2 = str2;
            while ( -- rem2 && isdigit( * ptr2 ) ) ++ ptr2;
            len2 = ptr2 - str2;

            //-- Compare the leading longer digits against all zero padding digits.
            if ( len1 > len2 ) {
                while ( len1 > len2 ) {
                    if ( * str1 > '0' ) return 1;
                    ++ str1; -- len1;
                }
            } else if ( len2 > len1 ) {
                while ( len2 > len1 ) {
                    if ( * str2 > '0' ) return -1;
                    ++ str2; -- len2;
                }
            }

            //-- Compare the rest of the digits very simply.
            while ( str1 < ptr1 ) {
                if ( * str1 != * str2 ) return ( * str1 < * str2 ) ? -1 : 1;
                ++ str1, ++ str2;
            }

	    continue;
        }
#endif

        //-- Otherwise compare as non-digits.
        {
            ${typeint}	ch1     ;
            ${typeint}	ch2     ;

            //-- If substring of spaces, then collapse and collate lower.
            ch1 = * str1 ++;
            ch2 = * str2 ++;

#ifdef MAP_OPT_SPACES
	    if ( opt_spaces ) {
		if ( ! isgraph( ch1 ) ) {
		    while ( -- rem1 && ! isgraph( * str1 ) ) ++ str1;
		    ch1 = 0;
		}
		if ( ! isgraph( ch2 ) ) {
		    while ( -- rem2 && ! isgraph( * str2 ) ) ++ str2;
		    ch2 = 0;
		}
	    }
#endif

#ifdef MAP_OPT_LOWER
	    //-- Remap to lower case for caseless comparison.
	    if ( opt_lower ) {
		ch1 = tolower( ch1 );
		ch2 = tolower( ch2 );
	    }
#endif

#ifdef MAP_OPT_UPPER
	    //-- Remap to upper case for caseless comparison.
	    if ( opt_upper ) {
		ch1 = toupper( ch1 );
		ch2 = toupper( ch2 );
	    }
#endif

#ifdef MAP_OPT_PATHS
	    //-- Remap collation for filepath separators even lower than spaces.
	    if ( opt_paths ) {
		if ( ch1 == '/' ) ch1 = -1;
		if ( ch2 == '/' ) ch2 = -1;
	    }
#endif

	    //-- If unequal then determine the difference orientation and return.
	    if ( ch1 != ch2 ) return ( ch1 < ch2 ) ? -1 : 1;

	    -- rem1;
	    -- rem2;
        }
    }
    //-- Never get here.
}

EOF
    fi
}

##############################################################################
function map_compare_alt {
    prefix
    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_compare_alt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied alternate
//		compare function.
//
// arguments	1 (struct map_link_${typearr} *)
//		2 (struct map_link_${typearr} *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const ${typecode} *) key 1
//		2 (size_t) length of key 1
//		3 (const ${typecode} *) key 2
//		4 (size_t) length of key 2
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_${typearr}_compare_alt (
    void *		arg_one
    ,
    void *		arg_two
    ,
    AVL *		arg_avl
    )
__PROTO_END__
{
    return ( * map_cmp_func( arg_avl ) ) (
	( (struct map_link_${typearr} *) arg_one )->key,
	( (struct map_link_${typearr} *) arg_one )->keylen,
	( (struct map_link_${typearr} *) arg_two )->key,
	( (struct map_link_${typearr} *) arg_two )->keylen
	);
}

EOF
    fi
}

##############################################################################
function map_compare_alt_opt {
    prefix
    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_compare_alt_opt (internal)
//
// purpose	Compare the array keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// features	This function is used to invoke a caller supplied comparison
//		function.
//
// arguments	1 (struct map_link_${typearr} *)
//		2 (struct map_link_${typearr} *)
//		3 (void *) alternate compare function
//
// returns	(int) comparison status, -1, 0, 1
//
// The alternate compare function has this interface:
//
// arguments	1 (const ${typecode} *) key 1
//		2 (size_t) length of key 1
//		3 (const ${typecode} *) key 2
//		4 (size_t) length of key 2
//		5 (int) options
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
int
map_${typearr}_compare_alt_opt (
    void *		arg_one
    ,
    void *		arg_two
    ,
    AVL *		arg_avl
    )
__PROTO_END__
{
    return ( * map_cmp_func( arg_avl ) ) (
	( (struct map_link_${typearr} *) arg_one )->key,
	( (struct map_link_${typearr} *) arg_one )->keylen,
	( (struct map_link_${typearr} *) arg_two )->key,
	( (struct map_link_${typearr} *) arg_two )->keylen,
	map_from_avl( arg_avl )->options
	);
}

EOF
    fi
}

##############################################################################
function map_new {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_new
//
// purpose	Create a new mapping with a string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_str_new map_ca_new
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_new
//
// purpose	Create a new mapping with the specific string key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
#define map_${typestr}_new map_${typearr}_new
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_new
//
// purpose	Create a new mapping with the specific array key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_${typearr}_new ()
__PROTO_END__
{
    MAP		this_map	;

    //-- Allocate a structure for the root of the mapping.
    if ( ! ( this_map = malloc( sizeof (struct map_root) ) ) ) return NULL;

    //-- Record the key type.
    this_map->key_type = ${typesymb};

    //-- Let the AVL layer initialize the rest of it.
    if ( ! avl_init( & (this_map->avl_tree),
		     struct map_node_${typearr},
		     link,
		     map_${typearr}_compare
		    )
	) return NULL;

    //-- Return the handle to the new mapping.
    return this_map;
}

EOF
    fi
}

##############################################################################
function map_cmp_func {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const ${typecode} *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const ${typecode} *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_func map_ca_cmp_func
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const ${typecode} *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const ${typecode} *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_${typestr}_cmp_func map_${typearr}_cmp_func
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_cmp_func
//
// purpose	Set the comparison function to the caller supplied function.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//
// comparison	1 (const ${typecode} *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const ${typecode} *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_${typearr}_cmp_func (
    MAP		arg_map
    ,
    int		(* arg_compare)( const ${typecode} *
				 ,
				 size_t
				 ,
				 const ${typecode} *
				 ,
				 size_t
				 )
   )
__PROTO_END__
{
    //-- Make sure the mapping is empty.
    map_empty( arg_map );

    //-- Set up for custom comparison.
    arg_map->options = 0;
    arg_map->cmp_func = (int (*)()) arg_compare;

    //-- Re-initialize the AVL layer.
    if ( ! avl_init( & (arg_map->avl_tree),
		     struct map_node_${typearr},
		     link,
		     map_${typearr}_compare_alt
		    )
	) return -1;

    return 0;
}

EOF
    fi
}

##############################################################################
function map_cmp_func_opt {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const ${typecode} *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const ${typecode} *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_func_opt map_ca_cmp_func_opt
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const ${typecode} *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const ${typecode} *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_${typestr}_cmp_func_opt map_${typearr}_cmp_func_opt
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const ${typecode} *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const ${typecode} *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_${typearr}_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const ${typecode} *
				,
				size_t
				,
				const ${typecode} *
				,
				size_t
				)
    ,
    int		arg_opt
   )
__PROTO_END__
{
    //-- Make sure the mapping is empty.
    map_empty( arg_map );

    //-- Set up for custom comparison.
    arg_map->options = arg_opt;
    arg_map->cmp_func = (int (*)()) arg_compare;

    //-- Re-initialize the AVL layer.
    if ( ! avl_init( & (arg_map->avl_tree),
		     struct map_node_${typearr},
		     link,
		     map_${typearr}_compare_alt_opt
		    )
	) return -1;

    return 0;
}

EOF
    fi
}

##############################################################################
function map_cmp_feature {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_cmp_feature
//
// purpose	Set the comparison feature to enable or disable the options
//		specified in the 3rd argument.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//		3 (int) which options
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, all previously disabled
//		(int)  > 0 : OK, one or more previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_feature map_ca_cmp_feature
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_cmp_feature
//
// purpose	Set the comparison feature to enable or disable the options
//		specified in the 3rd argument.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//		3 (int) which options
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, all previously disabled
//		(int)  > 0 : OK, one or more previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_${typestr}_cmp_feature map_${typearr}_cmp_feature
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_cmp_feature
//
// purpose	Set the comparison feature to enable or disable the options
//		specified in the 3rd argument.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//		3 (int) which options
//
// returns	(int)  < 0 : error
//		(int) == 0 : OK, all previously disabled
//		(int)  > 0 : OK, one or more previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_${typearr}_cmp_feature (
    MAP		arg_map
    ,
    int		arg_state
    ,
    int		arg_flag
   )
__PROTO_END__
{
    int		previous	;

    //-- Get the previous setting before making changes.
    previous = ( arg_map->options & arg_flag ) ? 1 : 0;

    //-- If changes are to be made, make them.
    if ( arg_state >= 0 ) {

	//-- Make sure the mapping is empty.
	map_empty( arg_map );

	//-- Change appropriate settings.
	arg_map->cmp_func = NULL;
	if ( arg_state == 0 ) arg_map->options &= ~arg_flag;
	if ( arg_state == 1 ) arg_map->options |= arg_flag;

	//-- Set up the appropriate AVL comparison function and re-initialize.
	if ( ! avl_init( & (arg_map->avl_tree),
			 struct map_node_${typearr}, link,
			 ( arg_map->options
			     & (MAP_OPT_LOWER|MAP_OPT_UPPER|MAP_OPT_DIGITS|MAP_OPT_SPACES|MAP_OPT_PATHS) )
			 ? map_${typearr}_compare_str
			 : map_${typearr}_compare
	    ) ) {
	    return -1;
	}
    }

    //-- Return the previous setting of this option.
    return previous;
}

EOF
    fi
}

##############################################################################
function map_cmp_lower {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_cmp_lower
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of lower case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_lower map_ca_cmp_lower
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_cmp_lower
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of lower case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_${typestr}_cmp_lower map_${typearr}_cmp_lower
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_cmp_lower
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of lower case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_${typearr}_cmp_lower (
    MAP		arg_map
    ,
    int		arg_change
   )
__PROTO_END__
{
    int		previous	;

    //-- Make sure the mapping is empty.
    map_empty( arg_map );

    //-- If not currently set for special string compare, change it.
    if ( arg_change >= 0 && arg_map->cmp_func != map_${typearr}_compare_str ) {
	arg_map->cmp_func = map_${typearr}_compare_str;
	arg_map->options = 0;
    }

    //-- Hold on to the previous options.
    previous = arg_map->options;

    //-- Make appropriate changes based on the change flag.
    if ( arg_change >= 0 ) {

	//-- Disable or enable the option.
	if ( arg_change == 0 ) {
	    arg_map->options &= ~(MAP_OPT_UPPER);
	} else if ( arg_change > 0 ) {
	    arg_map->options |= MAP_OPT_LOWER;
	    arg_map->options &= ~(MAP_OPT_UPPER);
	}

	//-- Re-initialize the AVL layer.
	if ( ! avl_init( & (arg_map->avl_tree),
			 struct map_node_${typearr},
			 link,
			 map_${typearr}_compare_str ) ) return -1;
    }

    return ! ! ( previous & MAP_OPT_LOWER );
}

EOF
    fi
}

##############################################################################
function map_cmp_upper {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_cmp_upper
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of upper case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_upper map_ca_cmp_upper
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_cmp_upper
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of upper case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_${typestr}_cmp_upper map_${typearr}_cmp_upper
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_cmp_upper
//
// purpose	Set the comparison feature to enable or disable alphabetic
//		case insensitive comparison by means of upper case conversion.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_${typearr}_cmp_upper (
    MAP		arg_map
    ,
    int		arg_change
   )
__PROTO_END__
{
    int		previous	;

    //-- Make sure the mapping is empty.
    map_empty( arg_map );

    //-- If not currently set for special string compare, change it.
    if ( arg_change >= 0 && arg_map->cmp_func != map_${typearr}_compare_str ) {
	arg_map->cmp_func = map_${typearr}_compare_str;
	arg_map->options = 0;
    }

    //-- Hold on to the previous options.
    previous = arg_map->options;

    //-- Make appropriate changes based on the change flag.
    if ( arg_change >= 0 ) {

	//-- Disable or enable the option.
	if ( arg_change == 0 ) {
	    arg_map->options &= ~(MAP_OPT_LOWER);
	} else if ( arg_change > 0 ) {
	    arg_map->options |= MAP_OPT_UPPER;
	    arg_map->options &= ~(MAP_OPT_LOWER);
	}

	//-- Re-initialize the AVL layer.
	if ( ! avl_init( & (arg_map->avl_tree),
			 struct map_node_${typearr},
			 link,
			 map_${typearr}_compare_str ) ) return -1;
    }

    return ! ! ( previous & MAP_OPT_UPPER );
}

EOF
    fi
}

##############################################################################
function map_cmp_digits {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_cmp_digits
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of decimal digits by their numeric value.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_digits map_ca_cmp_digits
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_cmp_digits
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of decimal digits by their numeric value.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_${typestr}_cmp_digits map_${typearr}_cmp_digits
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_cmp_digits
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of decimal digits by their numeric value.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_${typearr}_cmp_digits (
    MAP		arg_map
    ,
    int		arg_change
   )
__PROTO_END__
{
    int		previous	;

    //-- Make sure the mapping is empty.
    map_empty( arg_map );

    //-- If not currently set for special string compare, change it.
    if ( arg_change >= 0 && arg_map->cmp_func != map_${typearr}_compare_str ) {
	arg_map->cmp_func = map_${typearr}_compare_str;
	arg_map->options = 0;
    }

    //-- Hold on to the previous options.
    previous = arg_map->options;

    //-- Make appropriate changes based on the change flag.
    if ( arg_change >= 0 ) {

	//-- Disable or enable the option.
	if ( arg_change == 0 ) {
	    arg_map->options &= ~(MAP_OPT_DIGITS);
	} else if ( arg_change > 0 ) {
	    arg_map->options |= MAP_OPT_DIGITS;
	}

	//-- Re-initialize the AVL layer.
	if ( ! avl_init( & (arg_map->avl_tree),
			 struct map_node_${typearr},
			 link,
			 map_${typearr}_compare_str ) ) return -1;
    }

    return ! ! ( previous & MAP_OPT_DIGITS );
}

EOF
    fi
}

##############################################################################
function map_cmp_spaces {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_cmp_spaces
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of non-graphical characters as being collapsed
//		to a single binary 0 character.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_spaces map_ca_cmp_spaces
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_cmp_spaces
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of non-graphical characters as being collapsed
//		to a single binary 0 character.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_${typestr}_cmp_spaces map_${typearr}_cmp_spaces
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_cmp_spaces
//
// purpose	Set the comparison feature to enable or disable comparing
//		substrings of non-graphical characters as being collapsed
//		to a single binary 0 character.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_${typearr}_cmp_spaces (
    MAP		arg_map
    ,
    int		arg_change
   )
__PROTO_END__
{
    int		previous	;

    //-- Make sure the mapping is empty.
    map_empty( arg_map );

    //-- If not currently set for special string compare, change it.
    if ( arg_change >= 0 && arg_map->cmp_func != map_${typearr}_compare_str ) {
	arg_map->cmp_func = map_${typearr}_compare_str;
	arg_map->options = 0;
    }

    //-- Hold on to the previous options.
    previous = arg_map->options;

    //-- Make appropriate changes based on the change flag.
    if ( arg_change >= 0 ) {

	//-- Disable or enable the option.
	if ( arg_change == 0 ) {
	    arg_map->options &= ~(MAP_OPT_SPACES);
	} else if ( arg_change > 0 ) {
	    arg_map->options |= MAP_OPT_SPACES;
	}

	//-- Re-initialize the AVL layer.
	if ( ! avl_init( & (arg_map->avl_tree),
			 struct map_node_${typearr},
			 link,
			 map_${typearr}_compare_str ) ) return -1;
    }

    return ! ! ( previous & MAP_OPT_SPACES );
}

EOF
    fi
}

##############################################################################
function map_cmp_paths {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_cmp_paths
//
// purpose	Set the comparison feature to enable or disable file paths
//		with the '/' character lower than other characters.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_paths map_ca_cmp_paths
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_cmp_paths
//
// purpose	Set the comparison feature to enable or disable file paths
//		with the '/' character lower than other characters.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_${typestr}_cmp_paths map_${typearr}_cmp_paths
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_cmp_paths
//
// purpose	Set the comparison feature to enable or disable file paths
//		with the '/' character lower than other characters.
//
// arguments	1 (MAP) mapping handle
//		2 (int) -1: no change, 0: disable, 1: enable
//
// returns	(int) -1 : error
//		(int)  0 : OK, previously disabled
//		(int)  1 : OK, previously enabled
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_${typearr}_cmp_paths (
    MAP		arg_map
    ,
    int		arg_change
   )
__PROTO_END__
{
    int		previous	;

    //-- Make sure the mapping is empty.
    map_empty( arg_map );

    //-- If not currently set for special string compare, change it.
    if ( arg_change >= 0 && arg_map->cmp_func != map_${typearr}_compare_str ) {
	arg_map->cmp_func = map_${typearr}_compare_str;
	arg_map->options = 0;
    }

    //-- Hold on to the previous options.
    previous = arg_map->options;

    //-- Make appropriate changes based on the change flag.
    if ( arg_change >= 0 ) {

	//-- Disable or enable the option.
	if ( arg_change == 0 ) {
	    arg_map->options &= ~(MAP_OPT_PATHS);
	} else if ( arg_change > 0 ) {
	    arg_map->options |= MAP_OPT_PATHS;
	}

	//-- Re-initialize the AVL layer.
	if ( ! avl_init( & (arg_map->avl_tree),
			 struct map_node_${typearr},
			 link,
			 map_${typearr}_compare_str ) ) return -1;
    }

    return ! ! ( previous & MAP_OPT_PATHS );
}

EOF
    fi
}

##############################################################################
function map_find {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find_exact(m,k) map_ca_find((m),(k),~(size_t)0,0)
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_${typestr}_find_exact(m,k) map_${typearr}_find((m),(k),~(size_t)0,0)

//-----------------------------------------------------------------------------
// macro	map_${typearr}_find_exact
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If multiple members match,
//		this function does not define what particular member will be
//		selected.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_${typearr}_find_exact(m,k,l) map_${typearr}_find((m),(k),(l),0)
__FMACRO_END__

EOF
    fi

    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find_exact_lo(m,k) map_ca_find((m),(k),~(size_t)0,-1)
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_${typestr}_find_exact_lo(m,k) map_${typearr}_find((m),(k),~(size_t)0,-1)

//-----------------------------------------------------------------------------
// function	map_${typearr}_find_exact_lo
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the lowest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_${typearr}_find_exact_lo(m,k,l) map_${typearr}_find((m),(k),(l),-1)
__FMACRO_END__

EOF
    fi

    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find_lo_or_near(m,k) map_ca_find((m),(k),~(size_t)0,-2)
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_${typestr}_find_lo_or_near(m,k) map_${typearr}_find((m),(k),~(size_t)0,-2)

//-----------------------------------------------------------------------------
// macro	map_${typearr}_find_lo_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest lower key.  If there is
//		more than one exactly matching key, select the one in the
//		lowest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_${typearr}_find_lo_or_near(m,k,l) map_${typearr}_find((m),(k),(l),-2)
__FMACRO_END__

EOF
    fi

    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find_exact_hi(m,k) map_ca_find((m),(k),~(size_t)0,1)
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_${typestr}_find_exact_hi(m,k) map_${typearr}_find((m),(k),~(size_t)0,1)

//-----------------------------------------------------------------------------
// macro	map_${typearr}_find_exact_hi
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping.  If there is more than
//		one member of the same key, select the one in the highest
//		position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_${typearr}_find_exact_hi(m,k,l) map_${typearr}_find((m),(k),(l),1)
__FMACRO_END__

EOF
    fi

    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find_hi_or_near(m,k) map_ca_find((m),(k),~(size_t)0,2)
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_${typestr}_find_hi_or_near(m,k) map_${typearr}_find((m),(k),~(size_t)0,2)

//-----------------------------------------------------------------------------
// macro	map_${typearr}_find_hi_or_near
//
// purpose	Find a member with an exactly matching key of the specific
//		array key type for this mapping, or if there is no exactly
//		matching key, select the nearest higher key.  If there is
//		more than one exactly matching key, select the one in the
//		highest position.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_${typearr}_find_hi_or_near(m,k,l) map_${typearr}_find((m),(k),(l),2)
__FMACRO_END__

EOF
    fi

    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_str_find(m,k,d) map_ca_find((m),(k),~(size_t)0,(d))
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
#include <alloca.h>

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) array key pointer
//		3 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
#define map_${typestr}_find(m,k,d) map_${typearr}_find((m),(k),~(size_t)0,(d))
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_find
//
// purpose	Find a member with a matching key, or possibly a nearby key.
//
//		If the influential direction is none, return the first exact
//		match encountered, or return none.
//
//		If the influential direction is low, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		low direction.
//
//		If the influential direction is high, return the lowest exact
//		match encountered, or if there is no exact match: if the
//		near option is specified, choose the nearest member in the
//		high direction.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) array key pointer
//		3 (size_t) array key length
//		4 (int) direction and near option, -2, -1, 0, 1, 2
//			-2 = choose lowest exact, or nearest lower
//			-1 = choose lowest exact only
//			 0 = choose first exact only
//			 1 = choose highest exact only
//			 2 = choose highest exact, or nearest higher
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = no member found
//		(int)  0 = found a member with no data
//		(int) data type of found member
//-----------------------------------------------------------------------------
int
map_${typearr}_find (
    MAP			arg_map
    ,
    const ${typecode} *	arg_key_ptr
    ,
    size_t		arg_key_len
    ,
    int			arg_dir_near
   )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p		;
    struct map_link_${typearr} *	finder_p	;
    size_t		len		;

    //-- Make sure we have a correct mapping.
    if ( ! arg_map ) return -2;
    if ( arg_map->key_type != ${typesymb} ) return -2;

    //-- If the length is not given, determine it as zero terminated string.
    if ( arg_key_len == ~ (size_t) 0U ) arg_key_len = map_${typearr}_length( arg_key_ptr );

    //-- Set up a search link to pass the key.
    len = sizeof (struct map_link_${typearr}) + arg_key_len * sizeof (${typecode});
    if ( ! ( finder_p = alloca( len ) ) ) return -1;
    finder_p->keylen = arg_key_len;
    map_${typearr}_copy_array( finder_p->key, arg_key_ptr, arg_key_len );

    //-- Let the AVL layer do the search.
    node_p = avl_find( & (arg_map->avl_tree), finder_p, arg_dir_near );

    //-- Return the found data type or -1 if none found.
    return ( node_p ) ? node_p->data.type : -1;
}

EOF
    fi
}

##############################################################################
function map_insert {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_str_insert(m,k) map_ca_insert((m),(k),~(size_t)0)
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_${typestr}_insert(m,k) map_${typearr}_insert((m),(k),~(size_t)0)
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_${typearr}_insert (
    MAP			arg_map
    ,
    const ${typecode} *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;
    int			rc	;

    //-- Make sure we have a correct mapping.
    if ( ! arg_map ) return -2;
    if ( arg_map->key_type != ${typesymb} ) return -2;

    //-- If the length is not given, determine it as zero terminated string.
    if ( arg_key_len == ~ (size_t) 0U ) arg_key_len = map_${typearr}_length( arg_key_ptr );

    //-- Allocate a combined node and key space.  The node already
    //-- has the space for one element of the key so it is not
    //-- necessary to add an extra one for the terminating element.
    if ( ! ( node_p = malloc( sizeof (struct map_node_${typearr}) + sizeof (${typecode}) * arg_key_len ) ) ) return -2;

    //-- Initialize the node.
    node_p->data.type = 0;
    node_p->link.keylen = arg_key_len;

    //-- Copy the key.
    map_${typearr}_copy_array( node_p->link.key, arg_key_ptr, arg_key_len );

    //-- Insert node or return data type if already exists.
    rc = avl_insert( & (arg_map->avl_tree), node_p );
    if ( rc >= 0 ) return 0;
    free( node_p );
    return map_${typearr}_find( arg_map, arg_key_ptr, arg_key_len, 0 );
}

EOF
    fi
}

##############################################################################
function map_insert_dup {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_str_insert_dup(m,k) map_ca_insert_dup((m),(k),~(size_t)0)
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_${typestr}_insert_dup(m,k) map_${typearr}_insert_dup((m),(k),~(size_t)0)
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_${typearr}_insert_dup (
    MAP			arg_map
    ,
    const ${typecode} *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;
    size_t		len	;
    int			rc	;

    //-- Make sure we have a correct mapping.
    if ( ! arg_map ) return -2;
    if ( arg_map->key_type != ${typesymb} ) return -2;

    //-- If the length is not given, determine it as zero terminated string.
    if ( arg_key_len == ~ (size_t) 0U ) arg_key_len = map_${typearr}_length( arg_key_ptr );

    //-- Allocate a combined node and key space.  The node already
    //-- has the space for one element of the key so it is not
    //-- necessary to add an extra one for the terminating element.
    len = ( sizeof (struct map_node_${typearr}) ) + ( arg_key_len * sizeof (${typecode}) );
    if ( ! ( node_p = malloc( len ) ) ) return -2;

    //-- Initialize the node.
    node_p->data.type = 0;
    node_p->link.keylen = arg_key_len;

    //-- Copy the key.
    map_${typearr}_copy_array( node_p->link.key, arg_key_ptr, arg_key_len );

    //-- Insert the node.
    rc = avl_insert_dup( & (arg_map->avl_tree), node_p );

    //-- Return status based on insertion.
    if ( rc < 0 ) {
	free( node_p );
	return -1;
    }
    return rc;
}

EOF
    fi
}

##############################################################################
function map_key_ptr {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const ${typecode} *) pointer to shared key
//		(const ${typecode} *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_str_key_ptr map_ca_key_ptr
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const ${typecode} *) pointer to shared key
//		(const ${typecode} *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_${typestr}_key_ptr map_${typearr}_key_ptr
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const ${typecode} *) pointer to shared key
//		(const ${typecode} *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const ${typecode} *
map_${typearr}_key_ptr (
    MAP			arg_map
   )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;

    //-- Make sure we have a correct mapping and current node.
    if ( ! arg_map ) return NULL;
    if ( arg_map->key_type != ${typesymb} ) return NULL;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return NULL;

    //-- Return the key pointer.
    return node_p->link.key;
}

EOF
    fi
}

##############################################################################
function map_key_dup {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(${typecode} *) pointer to key duplicate copy
//		(${typecode} *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_str_key_dup map_ca_key_dup
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(${typecode} *) pointer to key duplicate copy
//		(${typecode} *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_${typestr}_key_dup map_${typearr}_key_dup
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(${typecode} *) pointer to key duplicate copy
//		(${typecode} *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
${typecode} *
map_${typearr}_key_dup (
    MAP			arg_map
    )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;
    ${typecode} *		key_p	;
    size_t		len	;

    //-- Make sure we have a correct mapping and current node.
    if ( ! arg_map ) return NULL;
    if ( arg_map->key_type != ${typesymb} ) return NULL;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return NULL;

    //-- Get key length.
    len = node_p->link.keylen;

    //-- Allocate space for key duplicate or return an error.
    if ( ! ( key_p = malloc( ( len + 1 ) * sizeof (${typecode}) ) ) ) return NULL;
    
    //-- Copy the key for duplication.
    map_${typearr}_copy_array( key_p, node_p->link.key, len );

    //-- Return the pointer to the duplicate.
    return key_p;
}

EOF
    fi
}

##############################################################################
function map_key_copy {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (${typecode} *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_str_key_copy map_ca_key_copy
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (${typecode} *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_${typestr}_key_copy map_${typearr}_key_copy
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_key_copy
//
// purpose	Get the key of the current member in the mapping, copying it to
//		the specified destination area.
//
// arguments	1 (MAP) mapping handle
//		2 (${typecode} *) where to store key
//		3 (size_t) maximum length of storage area
//
// returns	(size_t) length of key being returned (even if not all copied)
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_${typearr}_key_copy (
    MAP			arg_map
    ,
    ${typecode} *		arg_key_ptr
    ,
    size_t		arg_key_len
   )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;
    size_t		len	;

    //-- Make sure we have a correct mapping and current node.
    if ( ! arg_map ) return ~ (size_t) 0U;
    if ( arg_map->key_type != ${typesymb} ) return ~ (size_t) 0U;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return ~ (size_t) 0U;

    //-- Copy the key to the destination, truncated if necessary.
    len = node_p->link.keylen;
    if ( len > arg_key_len ) len = arg_key_len;
    map_${typearr}_copy_array( arg_key_ptr, node_p->link.key, len );

    //-- Return length for convenience.
    return node_p->link.keylen;
}

EOF
    fi
}

##############################################################################
function map_key_len {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_str_key_len map_ca_key_len
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_${typestr}_key_len map_${typearr}_key_len
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_${typearr}_key_len (
    MAP			arg_map
    )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;

    //-- Make sure we have a correct mapping and current node.
    if ( ! arg_map ) return ~ (size_t) 0U;
    if ( arg_map->key_type != ${typesymb} ) return ~ (size_t) 0U;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return ~ (size_t) 0U;

    //-- Return the key length.
    return node_p->link.keylen;
}

EOF
    fi
}

##############################################################################
function map_store {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_str_store(m,d) map_ca_store((m),(d),~(size_t)0)
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) data value to store
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_${typestr}_store(m,d) map_${typearr}_store((m),(d),~(size_t)0)
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_store
//
// purpose	Store the given data into the current member in the mapping.
//		String/array data is stored as a duplicate so the original
//		data can be modified or the space reused or freed after
//		this function returns.
//
// arguments	1 (MAP) mapping handle
//		2 (const ${typecode} *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member, out of memory
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_${typearr}_store (
    MAP			arg_map
    ,
    const ${typecode} *	arg_data_ptr
    ,
    size_t		arg_data_len
   )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;
    ${typecode} *	data_p	;

    //-- Make sure we have a mapping and a current node.
    if ( ! arg_map ) return -2;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return -2;

    //-- If the length is not given, determine it as zero terminated string.
    if ( arg_data_len == ~ (size_t) 0U ) arg_data_len = map_${typearr}_length( arg_data_ptr );

    //-- If the data type and length are the same, reuse the old data space.
    if ( node_p->data.type == ${typesymb} && node_p->data.mix.array.len == arg_data_len ) {
	data_p = node_p->data.mix.array.ptr;
    }

    //-- Else the old data is not suitable for reuse.
    else {

	//-- Free any old duplicated data.
	if ( map_type_is_string( node_p->data.type ) && node_p->data.mix.array.dup ) {
	    free( node_p->data.mix.array.ptr );
	}

	//-- Allocate space for the data to be duplicated.
	if ( ! ( data_p = malloc( ( arg_data_len + 1 ) * sizeof (${typecode}) ) ) ) return -2;
    }

    //-- Copy the data to the duplication space.
    map_${typearr}_copy_array( data_p, arg_data_ptr, arg_data_len );

    //-- Store data in the node.
    node_p->data.mix.array.ptr = (void *) data_p;
    node_p->data.mix.array.len = arg_data_len;
    node_p->data.mix.array.dup = 1;
    node_p->data.type = ${typesymb};

    //-- Return a success status.
    return 0;
}

EOF
    fi
}

##############################################################################
function map_store_share {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (${typecode} *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_str_store_share(m,d) map_ca_store_share((m),(d),~(size_t)0)
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (${typecode} *) data value to store
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
#define map_${typestr}_store_share(m,d) map_${typearr}_store_share((m),(d),~(size_t)0)
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_store_share
//
// purpose	Store the given data into the current member in the mapping.
//		The data pointer is stored as given and a duplicate is not
//		made.  This results in data being shared between the mapping
//		and the calling program.  The calling program may modify the
//		data in place, and is responsible for the consequences of
//		doing so since it changes the data (but not the type) stored
//		in the mapping.  Such a change would also affect references
//		of fetched data pointers.
//
// arguments	1 (MAP) mapping handle
//		2 (${typecode} *) data value to store
//		3 (size_t) length of data value
//
// returns	(int) -2 = error, no map, no member
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_${typearr}_store_share (
    MAP			arg_map
    ,
    ${typecode} *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;

    //-- Make sure we have a mapping and a current node.
    if ( ! arg_map ) return -2;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return -2;

    //-- Free any old duplicated data.
    if ( map_type_is_string( node_p->data.type ) && node_p->data.mix.array.dup ) {
	free( node_p->data.mix.array.ptr );
    }

    //-- Store the data type, length, and pointer.
    node_p->data.mix.array.ptr = (void *) arg_data_ptr;
    node_p->data.mix.array.len = arg_data_len;
    node_p->data.mix.array.dup = 0;
    node_p->data.type = ${typesymb};

    //-- Return a success status.
    return 0;
}

EOF
    fi
}

##############################################################################
function map_fetch_ptr {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const ${typecode} *) pointer to data string array
//		(const ${typecode} *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_str_fetch_ptr map_${typestr}_fetch_ptr
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const ${typecode} *) pointer to data string array
//		(const ${typecode} *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_${typestr}_fetch_ptr map_${typearr}_fetch_ptr
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_fetch_ptr
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const ${typecode} *) pointer to data string array
//		(const ${typecode} *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
const ${typecode} *
map_${typearr}_fetch_ptr (
    MAP			arg_map
   )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;

    //-- Make sure we have a mapping and a current node.
    if ( ! arg_map ) return NULL;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return NULL;

    //-- Make sure data type is correct.
    if ( node_p->data.type != ${typesymb} ) return NULL;

    //-- Return the data pointer.
    return (const ${typecode} *) ( node_p->data.mix.array.ptr );
}

EOF
    fi
}

##############################################################################
function map_fetch_dup {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(${typecode} *) pointer to duplicate data string array
//		(${typecode} *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_str_fetch_dup map_${typestr}_fetch_dup
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(${typecode} *) pointer to duplicate data string array
//		(${typecode} *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_${typestr}_fetch_dup map_${typearr}_fetch_dup
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(${typecode} *) pointer to duplicate data string array
//		(${typecode} *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
${typecode} *
map_${typearr}_fetch_dup (
    MAP			arg_map
   )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;
    ${typecode} *		data_p	;
    size_t		len	;

    //-- Make sure we have a mapping and a current node.
    if ( ! arg_map ) return NULL;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return NULL;

    //-- Make sure data type is correct.
    if ( node_p->data.type != ${typesymb} ) return NULL;

    //-- Get data length.
    len = node_p->data.mix.array.len;

    //-- Allocate space for data duplicate or return an error.
    if ( ! ( data_p = malloc( ( len + 1 ) * sizeof (${typecode}) ) ) ) return NULL;

    //-- Copy the data for duplication.
    map_${typearr}_copy_array( data_p, (${typecode} *) ( node_p->data.mix.array.ptr ), len );

    //-- Return the pointer to the duplicate.
    return data_p;
}

EOF
    fi
}

##############################################################################
function map_fetch_copy {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (${typecode} *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_str_fetch_copy map_${typestr}_fetch_copy
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (${typecode} *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_${typestr}_fetch_copy map_${typearr}_fetch_copy
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_fetch_copy
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (${typecode} *) pointer where to store data
//		3 (size_t) maximum length to store
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_${typearr}_fetch_copy (
    MAP			arg_map
    ,
    ${typecode} *		arg_data_ptr
    ,
    size_t		arg_data_len
   )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;
    size_t		len	;

    //-- Make sure we have a mapping and a current node.
    if ( ! arg_map ) return ~ (size_t) 0U;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return ~ (size_t) 0U;

    //-- Make sure data type is correct.
    if ( node_p->data.type != ${typesymb} ) return ~ (size_t) 0U;

    //-- Copy the data to the destination, truncated if necessary.
    len = node_p->data.mix.array.len;
    if ( len > arg_data_len ) len = arg_data_len;
    map_${typearr}_copy_array( arg_data_ptr, (${typecode} *) ( node_p->data.mix.array.ptr ), len );

    //-- Return original length for convenience.
    return node_p->data.mix.array.len;
}

EOF
    fi
}

##############################################################################
function map_fetch_len {
    prefix
    if test $typearr = ca; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_str_fetch_len map_${typestr}_fetch_len
__FMACRO_END__

EOF
    fi

    if oneof "${typearr}" ca sca uca ssa usa sia uia sla ula slla ulla; then
	cat <<EOF
__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_${typestr}_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
#define map_${typestr}_fetch_len map_${typearr}_fetch_len
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_${typearr}_fetch_len
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) ~0U = error, no map, no member, bad type
//		(size_t) length of full string (copy may be truncated)
//-----------------------------------------------------------------------------
size_t
map_${typearr}_fetch_len (
    MAP			arg_map
   )
__PROTO_END__
{
    struct map_node_${typearr} *	node_p	;

    //-- Make sure we have a mapping and a current node.
    if ( ! arg_map ) return ~ (size_t) 0U;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return ~ (size_t) 0U;

    //-- Make sure data type is correct.
    if ( node_p->data.type != ${typesymb} ) return ~ (size_t) 0U;

    //-- Return length.
    return node_p->data.mix.array.len;
}
EOF
    fi
}

##############################################################################

#-----------------------------------------------------------------------------
# Generate code into a specified file and list that file.
#-----------------------------------------------------------------------------
function srcfile {
    call="${1}"
    file="${2}"

    "${call}" >"${file}.tmp"
    if [[ $( wc -l < "${file}.tmp" ) -gt 32 ]]; then
	touch -r "${script}" "${file}.tmp"
	chmod 644 "${file}.tmp"
	mv -f "${file}.tmp" "${file}"
	ls -ld --full-time "${file}"
    else
	rm -f "${file}.tmp"
    fi
}
script="$0"

function code {
    typearr="${1}a"
    typestr="${1}s"
    typecode="${2}"
    typesymb=$( echo "map_data_${typearr}" | tr 'a-z' 'A-Z' )
    typeint="${2}"
    oneof "${typearr}" sca ssa && typeint="signed int"
    oneof "${typearr}" ca uca usa && typeint="unsigned int"

    srcfile map_header		"map_${typearr}.h"
    srcfile map_compare		"map_${typearr}_compare.c"
    srcfile map_compare_str	"map_${typearr}_compare_str.c"
    srcfile map_compare_alt	"map_${typearr}_compare_alt.c"
    srcfile map_compare_alt_opt	"map_${typearr}_compare_alt_opt.c"
    srcfile map_new		"map_${typearr}_new.c"
    srcfile map_cmp_func	"map_${typearr}_cmp_func.c"
    srcfile map_cmp_func_opt	"map_${typearr}_cmp_func_opt.c"
    srcfile map_cmp_feature	"map_${typearr}_cmp_feature.c"
    srcfile map_cmp_lower	"map_${typearr}_cmp_lower.c"
    srcfile map_cmp_upper	"map_${typearr}_cmp_upper.c"
    srcfile map_cmp_digits	"map_${typearr}_cmp_digits.c"
    srcfile map_cmp_spaces	"map_${typearr}_cmp_spaces.c"
    srcfile map_cmp_paths	"map_${typearr}_cmp_paths.c"
    srcfile map_find		"map_${typearr}_find.c"
    srcfile map_insert		"map_${typearr}_insert.c"
    srcfile map_insert_dup	"map_${typearr}_insert_dup.c"
    srcfile map_key_ptr		"map_${typearr}_key_ptr.c"
    srcfile map_key_dup		"map_${typearr}_key_dup.c"
    srcfile map_key_copy	"map_${typearr}_key_copy.c"
    srcfile map_key_len		"map_${typearr}_key_len.c"
    srcfile map_store		"map_${typearr}_store.c"
    srcfile map_store_share	"map_${typearr}_store_share.c"
    srcfile map_fetch_ptr	"map_${typearr}_fetch_ptr.c"
    srcfile map_fetch_dup	"map_${typearr}_fetch_dup.c"
    srcfile map_fetch_copy	"map_${typearr}_fetch_copy.c"
    srcfile map_fetch_len	"map_${typearr}_fetch_len.c"

    return
}

#-----------------------------------------------------------------------------
# Generate code for each different scalar type.
#-----------------------------------------------------------------------------
code	c	"char"
code	sc	"signed char"
code	uc	"unsigned char"
code	ss	"signed short"
code	us	"unsigned short"
code	si	"signed int"
code	ui	"unsigned int"
code	sl	"signed long"
code	ul	"unsigned long"
code	sll	"signed long long"
code	ull	"unsigned long long"

exit 0
