//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// program	linesort
//
// purpose	Perform a simple sort of input lines via string mapping.
//
// syntax	linesort < infile
//-----------------------------------------------------------------------------
#include <stdio.h>

#include <libh/map.h>
#include <libh/io.h>

//-----------------------------------------------------------------------------
// function	main
//-----------------------------------------------------------------------------
int
main (
    int		argc
    ,
    char * *	argv
    )
{
    char		line_buf	[4096]	;
    MAP			line_map	;


    //--------------------
    // Create the mapping.
    //--------------------
    line_map = map_str_new();

    //----------------------------
    // Setup mapping sort options.
    //----------------------------
    map_str_cmp_lower( line_map, 1 );
    map_str_cmp_digits( line_map, 1 );
    map_str_cmp_spaces( line_map, 1 );
    map_str_cmp_paths( line_map, 1 );

    //-----------------------------------------------
    // Read each line and insert it into the mapping.
    //-----------------------------------------------
    while ( get_line_file( line_buf, sizeof (line_buf), stdin ) >= 0 ) {
	map_str_insert_dup( line_map, line_buf );
    }
    
    //----------------------------------------------
    // Write every line from the mapping.
    //----------------------------------------------
    map_loop_forward( line_map ) {
	puts( map_str_key_ptr( line_map ) );
    }

    //------
    // Poof.
    //------
    map_destroy( line_map );

    return 0;
}
