//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		map_data_clear.c
//-----------------------------------------------------------------------------

#include "map_lib.h"

#if 0
//-----------------------------------------------------------------------------
// struct	map_link
// struct	map_node
//
// purpose	Define structures needed to keep the AVL trees used to hold
//		the mapping.
//-----------------------------------------------------------------------------
struct map_link {
    avl_link		link	;
    // The key is found here in the type specific structs.
};
struct map_node {
    struct map_mixed	data	;
    struct map_link	link	;
};
#endif

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_data_clear
// alias	map_none_store
//
// purpose	Clear the data value of the active member of a mapping so
//		that it is just like it was newly inserted with no value.
//
// arguments	1 (MAP) mapping handle
//
// returns	(int) -2 = error, no map
//		(int) -1 = no current member to clear
//		(int)  0 = current member cleared OK
//-----------------------------------------------------------------------------
#define map_none_store(m) map_data_clear((m))

int
map_data_clear (
    MAP		arg_map
    )
__PROTO_END__
{
    struct map_node *	node_p	;

    //-- Make sure we have a mapping.
    if ( ! arg_map ) return -2;

    //-- Get the active node and check it.
    node_p = avl_find( & (arg_map->avl_tree), NULL, 0 );
    if ( ! node_p ) return -1;

    //-- Free the data space that was used.
    if ( map_type_is_string( node_p->data.type ) && node_p->data.mix.array.dup ) {
	free( node_p->data.mix.array.ptr );
    }

    //-- Mark the node has having no data.
    node_p->data.type = 0;

    //-- Status is OK.
    return 0;
}
