//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		map_destroy.c
//-----------------------------------------------------------------------------

#include "map_lib.h"

#if 0
//-----------------------------------------------------------------------------
// struct	map_link
// struct	map_node
//
// purpose	Define structures needed to keep the AVL trees used to hold
//		the mapping.
//-----------------------------------------------------------------------------
struct map_link {
    avl_link		link	;
    // The key is found here in the type specific structs.
};
struct map_node {
    struct map_mixed	data	;
    struct map_link	link	;
};
#endif

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_destroy
//
// purpose	Destroy a mapping.  Delete all the nodes and free the mapping.
//		The mapping handle becomes invalid and must not be used again.
//
// arguments	1 (MAP) mapping handle
//
// returns	(long) -2 = error, no map
//		(long) number of members deleted
//
// note		In UNIX and similar virtual memory systems, if the process
//		is near the end of its execution, it is likely not necessary
//		to free the remaining space, as virtual memory cleanup will
//		remove it more efficiently.
//-----------------------------------------------------------------------------
long
map_destroy (
    MAP		arg_map
    )
__PROTO_END__
{
    long	count	;

    //-- Make sure we have a mapping.
    if ( ! arg_map ) return -2;

    //-- Let the AVL layer empty the whole tree, returning the node count.
    count = avl_empty( & (arg_map->avl_tree), map_free_node );

    //-- Free the mapping itself.
    free( arg_map );

    //-- Now we can return the count.
    return count;
}
