//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// file		map_same_new.c
//-----------------------------------------------------------------------------

#include "map_lib.h"

#if 0
//-----------------------------------------------------------------------------
// struct	map_link
// struct	map_node
//
// purpose	Define structures needed to keep the AVL trees used to hold
//		the mapping.
//-----------------------------------------------------------------------------
struct map_link {
    avl_link		link	;
    // The key is found here in the type specific structs.
};
struct map_node {
    struct map_mixed	data	;
    struct map_link	link	;
};
#endif

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_same_new
//
// purpose	Make a new mapping with the same key type as another given
//		mapping.
//
// arguments	1 (MAP) mapping handle of guide mapping
//
// returns	(MAP) NULL: error
//		(MAP) handle of new mapping
//
// warning	This will cause the compare function from the given mapping to
//		also be used in the new mapping.  If the compare function is
//		stateful or was dynamically loaded and gets unloaded, results
//		are unpredictable, and likely disasterous.
//-----------------------------------------------------------------------------
MAP
map_same_new (
    MAP		arg_map
    )
__PROTO_END__
{
    MAP		this_map	;

    //-- Make sure there is a given mapping.
    if ( ! arg_map ) return NULL;

    //-- Allocate a structure for the root of the mapping.
    if ( ! ( this_map = malloc( sizeof (struct map_root) ) ) ) return NULL;

    //-- Replicate compare function, options, and key_type.
    this_map->cmp_func = arg_map->cmp_func;
    this_map->options = arg_map->options;
    this_map->key_type = arg_map->key_type;

    //-- Let the AVL layer initialize the rest of it like the given one.
    if ( ! avl_init_alike( & (this_map->avl_tree), & (arg_map->avl_tree) ) ) return NULL;

    //-- Return the handle to the new mapping.
    return this_map;
}
