//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

#include "map_lib.h"

//-----------------------------------------------------------------------------
// struct       map_link_ca
// struct       map_node_ca
//
// purpose      Define structures needed to keep the AVL trees used to hold
//              a mapping of this particular key type.
//
// note		The key array size is determined when node is created.
//-----------------------------------------------------------------------------
struct map_link_ca {
    avl_link		link    ;
    size_t		keylen	;
    char		key	[1];
};

struct map_node_ca {
    struct map_mixed		data    ;
    struct map_link_ca	link    ;
};

//-----------------------------------------------------------------------------
// function	map_ca_length (internal, inline)
//
// purpose	Determine the length of a string of this key or data type.
//
// arguments	1 (const char *) string pointer
//
// returns	(size_t) length of string
//-----------------------------------------------------------------------------
inline static
size_t
map_ca_length (
    const char *	arg_str
    )
{
    size_t	len	;
    len = 0;
    while ( * arg_str ++ != (char) 0 ) ++ len;
    return len;
}

//-----------------------------------------------------------------------------
// function	map_ca_copy_array (internal, inline)
//
// purpose	Copy a string of this key or data type, and terminate it.
//
// arguments	1 (char *) destination pointer
//		2 (const char *) source pointer
//		3 (size_t) length to copy
//
// returns	(void) nothing
//-----------------------------------------------------------------------------
inline static
void
map_ca_copy_array (
    char *		arg_dst
    ,
    const char *	arg_src
    ,
    size_t		arg_len
    )
{
    while ( arg_len -- ) * arg_dst ++ = * arg_src ++;
    * arg_dst = (char) 0;
    return;
}

