//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

#include "map_ca.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_str_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_str_cmp_func_opt map_ca_cmp_func_opt
__FMACRO_END__

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_cs_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
#define map_cs_cmp_func_opt map_ca_cmp_func_opt
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_ca_cmp_func_opt
//
// purpose	Set the comparison function to the caller supplied function
//		with a 3rd argument option included.
//
// arguments	1 (MAP) mapping handle
//		2 (int *)() alternate comparison function
//		3 (int) option to pass along
//
// comparison	1 (const char *) pointer to key 1
//  arguments	2 (size_t) length of key 1
//		3 (const char *) pointer to key 2
//		4 (size_t) length of key 2
//
// comparison	(int) -1: key 1 <  key 2
//    returns	(int)  0: key 1 == key 2
//		(int)  1: key 1	 > key 2
//
// returns	(int) -1 : error
//		(int)  0 : OK
//
// WARNING	This function will remove all members from the mapping since
//		the collating order is disrupted by this setting.
//-----------------------------------------------------------------------------
int
map_ca_cmp_func_opt (
    MAP		arg_map
    ,
    int		(* arg_compare)(const char *
				,
				size_t
				,
				const char *
				,
				size_t
				)
    ,
    int		arg_opt
   )
__PROTO_END__
{
    //-- Make sure the mapping is empty.
    map_empty( arg_map );

    //-- Set up for custom comparison.
    arg_map->options = arg_opt;
    arg_map->cmp_func = (int (*)()) arg_compare;

    //-- Re-initialize the AVL layer.
    if ( ! avl_init( & (arg_map->avl_tree),
		     struct map_node_ca,
		     link,
		     map_ca_compare_alt_opt
		    )
	) return -1;

    return 0;
}

