//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "map_f.h"

//-----------------------------------------------------------------------------
// function	map_f_compare (internal)
//
// purpose	Compare the scalar keys in two AVL tree nodes.  This function
//		is not globally exported.  It is used only to establish the
//		comparison method for the AVL tree used to hold this mapping.
//
// arguments	1 (struct map_link_f *)
//		2 (struct map_link_f *)
//		3 (void *) extra pointer (not used)
//
// returns	(int) comparison status, -1, 0, 1
//-----------------------------------------------------------------------------
static
int
map_f_compare (
    void *		arg_one
    ,
    void *		arg_two
    ,
    AVL *		arg_avl
    )
{
    return ( ((struct map_link_f *) arg_one)->key
	   < ((struct map_link_f *) arg_two)->key ) ? -1
	 : ( ((struct map_link_f *) arg_one)->key
	   > ((struct map_link_f *) arg_two)->key ) ? 1
	 : 0;
}

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_f_new
//
// purpose	Create a new mapping with the specific scalar key type.
//
// arguments	-none-
//
// returns	(MAP) handle for the new mapping.
//		(MAP) NULL = error creating new mapping
//-----------------------------------------------------------------------------
MAP
map_f_new ()
__PROTO_END__
{
    MAP		this_map	;

    //-- Allocate a structure for the root of the mapping.
    if ( ! ( this_map = malloc( sizeof (struct map_root) ) ) ) return NULL;

    //-- Record the key type.
    this_map->key_type = MAP_DATA_F;

    if ( ! avl_init( & (this_map->avl_tree), struct map_node_f, link, map_f_compare ) ) return NULL;

    //-- Return the handle to the new mapping.
    return this_map;
}

