//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

#include "map_sca.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_scs_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_scs_key_len map_sca_key_len
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_sca_key_len
//
// purpose	Get the length of a key value of the current member in the
//		mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(size_t) key length
//		(size_t) ~0U = error, no map, bad type, no member
//-----------------------------------------------------------------------------
size_t
map_sca_key_len (
    MAP			arg_map
    )
__PROTO_END__
{
    struct map_node_sca *	node_p	;

    //-- Make sure we have a correct mapping and current node.
    if ( ! arg_map ) return ~ (size_t) 0U;
    if ( arg_map->key_type != MAP_DATA_SCA ) return ~ (size_t) 0U;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return ~ (size_t) 0U;

    //-- Return the key length.
    return node_p->link.keylen;
}

