//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

#include "map_sia.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_sis_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
#define map_sis_insert(m,k) map_sia_insert((m),(k),~(size_t)0)
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_sia_insert
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value, unless one already exists
//		with that exact key, in which case it is made active.
//
// arguments	1 (MAP) mapping handle
//		2 (const signed int *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int)  0 = found or inserted a member with no data
//		(int) >0 = data type of found member
//-----------------------------------------------------------------------------
int
map_sia_insert (
    MAP			arg_map
    ,
    const signed int *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
__PROTO_END__
{
    struct map_node_sia *	node_p	;
    int			rc	;

    //-- Make sure we have a correct mapping.
    if ( ! arg_map ) return -2;
    if ( arg_map->key_type != MAP_DATA_SIA ) return -2;

    //-- If the length is not given, determine it as zero terminated string.
    if ( arg_key_len == ~ (size_t) 0U ) arg_key_len = map_sia_length( arg_key_ptr );

    //-- Allocate a combined node and key space.  The node already
    //-- has the space for one element of the key so it is not
    //-- necessary to add an extra one for the terminating element.
    if ( ! ( node_p = malloc( sizeof (struct map_node_sia) + sizeof (signed int) * arg_key_len ) ) ) return -2;

    //-- Initialize the node.
    node_p->data.type = 0;
    node_p->link.keylen = arg_key_len;

    //-- Copy the key.
    map_sia_copy_array( node_p->link.key, arg_key_ptr, arg_key_len );

    //-- Insert node or return data type if already exists.
    rc = avl_insert( & (arg_map->avl_tree), node_p );
    if ( rc >= 0 ) return 0;
    free( node_p );
    return map_sia_find( arg_map, arg_key_ptr, arg_key_len, 0 );
}

