//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "map_sll.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_sll_insert_dup
//
// purpose	Insert a new node into a mapping with the specified key and
//		an initial undefined data value.  If a node already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long) scalar key type
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique node inserted
//		(int)  2 = duplicate node inserted
//-----------------------------------------------------------------------------
int
map_sll_insert_dup (
    MAP			arg_map
    ,
    signed long long		arg_key
    )
__PROTO_END__
{
    struct map_node_sll *	node_p	;
    int			rc	;

    //-- Make sure we have a correct mapping.
    if ( ! arg_map ) return -2;
    if ( arg_map->key_type != MAP_DATA_SLL ) return -2;

    //-- Allocate a combined node and key space.  The key is a scalar
    //-- so there is always just one element and it is in the node.
    if ( ! ( node_p = malloc( sizeof (struct map_node_sll) ) ) ) return -2;

    //-- Initialize the node.
    node_p->data.type = 0;
    node_p->link.key = arg_key;

    //-- Insert the node.
    rc = avl_insert_dup( & (arg_map->avl_tree), node_p );

    //-- Return status based on insertion.
    return ( rc == -1 ) ? 0 : rc;
}

