//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "map_sll.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_sll_store
//
// purpose	Store the given data into the current node in the mapping.
//
// arguments	1 (MAP) mapping handle
//		2 (signed long long) data value to store
//
// return	(int) -2 = error, no map, no node
//		(int)  0 = success, data stored
//-----------------------------------------------------------------------------
int
map_sll_store (
    MAP			arg_map
    ,
    signed long long		arg_data
    )
__PROTO_END__
{
    struct map_node_sll *	node_p	;

    //-- Make sure we have a mapping and a current node.
    if ( ! arg_map ) return -2;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return -2;

    //-- Free any old duplicated data.
    if ( map_type_is_string( node_p->data.type ) && node_p->data.mix.array.dup ) {
	free( node_p->data.mix.array.ptr );
    }

    //-- Store the data type and data value.
    node_p->data.type = MAP_DATA_SLL;
    node_p->data.mix.scalar.sll = arg_data;

    //-- Return success.
    return 0;
}

