//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

#include "map_ssa.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_sss_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed short *) pointer to key duplicate copy
//		(signed short *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_sss_key_dup map_ssa_key_dup
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_ssa_key_dup
//
// purpose	Get the key of the current member in the mapping, returning an
//		allocated and duplicate copy.  The calling program must free
//		the allocated memory when use is concluded to avoid lossage
//		of memory.
//
// arguments	1 (MAP) mapping handle
//
// returns	(signed short *) pointer to key duplicate copy
//		(signed short *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
signed short *
map_ssa_key_dup (
    MAP			arg_map
    )
__PROTO_END__
{
    struct map_node_ssa *	node_p	;
    signed short *		key_p	;
    size_t		len	;

    //-- Make sure we have a correct mapping and current node.
    if ( ! arg_map ) return NULL;
    if ( arg_map->key_type != MAP_DATA_SSA ) return NULL;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return NULL;

    //-- Get key length.
    len = node_p->link.keylen;

    //-- Allocate space for key duplicate or return an error.
    if ( ! ( key_p = malloc( ( len + 1 ) * sizeof (signed short) ) ) ) return NULL;
    
    //-- Copy the key for duplication.
    map_ssa_copy_array( key_p, node_p->link.key, len );

    //-- Return the pointer to the duplicate.
    return key_p;
}

