//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

#include "map_uca.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_ucs_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned char *) pointer to duplicate data string array
//		(unsigned char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
#define map_ucs_fetch_dup map_uca_fetch_dup
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_uca_fetch_dup
//
// purpose	Fetch the data from the current member in the mapping.
//
// arguments	1 (MAP) mapping handle
//
// returns	(unsigned char *) pointer to duplicate data string array
//		(unsigned char *) NULL = error, no map, no member, bad type
//-----------------------------------------------------------------------------
unsigned char *
map_uca_fetch_dup (
    MAP			arg_map
   )
__PROTO_END__
{
    struct map_node_uca *	node_p	;
    unsigned char *		data_p	;
    size_t		len	;

    //-- Make sure we have a mapping and a current node.
    if ( ! arg_map ) return NULL;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return NULL;

    //-- Make sure data type is correct.
    if ( node_p->data.type != MAP_DATA_UCA ) return NULL;

    //-- Get data length.
    len = node_p->data.mix.array.len;

    //-- Allocate space for data duplicate or return an error.
    if ( ! ( data_p = malloc( ( len + 1 ) * sizeof (unsigned char) ) ) ) return NULL;

    //-- Copy the data for duplication.
    map_uca_copy_array( data_p, (unsigned char *) ( node_p->data.mix.array.ptr ), len );

    //-- Return the pointer to the duplicate.
    return data_p;
}

