//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

#include "map_ulla.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_ulls_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
#define map_ulls_insert_dup(m,k) map_ulla_insert_dup((m),(k),~(size_t)0)
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_ulla_insert_dup
//
// purpose	Insert a new member into a mapping with the specified key and
//		an initial undefined data value.  If a member already exists,
//		insert a duplicate.
//
// arguments	1 (MAP) mapping handle
//		2 (const unsigned long long *) key array pointer
//		3 (size_t) key length
//
// returns	(int) -2 = error, no map, bad type
//		(int) -1 = not inserted
//		(int)  1 = new unique member inserted
//		(int)  2 = duplicate member inserted
//-----------------------------------------------------------------------------
int
map_ulla_insert_dup (
    MAP			arg_map
    ,
    const unsigned long long *	arg_key_ptr
    ,
    size_t		arg_key_len
   )
__PROTO_END__
{
    struct map_node_ulla *	node_p	;
    size_t		len	;
    int			rc	;

    //-- Make sure we have a correct mapping.
    if ( ! arg_map ) return -2;
    if ( arg_map->key_type != MAP_DATA_ULLA ) return -2;

    //-- If the length is not given, determine it as zero terminated string.
    if ( arg_key_len == ~ (size_t) 0U ) arg_key_len = map_ulla_length( arg_key_ptr );

    //-- Allocate a combined node and key space.  The node already
    //-- has the space for one element of the key so it is not
    //-- necessary to add an extra one for the terminating element.
    len = ( sizeof (struct map_node_ulla) ) + ( arg_key_len * sizeof (unsigned long long) );
    if ( ! ( node_p = malloc( len ) ) ) return -2;

    //-- Initialize the node.
    node_p->data.type = 0;
    node_p->link.keylen = arg_key_len;

    //-- Copy the key.
    map_ulla_copy_array( node_p->link.key, arg_key_ptr, arg_key_len );

    //-- Insert the node.
    rc = avl_insert_dup( & (arg_map->avl_tree), node_p );

    //-- Return status based on insertion.
    if ( rc < 0 ) {
	free( node_p );
	return -1;
    }
    return rc;
}

