//-----------------------------------------------------------------------------
// Copyright  2009 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/map
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 112 columns wide.
//-----------------------------------------------------------------------------

#include "map_ulla.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	map_ulls_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned long long *) pointer to shared key
//		(const unsigned long long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
#define map_ulls_key_ptr map_ulla_key_ptr
__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	map_ulla_key_ptr
//
// purpose	Get the key of the current member in the mapping, returning the
//		pointer to the copy used in the AVL tree.  The calling program
//		must not alter the contents of the key to avoid corrupting the
//		AVL tree.
//
// arguments	1 (MAP) mapping handle
//
// returns	(const unsigned long long *) pointer to shared key
//		(const unsigned long long *) NULL = error, no map, bad type, no member
//-----------------------------------------------------------------------------
const unsigned long long *
map_ulla_key_ptr (
    MAP			arg_map
   )
__PROTO_END__
{
    struct map_node_ulla *	node_p	;

    //-- Make sure we have a correct mapping and current node.
    if ( ! arg_map ) return NULL;
    if ( arg_map->key_type != MAP_DATA_ULLA ) return NULL;
    if ( ! ( node_p = avl_find( & (arg_map->avl_tree), NULL, 0 ) ) ) return NULL;

    //-- Return the key pointer.
    return node_p->link.key;
}

