__PREFIX_BEGIN__
//---------------------------------------------------------------------------------------------------
// Copyright  2012 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//---------------------------------------------------------------------------------------------------
// package	libh/mstr
// homepage	http://libh.slashusr.org/
//---------------------------------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//---------------------------------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//---------------------------------------------------------------------------------------------------

//---------------------------------------------------------------------------------------------------
// file		libh/mstr.h
//---------------------------------------------------------------------------------------------------
#ifndef __MSTR_H__
#define __MSTR_H__

__PREFIX_END__

__INCLUDE_BEGIN__
#include "libh/avl.h"
#include "libh/mstr.h"

__INCLUDE_END__

__DEFINE_BEGIN__

//---------------------------------------------------------------------------------------------------
// struct	mstr_d_s
//
// purpose	Object for a mapping between multiple strings and signed long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_d_s {
    AVL			avl_str		[1]	;
    AVL			avl_num		[1]	;
};
typedef struct mstr_d_s			mstr_d_t	;
typedef struct mstr_d_s *		mstr_d_p	;

//---------------------------------------------------------------------------------------------------
// struct       mstr_d_node_s
//
// purpose	Node for a mapping between multiple strings and signed long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_d_node_s {
    avl_link		link_str	;
    avl_link		link_num	;
    double		num		;
    const char *	str	[1]	;
};
typedef struct mstr_d_node_s		mstr_d_node_t       ;
typedef struct mstr_d_node_s *		mstr_d_node_p       ;

#define mstr_d_num_reset(m) ((mstr_d_node_p)avl_unselect((m)->avl_num))
#define mstr_d_str_reset(m) ((mstr_d_node_p)avl_unselect((m)->avl_str))
#define mstr_d_num_first(m) ((mstr_d_node_p)avl_first((m)->avl_num))
#define mstr_d_str_first(m) ((mstr_d_node_p)avl_first((m)->avl_str))
#define mstr_d_num_last(m) ((mstr_d_node_p)avl_last((m)->avl_num))
#define mstr_d_str_last(m) ((mstr_d_node_p)avl_last((m)->avl_str))
#define mstr_d_num_forward(m) ((mstr_d_node_p)avl_forward((m)->avl_num))
#define mstr_d_str_forward(m) ((mstr_d_node_p)avl_forward((m)->avl_str))
#define mstr_d_num_reverse(m) ((mstr_d_node_p)avl_reverse((m)->avl_num))
#define mstr_d_str_reverse(m) ((mstr_d_node_p)avl_reverse((m)->avl_str))
#define mstr_d_num_fwd_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_d_node_p)avl_forward((t)->avl_num,NULL);)
#define mstr_d_str_fwd_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_d_node_p)avl_forward((t)->avl_str,NULL);)
#define mstr_d_num_rev_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_d_node_p)avl_reverse((t)->avl_num,NULL);)
#define mstr_d_str_rev_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_d_node_p)avl_reverse((t)->avl_str,NULL);)
#define mstr_d_num_fwd_while(t,n) while((n)=(mstr_d_node_p)avl_forward((t)->avl_num,NULL))
#define mstr_d_str_fwd_while(t,n) while((n)=(mstr_d_node_p)avl_forward((t)->avl_str,NULL))
#define mstr_d_num_rev_while(t,n) while((n)=(mstr_d_node_p)avl_reverse((t)->avl_num,NULL))
#define mstr_d_str_rev_while(t,n) while((n)=(mstr_d_node_p)avl_reverse((t)->avl_str,NULL))

//---------------------------------------------------------------------------------------------------
// struct	mstr_ld_s
//
// purpose	Object for a mapping between multiple strings and signed long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_ld_s {
    AVL			avl_str		[1]	;
    AVL			avl_num		[1]	;
};
typedef struct mstr_ld_s		mstr_ld_t	;
typedef struct mstr_ld_s *		mstr_ld_p	;

//---------------------------------------------------------------------------------------------------
// struct       mstr_ld_node_s
//
// purpose	Node for a mapping between multiple strings and signed long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_ld_node_s {
    avl_link		link_str	;
    avl_link		link_num	;
    long double		num		;
    const char *	str	[1]	;
};
typedef struct mstr_ld_node_s		mstr_ld_node_t       ;
typedef struct mstr_ld_node_s *		mstr_ld_node_p       ;

#define mstr_ld_num_reset(m) ((mstr_ld_node_p)avl_unselect((m)->avl_num))
#define mstr_ld_str_reset(m) ((mstr_ld_node_p)avl_unselect((m)->avl_str))
#define mstr_ld_num_first(m) ((mstr_ld_node_p)avl_first((m)->avl_num))
#define mstr_ld_str_first(m) ((mstr_ld_node_p)avl_first((m)->avl_str))
#define mstr_ld_num_last(m) ((mstr_ld_node_p)avl_last((m)->avl_num))
#define mstr_ld_str_last(m) ((mstr_ld_node_p)avl_last((m)->avl_str))
#define mstr_ld_num_forward(m) ((mstr_ld_node_p)avl_forward((m)->avl_num))
#define mstr_ld_str_forward(m) ((mstr_ld_node_p)avl_forward((m)->avl_str))
#define mstr_ld_num_reverse(m) ((mstr_ld_node_p)avl_reverse((m)->avl_num))
#define mstr_ld_str_reverse(m) ((mstr_ld_node_p)avl_reverse((m)->avl_str))
#define mstr_ld_num_fwd_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_ld_node_p)avl_forward((t)->avl_num,NULL);)
#define mstr_ld_str_fwd_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_ld_node_p)avl_forward((t)->avl_str,NULL);)
#define mstr_ld_num_rev_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_ld_node_p)avl_reverse((t)->avl_num,NULL);)
#define mstr_ld_str_rev_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_ld_node_p)avl_reverse((t)->avl_str,NULL);)
#define mstr_ld_num_fwd_while(t,n) while((n)=(mstr_ld_node_p)avl_forward((t)->avl_num,NULL))
#define mstr_ld_str_fwd_while(t,n) while((n)=(mstr_ld_node_p)avl_forward((t)->avl_str,NULL))
#define mstr_ld_num_rev_while(t,n) while((n)=(mstr_ld_node_p)avl_reverse((t)->avl_num,NULL))
#define mstr_ld_str_rev_while(t,n) while((n)=(mstr_ld_node_p)avl_reverse((t)->avl_str,NULL))

//---------------------------------------------------------------------------------------------------
// struct	mstr_si_s
//
// purpose	Object for a mapping between multiple strings and signed int numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_si_s {
    AVL			avl_str		[1]	;
    AVL			avl_num		[1]	;
};
typedef struct mstr_si_s		mstr_si_t	;
typedef struct mstr_si_s *		mstr_si_p	;

//---------------------------------------------------------------------------------------------------
// struct       mstr_si_node_s
//
// purpose	Node for a mapping between multiple strings and signed int numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_si_node_s {
    avl_link		link_str	;
    avl_link		link_num	;
    signed int		num		;
    const char *	str	[1]	;
};
typedef struct mstr_si_node_s		mstr_si_node_t       ;
typedef struct mstr_si_node_s *		mstr_si_node_p       ;

#define mstr_si_num_reset(m) ((mstr_si_node_p)avl_unselect((m)->avl_num))
#define mstr_si_str_reset(m) ((mstr_si_node_p)avl_unselect((m)->avl_str))
#define mstr_si_num_first(m) ((mstr_si_node_p)avl_first((m)->avl_num))
#define mstr_si_str_first(m) ((mstr_si_node_p)avl_first((m)->avl_str))
#define mstr_si_num_last(m) ((mstr_si_node_p)avl_last((m)->avl_num))
#define mstr_si_str_last(m) ((mstr_si_node_p)avl_last((m)->avl_str))
#define mstr_si_num_forward(m) ((mstr_si_node_p)avl_forward((m)->avl_num))
#define mstr_si_str_forward(m) ((mstr_si_node_p)avl_forward((m)->avl_str))
#define mstr_si_num_reverse(m) ((mstr_si_node_p)avl_reverse((m)->avl_num))
#define mstr_si_str_reverse(m) ((mstr_si_node_p)avl_reverse((m)->avl_str))
#define mstr_si_num_fwd_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_si_node_p)avl_forward((t)->avl_num,NULL);)
#define mstr_si_str_fwd_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_si_node_p)avl_forward((t)->avl_str,NULL);)
#define mstr_si_num_rev_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_si_node_p)avl_reverse((t)->avl_num,NULL);)
#define mstr_si_str_rev_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_si_node_p)avl_reverse((t)->avl_str,NULL);)
#define mstr_si_num_fwd_while(t,n) while((n)=(mstr_si_node_p)avl_forward((t)->avl_num,NULL))
#define mstr_si_str_fwd_while(t,n) while((n)=(mstr_si_node_p)avl_forward((t)->avl_str,NULL))
#define mstr_si_num_rev_while(t,n) while((n)=(mstr_si_node_p)avl_reverse((t)->avl_num,NULL))
#define mstr_si_str_rev_while(t,n) while((n)=(mstr_si_node_p)avl_reverse((t)->avl_str,NULL))

//---------------------------------------------------------------------------------------------------
// struct	mstr_sl_s
//
// purpose	Object for a mapping between multiple strings and signed long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_sl_s {
    AVL			avl_str		[1]	;
    AVL			avl_num		[1]	;
};
typedef struct mstr_sl_s		mstr_sl_t	;
typedef struct mstr_sl_s *		mstr_sl_p	;

//---------------------------------------------------------------------------------------------------
// struct       mstr_sl_node_s
//
// purpose	Node for a mapping between multiple strings and signed long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_sl_node_s {
    avl_link		link_str	;
    avl_link		link_num	;
    signed long		num		;
    const char *	str	[1]	;
};
typedef struct mstr_sl_node_s		mstr_sl_node_t       ;
typedef struct mstr_sl_node_s *		mstr_sl_node_p       ;

#define mstr_sl_num_reset(m) ((mstr_sl_node_p)avl_unselect((m)->avl_num))
#define mstr_sl_str_reset(m) ((mstr_sl_node_p)avl_unselect((m)->avl_str))
#define mstr_sl_num_first(m) ((mstr_sl_node_p)avl_first((m)->avl_num))
#define mstr_sl_str_first(m) ((mstr_sl_node_p)avl_first((m)->avl_str))
#define mstr_sl_num_last(m) ((mstr_sl_node_p)avl_last((m)->avl_num))
#define mstr_sl_str_last(m) ((mstr_sl_node_p)avl_last((m)->avl_str))
#define mstr_sl_num_forward(m) ((mstr_sl_node_p)avl_forward((m)->avl_num))
#define mstr_sl_str_forward(m) ((mstr_sl_node_p)avl_forward((m)->avl_str))
#define mstr_sl_num_reverse(m) ((mstr_sl_node_p)avl_reverse((m)->avl_num))
#define mstr_sl_str_reverse(m) ((mstr_sl_node_p)avl_reverse((m)->avl_str))
#define mstr_sl_num_fwd_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_sl_node_p)avl_forward((t)->avl_num,NULL);)
#define mstr_sl_str_fwd_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_sl_node_p)avl_forward((t)->avl_str,NULL);)
#define mstr_sl_num_rev_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_sl_node_p)avl_reverse((t)->avl_num,NULL);)
#define mstr_sl_str_rev_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_sl_node_p)avl_reverse((t)->avl_str,NULL);)
#define mstr_sl_num_fwd_while(t,n) while((n)=(mstr_sl_node_p)avl_forward((t)->avl_num,NULL))
#define mstr_sl_str_fwd_while(t,n) while((n)=(mstr_sl_node_p)avl_forward((t)->avl_str,NULL))
#define mstr_sl_num_rev_while(t,n) while((n)=(mstr_sl_node_p)avl_reverse((t)->avl_num,NULL))
#define mstr_sl_str_rev_while(t,n) while((n)=(mstr_sl_node_p)avl_reverse((t)->avl_str,NULL))

//---------------------------------------------------------------------------------------------------
// struct	mstr_sll_s
//
// purpose	Object for a mapping between multiple strings and signed long long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_sll_s {
    AVL			avl_str		[1]	;
    AVL			avl_num		[1]	;
};
typedef struct mstr_sll_s		mstr_sll_t	;
typedef struct mstr_sll_s *		mstr_sll_p	;

//---------------------------------------------------------------------------------------------------
// struct       mstr_sll_node_s
//
// purpose	Node for a mapping between multiple strings and signed long long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_sll_node_s {
    avl_link		link_str	;
    avl_link		link_num	;
    signed long long	num		;
    const char *	str	[1]	;
};
typedef struct mstr_sll_node_s		mstr_sll_node_t       ;
typedef struct mstr_sll_node_s *	mstr_sll_node_p       ;

#define mstr_sll_num_reset(m) ((mstr_sll_node_p)avl_unselect((m)->avl_num))
#define mstr_sll_str_reset(m) ((mstr_sll_node_p)avl_unselect((m)->avl_str))
#define mstr_sll_num_first(m) ((mstr_sll_node_p)avl_first((m)->avl_num))
#define mstr_sll_str_first(m) ((mstr_sll_node_p)avl_first((m)->avl_str))
#define mstr_sll_num_last(m) ((mstr_sll_node_p)avl_last((m)->avl_num))
#define mstr_sll_str_last(m) ((mstr_sll_node_p)avl_last((m)->avl_str))
#define mstr_sll_num_forward(m) ((mstr_sll_node_p)avl_forward((m)->avl_num))
#define mstr_sll_str_forward(m) ((mstr_sll_node_p)avl_forward((m)->avl_str))
#define mstr_sll_num_reverse(m) ((mstr_sll_node_p)avl_reverse((m)->avl_num))
#define mstr_sll_str_reverse(m) ((mstr_sll_node_p)avl_reverse((m)->avl_str))
#define mstr_sll_num_fwd_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_sll_node_p)avl_forward((t)->avl_num,NULL);)
#define mstr_sll_str_fwd_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_sll_node_p)avl_forward((t)->avl_str,NULL);)
#define mstr_sll_num_rev_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_sll_node_p)avl_reverse((t)->avl_num,NULL);)
#define mstr_sll_str_rev_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_sll_node_p)avl_reverse((t)->avl_str,NULL);)
#define mstr_sll_num_fwd_while(t,n) while((n)=(mstr_sll_node_p)avl_forward((t)->avl_num,NULL))
#define mstr_sll_str_fwd_while(t,n) while((n)=(mstr_sll_node_p)avl_forward((t)->avl_str,NULL))
#define mstr_sll_num_rev_while(t,n) while((n)=(mstr_sll_node_p)avl_reverse((t)->avl_num,NULL))
#define mstr_sll_str_rev_while(t,n) while((n)=(mstr_sll_node_p)avl_reverse((t)->avl_str,NULL))

//---------------------------------------------------------------------------------------------------
// struct	mstr_ui_s
//
// purpose	Object for a mapping between multiple strings and unsigned int numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_ui_s {
    AVL			avl_str		[1]	;
    AVL			avl_num		[1]	;
};
typedef struct mstr_ui_s		mstr_ui_t	;
typedef struct mstr_ui_s *		mstr_ui_p	;

//---------------------------------------------------------------------------------------------------
// struct       mstr_ui_node_s
//
// purpose	Node for a mapping between multiple strings and unsigned int numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_ui_node_s {
    avl_link		link_str	;
    avl_link		link_num	;
    unsigned int	num		;
    const char *	str	[1]	;
};
typedef struct mstr_ui_node_s		mstr_ui_node_t       ;
typedef struct mstr_ui_node_s *		mstr_ui_node_p       ;

#define mstr_ui_num_reset(m) ((mstr_ui_node_p)avl_unselect((m)->avl_num))
#define mstr_ui_str_reset(m) ((mstr_ui_node_p)avl_unselect((m)->avl_str))
#define mstr_ui_num_first(m) ((mstr_ui_node_p)avl_first((m)->avl_num))
#define mstr_ui_str_first(m) ((mstr_ui_node_p)avl_first((m)->avl_str))
#define mstr_ui_num_last(m) ((mstr_ui_node_p)avl_last((m)->avl_num))
#define mstr_ui_str_last(m) ((mstr_ui_node_p)avl_last((m)->avl_str))
#define mstr_ui_num_forward(m) ((mstr_ui_node_p)avl_forward((m)->avl_num))
#define mstr_ui_str_forward(m) ((mstr_ui_node_p)avl_forward((m)->avl_str))
#define mstr_ui_num_reverse(m) ((mstr_ui_node_p)avl_reverse((m)->avl_num))
#define mstr_ui_str_reverse(m) ((mstr_ui_node_p)avl_reverse((m)->avl_str))
#define mstr_ui_num_fwd_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_ui_node_p)avl_forward((t)->avl_num,NULL);)
#define mstr_ui_str_fwd_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_ui_node_p)avl_forward((t)->avl_str,NULL);)
#define mstr_ui_num_rev_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_ui_node_p)avl_reverse((t)->avl_num,NULL);)
#define mstr_ui_str_rev_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_ui_node_p)avl_reverse((t)->avl_str,NULL);)
#define mstr_ui_num_fwd_while(t,n) while((n)=(mstr_ui_node_p)avl_forward((t)->avl_num,NULL))
#define mstr_ui_str_fwd_while(t,n) while((n)=(mstr_ui_node_p)avl_forward((t)->avl_str,NULL))
#define mstr_ui_num_rev_while(t,n) while((n)=(mstr_ui_node_p)avl_reverse((t)->avl_num,NULL))
#define mstr_ui_str_rev_while(t,n) while((n)=(mstr_ui_node_p)avl_reverse((t)->avl_str,NULL))

//---------------------------------------------------------------------------------------------------
// struct	mstr_ul_s
//
// purpose	Object for a mapping between multiple strings and unsigned long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_ul_s {
    AVL			avl_str		[1]	;
    AVL			avl_num		[1]	;
};
typedef struct mstr_ul_s		mstr_ul_t	;
typedef struct mstr_ul_s *		mstr_ul_p	;

//---------------------------------------------------------------------------------------------------
// struct       mstr_ul_node_s
//
// purpose	Node for a mapping between multiple strings and unsigned long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_ul_node_s {
    avl_link		link_str	;
    avl_link		link_num	;
    unsigned long	num		;
    const char *	str	[1]	;
};
typedef struct mstr_ul_node_s		mstr_ul_node_t       ;
typedef struct mstr_ul_node_s *		mstr_ul_node_p       ;

#define mstr_ul_num_reset(m) ((mstr_ul_node_p)avl_unselect((m)->avl_num))
#define mstr_ul_str_reset(m) ((mstr_ul_node_p)avl_unselect((m)->avl_str))
#define mstr_ul_num_first(m) ((mstr_ul_node_p)avl_first((m)->avl_num))
#define mstr_ul_str_first(m) ((mstr_ul_node_p)avl_first((m)->avl_str))
#define mstr_ul_num_last(m) ((mstr_ul_node_p)avl_last((m)->avl_num))
#define mstr_ul_str_last(m) ((mstr_ul_node_p)avl_last((m)->avl_str))
#define mstr_ul_num_forward(m) ((mstr_ul_node_p)avl_forward((m)->avl_num))
#define mstr_ul_str_forward(m) ((mstr_ul_node_p)avl_forward((m)->avl_str))
#define mstr_ul_num_reverse(m) ((mstr_ul_node_p)avl_reverse((m)->avl_num))
#define mstr_ul_str_reverse(m) ((mstr_ul_node_p)avl_reverse((m)->avl_str))
#define mstr_ul_num_fwd_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_ul_node_p)avl_forward((t)->avl_num,NULL);)
#define mstr_ul_str_fwd_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_ul_node_p)avl_forward((t)->avl_str,NULL);)
#define mstr_ul_num_rev_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_ul_node_p)avl_reverse((t)->avl_num,NULL);)
#define mstr_ul_str_rev_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_ul_node_p)avl_reverse((t)->avl_str,NULL);)
#define mstr_ul_num_fwd_while(t,n) while((n)=(mstr_ul_node_p)avl_forward((t)->avl_num,NULL))
#define mstr_ul_str_fwd_while(t,n) while((n)=(mstr_ul_node_p)avl_forward((t)->avl_str,NULL))
#define mstr_ul_num_rev_while(t,n) while((n)=(mstr_ul_node_p)avl_reverse((t)->avl_num,NULL))
#define mstr_ul_str_rev_while(t,n) while((n)=(mstr_ul_node_p)avl_reverse((t)->avl_str,NULL))

//---------------------------------------------------------------------------------------------------
// struct	mstr_ull_s
//
// purpose	Object for a mapping between multiple strings and unsigned long long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_ull_s {
    AVL			avl_str		[1]	;
    AVL			avl_num		[1]	;
};
typedef struct mstr_ull_s		mstr_ull_t	;
typedef struct mstr_ull_s *		mstr_ull_p	;

//---------------------------------------------------------------------------------------------------
// struct       mstr_ull_node_s
//
// purpose	Node for a mapping between multiple strings and unsigned long long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_ull_node_s {
    avl_link		link_str	;
    avl_link		link_num	;
    unsigned long long	num		;
    const char *	str	[1]	;
};
typedef struct mstr_ull_node_s		mstr_ull_node_t       ;
typedef struct mstr_ull_node_s *	mstr_ull_node_p       ;

#define mstr_ull_num_reset(m) ((mstr_ull_node_p)avl_unselect((m)->avl_num))
#define mstr_ull_str_reset(m) ((mstr_ull_node_p)avl_unselect((m)->avl_str))
#define mstr_ull_num_first(m) ((mstr_ull_node_p)avl_first((m)->avl_num))
#define mstr_ull_str_first(m) ((mstr_ull_node_p)avl_first((m)->avl_str))
#define mstr_ull_num_last(m) ((mstr_ull_node_p)avl_last((m)->avl_num))
#define mstr_ull_str_last(m) ((mstr_ull_node_p)avl_last((m)->avl_str))
#define mstr_ull_num_forward(m) ((mstr_ull_node_p)avl_forward((m)->avl_num))
#define mstr_ull_str_forward(m) ((mstr_ull_node_p)avl_forward((m)->avl_str))
#define mstr_ull_num_reverse(m) ((mstr_ull_node_p)avl_reverse((m)->avl_num))
#define mstr_ull_str_reverse(m) ((mstr_ull_node_p)avl_reverse((m)->avl_str))
#define mstr_ull_num_fwd_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_ull_node_p)avl_forward((t)->avl_num,NULL);)
#define mstr_ull_str_fwd_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_ull_node_p)avl_forward((t)->avl_str,NULL);)
#define mstr_ull_num_rev_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_ull_node_p)avl_reverse((t)->avl_num,NULL);)
#define mstr_ull_str_rev_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_ull_node_p)avl_reverse((t)->avl_str,NULL);)
#define mstr_ull_num_fwd_while(t,n) while((n)=(mstr_ull_node_p)avl_forward((t)->avl_num,NULL))
#define mstr_ull_str_fwd_while(t,n) while((n)=(mstr_ull_node_p)avl_forward((t)->avl_str,NULL))
#define mstr_ull_num_rev_while(t,n) while((n)=(mstr_ull_node_p)avl_reverse((t)->avl_num,NULL))
#define mstr_ull_str_rev_while(t,n) while((n)=(mstr_ull_node_p)avl_reverse((t)->avl_str,NULL))

//---------------------------------------------------------------------------------------------------
// struct	mstr_ptr_s
//
// purpose	Object for a mapping between multiple strings and unsigned long long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_ptr_s {
    AVL			avl_str		[1]	;
    AVL			avl_ptr		[1]	;
};
typedef struct mstr_ptr_s		mstr_ptr_t	;
typedef struct mstr_ptr_s *		mstr_ptr_p	;

//---------------------------------------------------------------------------------------------------
// struct       mstr_ptr_node_s
//
// purpose	Node for a mapping between multiple strings and unsigned long long numbers.
//---------------------------------------------------------------------------------------------------
struct mstr_ptr_node_s {
    avl_link		link_str	;
    avl_link		link_ptr	;
    void *		ptr		;
    const char *	str	[1]	;
};
typedef struct mstr_ptr_node_s		mstr_ptr_node_t       ;
typedef struct mstr_ptr_node_s *	mstr_ptr_node_p       ;

#define mstr_ptr_num_reset(m) ((mstr_ptr_node_p)avl_unselect((m)->avl_num))
#define mstr_ptr_str_reset(m) ((mstr_ptr_node_p)avl_unselect((m)->avl_str))
#define mstr_ptr_num_first(m) ((mstr_ptr_node_p)avl_first((m)->avl_num))
#define mstr_ptr_str_first(m) ((mstr_ptr_node_p)avl_first((m)->avl_str))
#define mstr_ptr_num_last(m) ((mstr_ptr_node_p)avl_last((m)->avl_num))
#define mstr_ptr_str_last(m) ((mstr_ptr_node_p)avl_last((m)->avl_str))
#define mstr_ptr_num_forward(m) ((mstr_ptr_node_p)avl_forward((m)->avl_num))
#define mstr_ptr_str_forward(m) ((mstr_ptr_node_p)avl_forward((m)->avl_str))
#define mstr_ptr_num_reverse(m) ((mstr_ptr_node_p)avl_reverse((m)->avl_num))
#define mstr_ptr_str_reverse(m) ((mstr_ptr_node_p)avl_reverse((m)->avl_str))
#define mstr_ptr_num_fwd_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_ptr_node_p)avl_forward((t)->avl_num,NULL);)
#define mstr_ptr_str_fwd_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_ptr_node_p)avl_forward((t)->avl_str,NULL);)
#define mstr_ptr_num_rev_for(t,n) for(avl_unselect((t)->avl_num);(n)=(mstr_ptr_node_p)avl_reverse((t)->avl_num,NULL);)
#define mstr_ptr_str_rev_for(t,n) for(avl_unselect((t)->avl_str);(n)=(mstr_ptr_node_p)avl_reverse((t)->avl_str,NULL);)
#define mstr_ptr_num_fwd_while(t,n) while((n)=(mstr_ptr_node_p)avl_forward((t)->avl_num,NULL))
#define mstr_ptr_str_fwd_while(t,n) while((n)=(mstr_ptr_node_p)avl_forward((t)->avl_str,NULL))
#define mstr_ptr_num_rev_while(t,n) while((n)=(mstr_ptr_node_p)avl_reverse((t)->avl_num,NULL))
#define mstr_ptr_str_rev_while(t,n) while((n)=(mstr_ptr_node_p)avl_reverse((t)->avl_str,NULL))

__DEFINE_END__

__SUFFIX_BEGIN__
#endif /* __MSTR_H__ */
__SUFFIX_END__
