//---------------------------------------------------------------------------------------------------
// Copyright  2012 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//---------------------------------------------------------------------------------------------------
// package	libh/mstr_ld
// homepage	http://libh.slashusr.org/
//---------------------------------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//---------------------------------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//---------------------------------------------------------------------------------------------------

#include "mstr_lib.h"

__PROTO_BEGIN__
//---------------------------------------------------------------------------------------------------
// function	mstr_ld_sub
//
// purpose	Subtract a value from the numeric value indexed by multiple strings.
//
// arguments	1 (mstr_ld_p) pointer to mapping
//		2 (long double) number value to be subtracted
//		3... (const char *) one or more strings for index
//
// returns	(int) == 0 : OK, node found, value subtracted
//		(int) == 1 : OK, node added, value stored
//		(int)  < 0 : ERROR
//---------------------------------------------------------------------------------------------------
// macros	mstr_ld_dec
//
// purpose	Deccrement the numeric value indexed by multiple strings to zero or one.
//
// arguments	1 (mstr_ld_p) pointer to mapping
//		2... (const char *) one or more strings for index
//
// returns	(int) == 0 : OK, node found, value decremented
//		(int) == 1 : OK, node added, value stored
//		(int)  < 0 : ERROR
//---------------------------------------------------------------------------------------------------

//---------------------------------------------------------
// Define call macros to append a NULL at end of arguments.
//---------------------------------------------------------
#define mstr_ld_dec(t,s...)	((mstr_ld_sub)((t),(1.0L),s,NULL))
#define mstr_ld_sub(t,n,s...)	((mstr_ld_sub)((t),(n),s,NULL))

int
( mstr_ld_sub )( mstr_ld_p map_p, long double number, ... )
__PROTO_END__
{
    mstr_ld_node_t *	node_ptr	;
    mstr_ld_node_t *	node_tmp	;
    const char * *      str_ptr_ptr     ;
    const char *        str_ptr         ;
    size_t              node_size       ;
    va_list		var_arg		;

    //--------------------------------------
    // Make sure we actually have a mapping.
    //--------------------------------------
    if ( ! map_p ) return -1;

    //---------------------------------------------
    // Determine full node size to include strings.
    //---------------------------------------------
    node_size = sizeof (mstr_ld_node_t);
    va_start( var_arg, number );
    while ( ( str_ptr = va_arg( var_arg, const char * ) ) ) node_size += sizeof (const char *);
    va_end( var_arg );

    //----------------------------------
    // Build search argument dummy node.
    //----------------------------------
    node_tmp = alloca( node_size );
    if ( ! node_tmp ) return -2;
    str_ptr_ptr = node_tmp->str;
    va_start( var_arg, number );
    while ( ( str_ptr = va_arg( var_arg, const char * ) ) ) {
	if ( ! ( str_ptr = keepstr( str_ptr ) ) ) return -3;
	* str_ptr_ptr ++ = str_ptr;
    }
    * str_ptr_ptr ++ = str_ptr;
    va_end( var_arg );

    //---------------------------------------------
    // Search for existing node with these strings.
    //---------------------------------------------
    node_ptr = avl_find_exact( map_p->avl_str, node_tmp );

    //-----------------------------------------------------------------------
    // If node is found, remove it from number space, change number in place.
    //-----------------------------------------------------------------------
    if ( node_ptr ) {
        avl_delete( map_p->avl_num, node_ptr );
        node_ptr->num -= number;
	avl_insert_dup( map_p->avl_num, node_ptr ); 
	return 0;
    }

    //------------------------------------------------------------
    // Else if not found, build a real node, fill it in, index it.
    //------------------------------------------------------------
    node_ptr = malloc( node_size );
    if ( ! node_ptr ) return -4;
    node_ptr->num = number;
    memcpy( node_ptr->str, node_tmp->str, node_size - offsetof( mstr_ld_node_t, str ) );
    avl_insert( map_p->avl_str, node_ptr );
    avl_insert_dup( map_p->avl_num, node_ptr ); 
    return 1;
}
