//---------------------------------------------------------------------------------------------------
// Copyright  2012 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//---------------------------------------------------------------------------------------------------
// package	libh/mstr_ptr
// homepage	http://libh.slashusr.org/
//---------------------------------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//---------------------------------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//---------------------------------------------------------------------------------------------------

#include "mstr_lib.h"

__PROTO_BEGIN__
//---------------------------------------------------------------------------------------------------
// function	mstr_ptr_del
//
// purpose	Delete a numeric value indexed by multiple strings.
//
// arguments	1 (mstr_ptr_p) pointer to mapping
//		2... (const char *) one or more strings for index
//
// returns	(int) == 0 : OK, node not found, nothing to delete
//		(int) == 1 : OK, node found and deleted.
//		(int)  < 0 : ERROR
//---------------------------------------------------------------------------------------------------

//---------------------------------------------------------
// Define call macros to append a NULL at end of arguments.
//---------------------------------------------------------
#define mstr_ptr_del(t,s...)	((mstr_ptr_del)((t),s,NULL))

int
( mstr_ptr_del )( mstr_ptr_p map_p, unsigned long long number, ... )
__PROTO_END__
{
    mstr_ptr_node_t *	node_ptr	;
    const char * *      str_ptr_ptr     ;
    const char *        str_ptr         ;
    size_t              node_size       ;
    va_list		var_arg		;

    //--------------------------------------
    // Make sure we actually have a mapping.
    //--------------------------------------
    if ( ! map_p ) return -1;

    //---------------------------------------------
    // Determine full node size to include strings.
    //---------------------------------------------
    node_size = sizeof (mstr_ptr_node_t);
    va_start( var_arg, number );
    while ( ( str_ptr = va_arg( var_arg, const char * ) ) ) node_size += sizeof (const char *);
    va_end( var_arg );

    //----------------------------------
    // Build search argument dummy node.
    //----------------------------------
    node_ptr = alloca( node_size );
    if ( ! node_ptr ) return -2;
    str_ptr_ptr = node_ptr->str;
    va_start( var_arg, number );
    while ( ( str_ptr = va_arg( var_arg, const char * ) ) ) {
	if ( ! ( str_ptr = keepstr( str_ptr ) ) ) return -3;
	* str_ptr_ptr ++ = str_ptr;
    }
    * str_ptr_ptr ++ = str_ptr;
    va_end( var_arg );

    //---------------------------------------------
    // Search for existing node with these strings.
    //---------------------------------------------
    node_ptr = avl_find_exact( map_p->avl_str, node_ptr );

    //-----------------------------------------
    // If node is not found, nothing to remove.
    //-----------------------------------------
    if ( ! node_ptr ) return 0;

    //----------------------------------------------------
    // If node is found, remove it from both index spaces.
    //----------------------------------------------------
    avl_delete( map_p->avl_ptr, node_ptr );
    avl_delete( map_p->avl_str, node_ptr );
    free( node_ptr );
    return 1;
}
