//---------------------------------------------------------------------------------------------------
// Copyright  2012 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//---------------------------------------------------------------------------------------------------
// package	libh/mstr_ptr
// homepage	http://libh.slashusr.org/
//---------------------------------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//---------------------------------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//---------------------------------------------------------------------------------------------------

#include "mstr_lib.h"

//---------------------------------------------------------------------------------------------------
// function	mstr_ptr_cmps (private)
//
// purpose	Compare two nodes of a mapping by comparing the string tuple of each.
//		The unequal string pairing in the tuples defines the ordering of the nodes.
//		If one tuple is shorter than the other at unequal, it is the lower one.
//
// arguments	1 (const struct mstr_ptr_node_s *) pointer to node 1 to access key string tuple
//		2 (const struct mstr_ptr_node_s *) pointer to node 2 to access key string tuple
//		3 (cost AVL *) pointer to AVL tree root (not needed for mstr_ptr)
//
// returns	(int) state (-1, 0, 1) of string tuple comparison.
//---------------------------------------------------------------------------------------------------
static int
mstr_ptr_cmps( mstr_ptr_node_p node1_p, mstr_ptr_node_p node2_p, AVL *tree_p ) {
    const char * *	str1_p	;
    const char * *	str2_p	;

    str1_p = node1_p->str;
    str2_p = node2_p->str;
    while ( * str1_p && * str2_p ) {
	int cmp;
	cmp = strcmp( * str1_p, * str2_p );
	if ( cmp != 0 ) return cmp;
	++ str1_p;
	++ str2_p;
    }
    return ( * str1_p ) ? 1 : ( * str2_p ) ? -1 : 0;
}

//---------------------------------------------------------------------------------------------------
// function	mstr_ptr_cmpn (private)
//
// purpose	Compare two nodes of a mapping by comparing the pointer.
//		It is valid for two or more nodes to have the same pointer.
//		If the pointer is equal, compare the string tuple via mstr_ptr_cmps.
//
// arguments	1 (const struct mstr_ptr_node_s *) pointer to node 1 to access key string tuple
//		2 (const struct mstr_ptr_node_s *) pointer to node 2 to access key string tuple
//		3 (cost AVL *) pointer to AVL tree root (not needed for mstr_ptr)
//
// returns	(int) state (-1, 0, 1) of pointer comparison.
//---------------------------------------------------------------------------------------------------
static int
mstr_ptr_cmpn( mstr_ptr_node_p node1_p, mstr_ptr_node_p node2_p, AVL *tree_p ) {
    if ( node1_p->ptr < node1_p->ptr ) return -1;
    if ( node1_p->ptr > node1_p->ptr ) return 1;
    return mstr_ptr_cmps( node1_p, node2_p, tree_p );
}

__PROTO_BEGIN__
//---------------------------------------------------------------------------------------------------
// function	mstr_ptr_init
//
// purpose	Initialize a mapping of string tuple indexed pointers
//
// arguments	(mstr_ptr_p) pointer to mapping object to be initialized.
//---------------------------------------------------------------------------------------------------
int
mstr_ptr_init( mstr_ptr_p map_p )
__PROTO_END__
{
    if ( ! map_p ) return -1;
    avl_init( map_p->avl_str, mstr_ptr_node_t, link_str, mstr_ptr_cmps );
    avl_init( map_p->avl_ptr, mstr_ptr_node_t, link_ptr, mstr_ptr_cmpn );
    return 0;
}
