//---------------------------------------------------------------------------------------------------
// Copyright  2012 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//---------------------------------------------------------------------------------------------------
// package	libh/mstr_ui
// homepage	http://libh.slashusr.org/
//---------------------------------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//---------------------------------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//---------------------------------------------------------------------------------------------------

#include "mstr_lib.h"

__PROTO_BEGIN__
//---------------------------------------------------------------------------------------------------
// function	mstr_ui_get
//
// purpose	Get the numeric value indexed by multiple strings.
//
// arguments	1 (mstr_ui_p) pointer to mapping to be set
//		2 (unsigned int *) where to store numeric value
//		3... (const char *) one or more strings for index
//
// returns	(int) == 0 : value is not present (so not stored)
//		(int) == 1 : value is present and stored (if pointer is not null)
//		(int)  < 0 : ERROR
//---------------------------------------------------------------------------------------------------
// macros	mstr_ui_test
//
// purpose	Test if multiple strings are indexed.
//
// arguments	1 (mstr_ui_p) pointer to mapping to be set
//		2... (const char *) one or more strings for index
//
// returns	(int) == 0 : value is not present
//		(int) == 1 : value is present
//		(int)  < 0 : ERROR
//---------------------------------------------------------------------------------------------------

//---------------------------------------------------------
// Define call macros to append a NULL at end of arguments.
//---------------------------------------------------------
#define mstr_ui_test(t,s...)	((mstr_ui_get)((t),NULL,s,NULL))
#define mstr_ui_get(t,p,s...)	((mstr_ui_get)((t),(p),s,NULL))

int
( mstr_ui_get )( mstr_ui_p map_p, unsigned int *num_p, ... )
__PROTO_END__
{
    mstr_ui_node_t *	node_ptr	;
    mstr_ui_node_t *	node_tmp	;
    const char * *      str_ptr_ptr     ;
    const char *        str_ptr         ;
    size_t              node_size       ;
    va_list		var_arg		;

    //--------------------------------------
    // Make sure we actually have a mapping.
    //--------------------------------------
    if ( ! map_p ) return -1;

    //---------------------------------------------
    // Determine full node size to include strings.
    //---------------------------------------------
    node_size = sizeof (mstr_ui_node_t);
    va_start( var_arg, num_p );
    while ( ( str_ptr = va_arg( var_arg, const char * ) ) ) node_size += sizeof (const char *);
    va_end( var_arg );

    //----------------------------------
    // Build search argument dummy node.
    //----------------------------------
    node_tmp = alloca( node_size );
    if ( ! node_tmp ) return -2;
    str_ptr_ptr = node_tmp->str;
    va_start( var_arg, num_p );
    while ( ( * str_ptr_ptr ++ = va_arg( var_arg, const char * ) ) );
    va_end( var_arg );

    //---------------------------------------------
    // Search for existing node with these strings.
    //---------------------------------------------
    node_ptr = avl_find_exact( map_p->avl_str, node_tmp );

    //------------------------------------------------
    // If node is not found, just return a one status.
    //------------------------------------------------
    if ( ! node_ptr ) return 1;

    //----------------------------------------------------------
    // Else if found, store the number and return a zero status.
    //----------------------------------------------------------
    if ( num_p ) * num_p = node_ptr->num;
    return 0;
}
