//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/net
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "net_lib.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	ipv4_addr_in_cidr_count_arg
//
// purpose	Given a CIDR network block (address and prefix), and a list of
//		addresses in variable arguments (stopping at address 0.0.0.0),
//		return the count of how many of those addresses are in the
//		specified network block.
//
// arguments	1 (ipv4_t) address to define CIDR block
//		2 (int) prefix length (1..32)
//		3..N (ipv4_t) addresses to check
//
// returns	(int) count of addresses in the given CIDR block.
//-----------------------------------------------------------------------------
#define ipv4_addr_in_cidr_count_arg(n,p,a...) ((ipv4_addr_in_cidr_count_arg)((n),(p),a,((ipv4_t)0)))

__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	(ipv4_addr_in_cidr_count_arg)
//
// purpose	Given a CIDR network block (address and prefix), and a list of
//		addresses in variable arguments (stopping at address 0.0.0.0),
//		return the count of how many of those addresses are in the
//		specified network block.
//
// arguments	1 (ipv4_t) address to define CIDR block
//		2 (int) prefix length (1..32)
//		3..N (ipv4_t) addresses to check
//		N+1 (ipv4_t) 0 to indicate end of list
//
// returns	(int) count of addresses in the given CIDR block.
//-----------------------------------------------------------------------------
int
(ipv4_addr_in_cidr_count_arg) (
    ipv4_t		arg_network
    ,
    int			arg_prefix
    ,
    ...
    )
__PROTO_END__
{
    va_list		var_args	;
    ipv4_t		addr		;
    ipv4_t		base		;
    ipv4_t		mask		;
    int			count		;

    if ( arg_prefix > 32 || arg_prefix <= 0 ) return 0;

    mask = ( (ipv4_t) 0xffffffff ) << ( 32 - arg_prefix );
    base = arg_network & mask;

    count = 0;
    va_start( var_args, arg_prefix );
    while ( ( addr = va_arg( var_args, ipv4_t ) ) != (ipv4_t) 0 ) {
	if ( ( addr & mask ) == base ) ++ count;
    }

    return count;
}
