//-----------------------------------------------------------------------------
// Copyright  2005 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/string
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "net_lib.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	ipv4_table_empty
//
// purpose	Empty a no longer needed instance of an IPv4 lookup table.
//
// arguments	1 (ipv4_table_p) pointer to table to empty
//
// returns	(void)
//-----------------------------------------------------------------------------
#define ipv4_table_empty(t) ({							\
	ipv4_table_p libh__table_p;						\
	libh__table_p = (t);							\
	if ( libh__table_p && libh__table_p->apex ) {				\
		ipv4_table_empty_node( libh__table_p->apex.next[0], 31 );	\
		ipv4_table_empty_node( libh__table_p->apex.next[1], 31 );	\
	}									\
})

__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	ipv4_table_empty_node (internal only)
//
// purpose	Recursively empty each node of an IPv4 lookup table.
//
// arguments	1 (ipv4_table_node_p) pointer to table to empty
//		2 (int) depth
//
// returns	(void)
//-----------------------------------------------------------------------------
void
ipv4_table_empty_node (
    ipv4_table_node_p	arg_node
    ,
    int			arg_depth
    )
__PROTO_END__
{
    if ( ! arg_node ) return;

    if ( -- arg_depth ) {
	ipv4_table_empty_node( arg_node->next[0], arg_depth );
	ipv4_table_empty_node( arg_node->next[1], arg_depth );
    }

    arg_node->next[0]	= NULL;
    arg_node->next[1]	= NULL;
    arg_node->prev	= NULL;
    arg_node->data	= NULL;

    free( arg_node );

    return;
}
