//-----------------------------------------------------------------------------
// Copyright  2005 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/string
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "net_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	ipv4_table_find
//
// purpose	Find an IPv4 address entry in an IPv4 lookup table.
//
// arguments	1 (ipv4_table_p) pointer to table to look in
//		2 (ipv4_t) IPv4 address to look up
//
// returns	(void *) pointer associated with entry, or NULL if not found.
//-----------------------------------------------------------------------------
void *
ipv4_table_find (
    ipv4_table_p	arg_table
    ,
    ipv4_t		arg_addr
    )

__PROTO_END__
{
    ipv4_table_node_p	this_ptr	;
    ipv4_t		mask		;
    int			prefix		;


    if ( ! arg_table ) return NULL;

    //-------------------------------
    // Start at the apex of the tree.
    //-------------------------------
    this_ptr = (ipv4_table_node_p) & arg_table->apex;
    prefix = 0;
    mask = 0x80000000;

    //------------------------------------------------------
    // Start with the first data pointer even if it is NULL.
    //------------------------------------------------------
    arg_table->last = this_ptr;

    //--------------------------------
    // Scan for last set data pointer.
    //--------------------------------
    for (;;) {

	//----------------------------------------------------
	// This will be either the next node or the last data.
	//----------------------------------------------------
	this_ptr = this_ptr->next[ ( arg_addr & mask ) ? 1 : 0 ];
	if ( ! this_ptr ) break;

	//---------------------------------
	// If this is the lowest level, use
	// next pointers as data and quit.
	//---------------------------------
	if ( prefix == 32 ) {
	    arg_table->last = this_ptr;
	    arg_table->prefix = prefix;
	    break;
	}

	//-----------------------------------
	// If there is data here, pick it up.
	//-----------------------------------
	if ( this_ptr->data ) {
	    arg_table->last = this_ptr->data;
	    arg_table->prefix = prefix;
	}

	//---------------------------------
	// Go down to the next lower level.
	//---------------------------------
	++ prefix;
	mask >>= 1;
    }

    //----------------------------------------
    // Return the data from the selected node.
    //----------------------------------------
    return (void *) arg_table->last;
}
