//-----------------------------------------------------------------------------
// Copyright  2005 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/string
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "net_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	ipv4_table_find_all
//
// purpose	Find all IPv4 address entries in an IPv4 lookup table that
//		match a given IP address.
//
// arguments	1 (ipv4_table_p) pointer to table to look in
//		2 (ipv4_t) IPv4 address to look up
//		3 ((void *)[33]) array of associated data pointers found
//
// returns	(int) number of data pointers found
//
// note		The array pointed to in argument 3 is filled in indexed by
//		the prefix value from 0 to 32.
//-----------------------------------------------------------------------------
unsigned long
ipv4_table_find_all (
    ipv4_table_p	arg_table
    ,
    ipv4_t		arg_addr
    ,
    void * *		arg_list_p
    )

__PROTO_END__
{
    ipv4_table_node_p	this_ptr	;
    unsigned long	count		;
    ipv4_t		mask		;


    if ( ! arg_table ) return 0;

    //-------------------------------
    // Start at the apex of the tree.
    //-------------------------------
    this_ptr = (ipv4_table_node_p) & arg_table->apex;
    mask = 0x80000000;
    count = 0;

    //-----------------------------------------
    // Collect data pointers into result array.
    //-----------------------------------------
    for (;;) {

	//--------------------------------------------
	// Collect this data, and count it if present.
	//--------------------------------------------
	if ( ( * arg_list_p ++ = this_ptr->data ) ) ++ count;

        //----------------------------------------------------
        // This will be either the next node or the last data.
        //----------------------------------------------------
	this_ptr = this_ptr->next[ ( arg_addr & mask ) ? 1 : 0 ];
	if ( ! this_ptr ) break;

	//--------------------------------------
	// If this is the lowest level, use next
	// node pointer as the data, then quit.
	//--------------------------------------
	if ( mask == 1 ) {
	    if ( ( * arg_list_p ++ = (void *) this_ptr ) ) ++ count;
	    return count;
	}

	//---------------------------------
	// Go down to the next lower level.
	//---------------------------------
	mask >>= 1;
    }

    //---------------------------------------------------
    // Fill the remainder of the result array with NULLs.
    //---------------------------------------------------
    while ( ( mask >>= 1 ) ) {
	* arg_list_p ++ = NULL;
    }
    return count;
}
