//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/netstr
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "net_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	is_invalid_hostname
//
// purpose	Check if the given string represents an invalid hostname.
//		If it is invalid, return a pointer to the first invalid
//		character.  Otherwise return a NULL pointer.
//
// arguments	1 (const char *) string to check for hostname
//
// returns	(char *) pointer to first invalid character
//		(char *) NULL if string is a valid hostname
//-----------------------------------------------------------------------------
char *
is_invalid_hostname (
    const char *	arg_host_ptr
    )
__PROTO_END__
{
    char *		host_ptr	;
    int			this_ch		;
    int			start_part	;

    //-- Yes, this strips off the const attribute;
    * ( (const char * *) & host_ptr ) = arg_host_ptr;

    //-- Scan the hostname for bad stuff.
    start_part = 1;
    for (;;) {
	if ( ! ( this_ch = * host_ptr ) ) {
	    host_ptr = NULL;
	    break;
	}
	if ( '.' == this_ch ) {
	    if ( start_part ) break;
	    start_part = 1;
	} else {
	    start_part = 0;
	    if ( ! ( 'a' <= this_ch && this_ch <= 'z' ) &&
		 ! ( 'A' <= this_ch && this_ch <= 'Z' ) &&
		 ! ( '-' == this_ch ) ) break;
	}
	++ host_ptr;
    }
    return host_ptr;
}
