//-----------------------------------------------------------------------------
// Copyright  2010 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/optplus
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "optplus_lib.h"

__PROTO_BEGIN__
//---------------------------------------------------------------------------------------------------
// function	optplus_arg_dup
//
// purpose	Get the whole next argument (even if it looks like an option) from an option scanning
//		object as a heap allocated duplicate string.
//
// arguments	1 (optplus_p) pointer to option scanning object
//
// returns	(char *) == NULL : no more arguments or out of memory
//		(char *) != NULL : pointer to duplicate copy of argument string
//---------------------------------------------------------------------------------------------------
char *
optplus_arg_dup (
    optplus_p	this_optplus
    )
__PROTO_END__
{
    const char *	arg_ptr		;
    const char *	end_ptr		;
    char *		dup_ptr		;
    size_t		len		;

    if ( this_optplus->argx >= this_optplus->argz ) return NULL;	//-- no arguments remain
    if ( ! * this_optplus->argx ) return NULL;				//-- null argument pointer
    arg_ptr = * this_optplus->argx;					//-- get the next argument
    for ( end_ptr = arg_ptr; * end_ptr; ++ end_ptr );
    len = end_ptr + 1 - arg_ptr;
    dup_ptr = malloc( len );
    if ( dup_ptr ) {
	memcpy( dup_ptr, arg_ptr, len );
	this_optplus->argt = this_optplus->argx;			//-- remember this argument
	++ this_optplus->argx;						//-- update next for later
    }
    return dup_ptr;
}
