//-----------------------------------------------------------------------------
// Copyright  2010 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/optplus
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "optplus_lib.h"
#include <ctype.h>
#include <libh/string.h>

__PROTO_BEGIN__
//---------------------------------------------------------------------------------------------------
// function	optplus_boolean
//
// purpose	For options matching a given name, process the value as boolean and store the
//		the selected result.  If there is no match, do not store a value.
//
// arguments	1 (optplus_p) point to optplus object
//		2 (int *) where to store value
//		3 (int) value to store if the option value is false	--foo=no
//		4 (int) value to store if the option value is true	--foo=yes
//		5 (int) value to store if the option value is absent	--foo
//		6..N (const char *)  one or more names to match for this option
//
// returns	(int)  < 0 : error
//		(int) == 0 : no match
//		(int)  > 0 : match (value can be false, true, or absent)
//
// example	if ( optplus_boolean( opt_p, & flag, 0, 1, 1, "v", "verbose" ) ) continue;
//---------------------------------------------------------------------------------------------------
#define optplus_boolean(f,t,a,n...) ((optplus_boolean)((f),(t),(a),n...,(NULL)))
int
(optplus_boolean) (
    optplus_p	this_optplus
    ,
    int *	arg_value_p
    ,
    int		arg_false
    ,
    int		arg_true
    ,
    int		arg_absent
    ,
    ...
    )
__PROTO_END__
{
    va_list			var_args	;
    const unsigned char *	name_ptr	;
    const unsigned char *	cand_ptr	;
    int				value		;

    if ( ! this_optplus || ! this_optplus->name_str ) return -1;
    va_start( var_args, arg_absent );
    for (;;) {
	name_ptr = (const unsigned char *) this_optplus->name_str;
	cand_ptr = va_arg( var_args, const unsigned char * );		if ( ! cand_ptr ) break;
	while ( * cand_ptr && * name_ptr && * name_ptr != '=' &&
		tolower( * name_ptr ) == * cand_ptr ) { ++ name_ptr; ++ cand_ptr; }
	if ( * cand_ptr == 0 && ( * name_ptr == 0 || * name_ptr == '=' ) ) {
	    if ( this_optplus->value_str ) {
		value = str_yes_no( this_optplus->value_str ) ? arg_true : arg_false;
	    } else {
		value = arg_absent;
	    }
	    * arg_value_p = value;
	    return 1;
	}
    }
    return 0;
}
