//-----------------------------------------------------------------------------
// Copyright  2010 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/optplus
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "optplus_lib.h"

__PROTO_BEGIN__
//---------------------------------------------------------------------------------------------------
// function	optplus_name_cmp
//
// purpose	Compare the latest option name string to a given string.
//
// note		Since an option name may end at either the nul character or at
//		the '=' character, this comparison function allows acting on
//		either character as the end of the string for the option name.
//
// arguments	1 (optplus_p) pointer to option scanning object
//		2 (const char *) pointer to given string to compare
//
// returns	(int) -1 : option name string  < given string
//		(int)  0 : option name string == given string
//		(int)  1 : option name string  > given string
//---------------------------------------------------------------------------------------------------
int
optplus_name_cmp (
    optplus_p		this_optplus
    ,
    const char *	arg_string
    )
__PROTO_END__
{
    const unsigned char *	p1	;
    const unsigned char *	p2	;
    unsigned int		c1	;
    unsigned int		c2	;

    if ( ! ( p1 = (const unsigned char *) this_optplus->name_str ) ) return -1;
    if ( ! ( p2 = (const unsigned char *) arg_string ) ) return 1;
    while ( * p2 && * p1 == * p2 ) { ++ p1; ++ p2; }
    c1 = * p1;
    if ( c1 == '=' ) c1 = 0;
    c2 = * p2;
    return ( c1 < c2 ) ? -1 : ( c1 > c2 ) ? 1 : 0;
}
