//-----------------------------------------------------------------------------
// Copyright  2010 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/optplus
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "optplus_lib.h"
#include <ctype.h>

__PROTO_BEGIN__
//---------------------------------------------------------------------------------------------------
// function	optplus_name_match_nocase
//
// purpose	Compare the latest option name string to a given string
//		ignoring case and return an option match state.
//
// note		Since an option name may end at either the nul character or at
//		the '=' character, this comparison function allows acting on
//		either character as the end of the string for the option name.
//
// arguments	1 (optplus_p) pointer to option scanning object
//		2..N (const char *) pointers to given strings to compare
//
// returns	(int)  < 0 : error
//		(int) == 0 : given string does not match option name
//		(int)  > 0 : given string does match option name
//---------------------------------------------------------------------------------------------------
#define optplus_name_match_nocase(o,a...) ((optplus_name_match_nocase)((o),a,(NULL)))
int
(optplus_name_match_nocase) (
    optplus_p		this_optplus
    ,
    ...
    )
__PROTO_END__
{
    va_list			var_args	;
    const unsigned char *	name_ptr	;
    const unsigned char *	cand_ptr	;
    int				match_num	;

    if ( ! this_optplus || ! this_optplus->name_str ) return -1;
    va_start( var_args, this_optplus );
    for ( match_num = 1; ++ match_num; ) {
	name_ptr = (const unsigned char *) this_optplus->name_str;
	cand_ptr = va_arg( var_args, const unsigned char * );		if ( ! cand_ptr ) break;
	while ( * cand_ptr && * name_ptr && * name_ptr != '=' &&
		tolower( * name_ptr ) == tolower( * cand_ptr ) ) { ++ name_ptr; ++ cand_ptr; }
	if ( * cand_ptr == 0 && ( * name_ptr == 0 || * name_ptr == '=' ) ) goto optplus_name_match_nocase_found;
    }
    match_num = 0;

 optplus_name_match_nocase_found:
    va_end( var_args );
    return match_num;
}
