//-----------------------------------------------------------------------------
// Copyright  2010 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/optplus
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "optplus_lib.h"

__PROTO_BEGIN__
//---------------------------------------------------------------------------------------------------
// function	optplus_next
//
// purpose	Get the next option or argument from an option scanning object,
//		returning a code for the type of option or argument, or the
//		status of an exception.
//
// arguments	1 (optplus_p) pointer to option scanning object
//
// returns	(int) ==  -1 : error
//		(int) ==   0 : no more options or arguments
//		(int) ==   1 : string or name=value
//		(int) == '-' : -x or -x=value
//		(int) == '+' : +x or +x=value
//		(int) == char_code_2('-','-') : --option or --option=value
//		(int) == char_code_2('-','+') : -+option or -+option=value
//		(int) == char_code_2('+','-') : +-option or +-option=value
//		(int) == char_code_2('+','+') : ++option or ++option=value
//---------------------------------------------------------------------------------------------------
//	#define OPTPLUS_TYPE_ERROR		-1
//	#define OPTPLUS_TYPE_END		0
//	#define OPTPLUS_TYPE_ARGUMENT		1
//	#define OPTPLUS_TYPE_MINUS		'-'
//	#define OPTPLUS_TYPE_PLUS		'+'
//	#define OPTPLUS_TYPE_MINUS_MINUS	char_code_2('-','-')
//	#define OPTPLUS_TYPE_MINUS_PLUS		char_code_2('-','+')
//	#define OPTPLUS_TYPE_PLUS_MINUS		char_code_2('+','-')
//	#define OPTPLUS_TYPE_PLUS_PLUS		char_code_2('+','+')
//---------------------------------------------------------------------------------------------------
int
optplus_next (
    optplus_p	this_optplus
    )
__PROTO_END__
{
    const char *	arg_ptr		;
    const char *	end_ptr		;

    if ( this_optplus->count > 0 &&
	 this_optplus->type <= 0 ) return this_optplus->type;	//-- previously no more
    this_optplus->count ++;

    //------------------------------------------------
    // Repeat until an option is found or none remain.
    //------------------------------------------------
    for (;;) {

	//-----------------------------------------------------------------
	// If a single letter option group is being scanned, continue that.
	//-----------------------------------------------------------------
	if ( this_optplus->char_ptr ) {
	    int ch;
	    ch = ( * ++ this_optplus->char_ptr );	//-- next single character
	    if ( ! ch ) {				//-- end of this argument
		this_optplus->pre_char[0] = 0;
		this_optplus->char_ptr = NULL;
		continue;				//-- continue at next argument
	    }
	    if ( ch == '-' || ch == '+' ) {		//-- change prefix
		this_optplus->pre_char[0] = ch;
		continue;				//-- continue in this argument
	    }
	    if ( this_optplus->char_ptr[1] == '=' ) {	//-- single letter assignment
		this_optplus->value_str = this_optplus->char_ptr + 2;
		for ( end_ptr = this_optplus->value_str; * end_ptr; ++ end_ptr );
		this_optplus->value_end = end_ptr;
		this_optplus->value_len = end_ptr - this_optplus->value_str;
		this_optplus->char_ptr = NULL;		//-- no more single letters here
	    }
	    this_optplus->char_code[0] = ch;		//-- name is single letter
	    this_optplus->name_str = this_optplus->char_code;
	    this_optplus->name_end = this_optplus->char_code + 1;
	    this_optplus->type = this_optplus->pre_char[0];
	    return this_optplus->type;			//-- return prefix as type
	}

	//-----------------------------------------------------------------------------
	// Use another argument.
	//-----------------------------------------------------------------------------
	if ( this_optplus->argx >= this_optplus->argz ) break;		//-- no arguments remain
	if ( ! * this_optplus->argx ) break;				//-- null argument pointer
	arg_ptr = * ( this_optplus->argt = this_optplus->argx );	//-- get the next argument
	++ this_optplus->argx;						//-- update next for later
	for ( end_ptr = arg_ptr; * end_ptr; ++ end_ptr );		//-- get string end pointer

	//-----------------------------------------------------------------------------
	// Check if argument is prefixed, either short or long.
	//-----------------------------------------------------------------------------
	if ( arg_ptr[0] == '-' || arg_ptr[0] == '+' ) {
	    this_optplus->pre_char[0] = arg_ptr[0];
	    if ( arg_ptr[1] == 0 ) {				//-- single prefix alone
		this_optplus->name_str = NULL;			//-- no name
		this_optplus->name_end = NULL;
		this_optplus->name_len = 0;
		this_optplus->value_str = NULL;			//-- no value
		this_optplus->value_end = NULL;
		this_optplus->value_len = 0;
		this_optplus->pre_char[1] = 0;
		this_optplus->type = this_optplus->pre_char[0];
		return this_optplus->type;			//-- return prefix as type
	    }
	    if ( arg_ptr[1] != '-' && arg_ptr[1] != '+' ) {	//-- single prefix
		this_optplus->char_ptr = arg_ptr;
		this_optplus->name_len = 1;			//-- name will be one character
		this_optplus->value_str = NULL;			//-- usually no value
		this_optplus->value_end = NULL;
		this_optplus->value_len = 0;
		this_optplus->pre_char[1] = 0;
		continue;					//-- go pick single option
	    }
	    this_optplus->pre_char[1] = arg_ptr[1];		//-- double prefix
	    if ( arg_ptr[2] == 0 ) {				//-- double prefix alone
		this_optplus->name_str = NULL;			//-- no name
		this_optplus->name_end = NULL;
		this_optplus->name_len = 0;
		this_optplus->value_str = NULL;			//-- no value
		this_optplus->value_end = NULL;
		this_optplus->value_len = 0;
	    } else {
		const char * value_ptr;
		arg_ptr += 2;					//-- step past double prefix
		value_ptr = strchr( arg_ptr, '=' );		//-- check if assignment
		if ( value_ptr ) {
		    this_optplus->name_str = arg_ptr;		//-- name part
		    this_optplus->name_end = value_ptr;
		    this_optplus->name_len = value_ptr - arg_ptr;
		    this_optplus->value_str = ++ value_ptr;	//-- value part
		    this_optplus->value_end = end_ptr;
		    this_optplus->value_len = end_ptr - value_ptr;
		} else {
		    this_optplus->name_str = arg_ptr;		//-- option name
		    this_optplus->name_end = end_ptr;
		    this_optplus->name_len = end_ptr - arg_ptr;
		    this_optplus->value_str = NULL;
		    this_optplus->value_end = NULL;
		    this_optplus->value_len = 0;
		}
	    }
	    this_optplus->type = char_code_2( this_optplus->pre_char[1], this_optplus->pre_char[0] );
	    return this_optplus->type;			//-- return prefix as type
	}

	//-----------------------------------------------------------------------------
	// Argument has no prefix.
	//-----------------------------------------------------------------------------
	this_optplus->pre_char[0] = 0;
	this_optplus->pre_char[1] = 0;
	{
	    const char * value_ptr;
	    value_ptr = strchr( arg_ptr, '=' );			//-- check if assignment
	    if ( value_ptr ) {
		this_optplus->name_str = arg_ptr;		//-- name part
		this_optplus->name_end = value_ptr;
		this_optplus->name_len = value_ptr - arg_ptr;
		this_optplus->value_str = ++ value_ptr;		//-- value part
		this_optplus->value_end = end_ptr;
		this_optplus->value_len = end_ptr - value_ptr;
	    } else {
		this_optplus->name_str = arg_ptr;		//-- whole argument
		this_optplus->name_end = end_ptr;
		this_optplus->name_len = end_ptr - arg_ptr;
		this_optplus->value_str = NULL;
		this_optplus->value_end = NULL;
		this_optplus->value_len = 0;
	    }
	}
	this_optplus->type = OPTPLUS_TYPE_ARGUMENT;
	return OPTPLUS_TYPE_ARGUMENT;
    }

    //----------------------------------------------------------------------------------------
    // No more arguments available.
    //----------------------------------------------------------------------------------------
    this_optplus->pre_char[0] = 0;
    this_optplus->pre_char[1] = 0;
    this_optplus->type = 0;
    return 0;
}
