//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/string
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "string_lib.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	mem_cpy
// alias	mem_copy
//
// purpose	Copy to specified space, limited by given size, the contents
//		of one or more strings of bytes specified with pointer and
//		length pairs.  Multiple data sources are concatenated.
//
// arguments	1 (char *) target space to copy strings to
//		2 (size_t) maximum size of target space
//		3 (const char *) source string to copy
//		4 (size_t) length of source string to copy
//		... repeat last 2
//
// returns	(size_t) total bytes actually copied
//-----------------------------------------------------------------------------
#define mem_cpy(t,s,p,x...) ((mem_cpy)((t),(s),(p),x,(const char*)(NULL)))
#define mem_copy(t,s,p,x...) ((mem_cpy)((t),(s),(p),x,(const char*)(NULL)))

__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	mem_cpy
//
// purpose	Copy to specified space, limited by given size, the contents
//		of one or more strings of bytes specified with pointer and
//		length pairs.  Multiple data sources are concatenated.
//
// arguments	1 (char *) target space to copy strings to
//		2 (size_t) maximum size of target space
//		3 (const char *) source string to copy
//		4 (size_t) length of source string to copy
//		... repeat last 2
//		N (const char *) NULL to end variable arguments
//
// returns	(size_t) total bytes actually copied
//-----------------------------------------------------------------------------
size_t
(mem_cpy) (
    char *		arg_target
    ,
    size_t		arg_size
    ,
    const char *	arg_source
    ,
    size_t		arg_len
    ,
    ...
    )
__PROTO_END__
{
    va_list		var_args	;
    char *		target_ptr	;

    target_ptr = arg_target;
    va_start( var_args, arg_len );
    for (;;) {

	//-- Truncated or exact, this is the last to copy.
	if ( arg_len >= arg_size ) {
	    memcpy( target_ptr, arg_source, arg_size );
	    break;
	}

	//-- Copy a whole string.
	memcpy( target_ptr, arg_source, arg_len );
	target_ptr += arg_len;
	arg_size -= arg_len;

	//-- Get next string.
	arg_source = va_arg( var_args, const char * );
	if ( ! arg_source ) break;
	arg_len = va_arg( var_args, size_t );
    }
    va_end( var_args );

    //-- Return the length actually copied.
    return target_ptr - arg_target;
}
