//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/string
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "string_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	str_app_ch
//
// purpose	Append to the first string (given with maximum available space)
//		a single character value.
//
//		Do append only if there is space for one more character.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (int) (char) character to be appended
//
// returns	(size_t) new length of string, or ~0 for error
//-----------------------------------------------------------------------------
size_t
str_app_ch (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    int			arg_char
    )
__PROTO_END__
{
    char *		target_ptr	;
    char *		target_end	;

    //-- Establish bounds.
    target_ptr = arg_target;
    target_end = target_ptr + arg_length - 1;

    //-- Find end of existing string.
    while ( target_ptr < target_end && * target_ptr ) ++ target_ptr;

    //-- If no space available, abort.
    if ( target_ptr >= target_end ) return ~0;

    //-- Append one character and new terminator.
    * target_ptr = arg_char;
    * ++ target_ptr = 0;

    //-- Return with new length.
    return target_ptr - arg_target;
}
