//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/string
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "string_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	str_app_ipv4
//
// purpose	Append to the first string (given with maximum available space)
//		a string of dotted decimal digits representing the IPv4 network
//		address given as an unsigned long.
//
//		Do append only if all the digits can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (unsigned long) the IPv4 address to convert
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_ipv4 (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    unsigned long	arg_ipv4
    )
__PROTO_END__
{
    char *		target_ptr		;
    char *		target_end		;
    unsigned int	octet			;
    int			count			;

    //-- Determine end of string and check for space.
    target_end = ( target_ptr = arg_target ) + arg_length - 1;
    while ( target_ptr < target_end && * target_ptr ) ++ target_ptr;
    if ( target_ptr >= target_end ) return ~0;

    //-- Convert IP address the hard way.
    count = 4;
    for (;;) {
	octet = arg_ipv4 >> 24 & 255;
	if ( octet >= 100 ) {
	    * target_ptr = octet / 100 + '0';
	    octet %= 100;
	}
	if ( ++ target_ptr >= target_end ) goto fail;
	if ( octet >= 10 ) {
	    * target_ptr = octet / 10 + '0';
	    octet %= 10;
	}
	if ( ++ target_ptr >= target_end ) goto fail;
	* target_ptr = octet + '0';
	if ( -- count == 0 ) break;
	arg_ipv4 <<= 8;
	if ( ++ target_ptr >= target_end ) goto fail;
	* target_ptr = '.';
    }

    //-- Get new length and return it.
    while ( target_ptr < target_end && * target_ptr ) ++ target_ptr;
    return target_ptr - arg_target;

 fail:
    //-- Undo appends, restore original string, and return error.
    * target_ptr = 0;
    return ~0;
}
