//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/string
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "string_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	str_app_lower
//
// purpose	Append to the first string (given with maximum available space)
//		the contents of another string converted to lower case.
//
//		Do append only if the entire source string can be appended.
//
// arguments	1 (char *) pointer to target string
//		2 (size_t) maximum space in target string
//		3 (const char *) source string to append
//
// returns	(size_t) new length of string, or ~0 for error
//
// note		Contents of available space after the termination character
//		are destroyed if the append fails due to insufficient space.
//		Nothing should be kept there, anyway.
//-----------------------------------------------------------------------------
size_t
str_app_lower (
    char *		arg_target
    ,
    size_t		arg_length
    ,
    const char *	arg_source
    )
__PROTO_END__
{
    char *		target_ptr	;
    char *		target_end	;
    char *		target_undo	;
    const char *	source_ptr	;

    //-- Determine end of string and check for space.
    target_ptr = arg_target;
    target_end = target_ptr + arg_length - 1;
    while ( target_ptr < target_end && * target_ptr ) ++ target_ptr;
    if ( target_ptr >= target_end ) return ~0;

    //-- Remember where to undo the append.
    target_undo = target_ptr;

    //-- Append new string until end or no more space.
    source_ptr = arg_source;
    while ( target_ptr < target_end && ( * target_ptr = tolower( * source_ptr ) ) ) {
	++ target_ptr;
	++ source_ptr;
    }

    //-- If the string did not fit, undo and abort.
    if ( target_ptr >= target_end ) {
	* target_undo = 0;
	return ~0;
    }

    //-- Return with new length.
    return target_ptr - arg_target;
}
