//-----------------------------------------------------------------------------
// Copyright  2010 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/string
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "string_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	str_cmp_path_upper
//
// purpose	Compare two strings, while treating each instance of a given
//		character (delimiter) as the lowest value for comparison, and
//		ignoring case by treating alphabetic characters as upper case.
//
// note		The str_cmp_path_lower() and str_cmp_path_upper() functions
//		will have different collating sequences of alphabetics when
//		compared to other characters that are between lower and upper
//		character codes.
//
// arguments	1 (const char *) string one
//		2 (const char *) string two
//		3 (int) delimiter character
//
// returns	(int)  < 0 : string one  < string two
//		(int) == 0 : string one == string two
//		(int)  > 0 : string one  > string two
//-----------------------------------------------------------------------------
int
str_cmp_path_upper (
    const char *	arg_str_one
    ,
    const char *	arg_str_two
    ,
    int			arg_del_char
    )
__PROTO_END__
{
    unsigned const char *	p1	;
    unsigned const char *	p2	;
    int				c1	;
    int				c2	;

    p1 = (unsigned const char *) arg_str_one;
    p2 = (unsigned const char *) arg_str_two;
    for (;;) {
	c1 = * p1;
	c2 = * p2;
	if ( ! c1 ) break;
	if ( ! c2 ) break;
	if ( c1 == arg_del_char ) c1 = 0;	c1 = toupper( c1 );
	if ( c2 == arg_del_char ) c2 = 0;	c2 = toupper( c2 );
	if ( c1 != c2 ) break;
	++ p1;
	++ p2;
    }
    return (int) c1 - (int) c2;
}
