//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/string
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "string_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	str_dup_lim
//
// purpose	Duplicate into allocated space the contents of another string
//		limited by number of characters.
//
// arguments	1 (const char *) source string to duplicate
//		2 (size_t) limit length of source string
//
// returns	(char *) new allocated duplicate string, caller must free
//
// note		This function is similar to the common but non-standard
//		function strdup().
//-----------------------------------------------------------------------------
char *
str_dup_lim (
    const char *	arg_source
    ,
    size_t		arg_srclen
    )
__PROTO_END__
{
    char *		ptr_string	;
    size_t		len_source	;

    //-- Determine end of string and allocate new space.
    {
	const char *	ptr_source	;

	ptr_source = arg_source;
	while ( * ptr_source ) ++ ptr_source;
	len_source = ptr_source - arg_source;
	if ( len_source > arg_srclen ) len_source = arg_srclen;
	ptr_string = (char *) malloc( len_source + 1 );
	if ( ! ptr_string ) return ptr_string;
    }

    //-- Copy contents to new string.
    {
	const char *	ptr_source	;
	char *		ptr_target	;

	ptr_source = arg_source;
	ptr_target = ptr_string;
	while ( len_source -- && ( * ptr_target = * ptr_source ) ) {
	    ++ ptr_source;
	    ++ ptr_target;
	}
	* ptr_target = 0;
    }

    return ptr_string;
}
