//-----------------------------------------------------------------------------
// Copyright  2003 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/string
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "string_lib.h"

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	str_find_nocase_arg
//
// purpose      Search arguments 2..N for a string that matches argument 1,
//              returning 0 for not found, and 1..N-1 for which argument is
//              found.  The comparison is NOT case sensitive.
//
// arguments    1 (const char *) string to search for
//              2..N (const char *) strings to search
//
// returns      (int) == -1 : error
//              (int) ==  0 : not found
//              (int>  >  0 : found string at argument N-1
//-----------------------------------------------------------------------------
#define str_find_nocase_arg(s,a...) ((str_find_nocase_arg)((s),a,(const char*)(NULL)))

__FMACRO_END__

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	(str_find_nocase_arg)
//
// purpose      Search arguments 2..N for a string that matches argument 1,
//              returning 0 for not found, and 1..N-1 for which argument is
//              found.  The comparison is NOT case sensitive.
//
// arguments    1 (const char *) string to search for
//              2..N (const char *) strings to search
//		N+1 (const char *) NULL to indicate end of list
//
// returns      (int) == -1 : error
//              (int) ==  0 : not found
//              (int>  >  0 : found string at argument N-1
//-----------------------------------------------------------------------------
size_t
(str_find_nocase_arg) (
    const char *	arg_string
    ,
    ...
    )
__PROTO_END__
{
    va_list		var_args	;
    const char *	next_string	;
    int			count		;

    if ( ! arg_string ) return -1;
    va_start( var_args, arg_string );
    count = 0;
    while ( ( next_string = va_arg( var_args, const char * ) ) ) {
	++ count;
	if ( str_cmp_lower( arg_string, next_string ) == 0 ) goto finish;
    }
    count = 0;

 finish:
    va_end( var_args );
    return count;
}
