//-----------------------------------------------------------------------------
// Copyright  2010 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/optplus
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "string_lib.h"
#include <ctype.h>

__PROTO_BEGIN__
//---------------------------------------------------------------------------------------------------
// function	va_str_match_lower
//
// purpose	Check if a given candidate (ended at nul or an alternate termination character)
//		matches any string in a va_list. Each string in list must already be in lower case
//		to match.  The candidate string may be lower or upper or mixed.  Characters are
//		compared in lower case.
//
// arguments	1 (const char *) given string to match (nul or alt terminated strings)
//		2 (int) alternate string termination character
//		3 (va_list) list of strings to match (NULL terminated list of nul terminated strings)
//
// returns	(int)  < 0 : error
//		(int) == 0 : no match
//		(int)  > 0 : match, index of match starting at 1
//---------------------------------------------------------------------------------------------------
int
va_str_match_lower (
    const char *	arg_name
    ,
    int			arg_alt
    ,
    va_list		arg_list
    )
__PROTO_END__
{
    const char *	name_ptr	;
    const char *	cand_ptr	;
    int			number		;

    number = 0;
    for (;;) {
	name_ptr = arg_name;
	cand_ptr = va_arg( arg_list, const char * );
	if ( ! cand_ptr ) return 0;
	++ number;
	while ( * cand_ptr &&
		* name_ptr &&
		* name_ptr != arg_alt &&
		tolower( * name_ptr ) == * cand_ptr ) {
	    ++ name_ptr;
	    ++ cand_ptr;
	}
	if ( * cand_ptr == 0 && ( * name_ptr == 0 || * name_ptr == arg_alt ) ) return number;
    }
}
