//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/time
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------
// reference	Explanatory Supplement to the Astronomical Almanac,
//		P. Kenneth Seidelmann, editor  [ISBN 0-935702-68-7]
//		http://shop.bn.com/bookSearch/isbnInquiry.asp?isbn=0935702687
//-----------------------------------------------------------------------------

#include <ctype.h>

#include "time_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	ecal_to_eday
//
// purpose	Convert calendar elements to an Earth Day value.
//
// arguments	1 (ecal_t *) calendar struct to be converted
//
// returns	(eday_t) resultant Earth Day value
//-----------------------------------------------------------------------------
eday_t
ecal_to_eday (
    ecal_t *	arg_cal_p
    )
__PROTO_END__
{
    //---------------------------
    // Validate pointer argument.
    //---------------------------
    if ( ! arg_cal_p ) return EDAY_MIN;

    //------------------------------------------------------------------
    // Choose between Gregorian and Julian.
    //------------------------------------------------------------------
    if ( arg_cal_p->type == ECAL_TYPE_GREGORIAN ) {
	return ymd_gregorian_to_eday( arg_cal_p->year, arg_cal_p->month, arg_cal_p->mday );
    }
    else if ( arg_cal_p->type == ECAL_TYPE_JULIAN ) {
	return ymd_julian_to_eday( arg_cal_p->year, arg_cal_p->month, arg_cal_p->mday );
    }
    return EDAY_MIN;
}
