//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/time
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------
// reference	Explanatory Supplement to the Astronomical Almanac,
//		P. Kenneth Seidelmann, editor  [ISBN 0-935702-68-7]
//		http://shop.bn.com/bookSearch/isbnInquiry.asp?isbn=0935702687
//-----------------------------------------------------------------------------

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	eday_get_day_of_week_gregorian
// alias	eday_get_day_of_week
//
// purpose	Extract day of week in the Gregorian calendar from eday_t.
//
// argument	1 (eday_t) Earth Day value
//
// returns	(int) Gregorian calendar day of week (1 .. 7)
//
// note		The result is always the same for both Gregorian and Julian.
//-----------------------------------------------------------------------------
#define eday_get_day_of_week_gregorian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_gregorian( ( e ), & libh__ecal );				\
	libh__ecal.wday;							\
})

#define eday_get_day_of_week(e) ({						\
	ecal_t libh__ecal;							\
	eday_to_ecal( ( e ), & libh__ecal );					\
	libh__ecal.wday;							\
})

//-----------------------------------------------------------------------------
// function	eday_get_day_of_week_julian
//
// purpose	Extract day of week in the Julian calendar from eday_t.
//
// argument	1 (eday_t) Earth Day value
//
// returns	(int) Julian calendar day of week (1 .. 7)
//
// note		The result is always the same for both Gregorian and Julian.
//-----------------------------------------------------------------------------
#define eday_get_day_of_week_julian(e) ({					\
	ecal_t libh__ecal;							\
	eday_to_ecal_julian( ( e ), & libh__ecal );					\
	libh__ecal.wday;							\
})

__FMACRO_END__
