//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/time
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------
// reference	Explanatory Supplement to the Astronomical Almanac,
//		P. Kenneth Seidelmann, editor  [ISBN 0-935702-68-7]
//		http://amazon.com/exec/obidos/ASIN/0935702687
//		http://shop.bn.com/bookSearch/isbnInquiry.asp?isbn=0935702687
//-----------------------------------------------------------------------------

#include "time_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	eday_to_timespec
//
// purpose	Convert earth day to struct timespec.  The converted time is
//		the beginning of the day.
//
// arguments	1 (struct timespec *) pointer to destination struct timespec
//		2 (eday_t) earth day
//
// returns	(void)
//-----------------------------------------------------------------------------
void
eday_to_timespec (
    struct timespec *	arg_timets_p
    ,
    eday_t		arg_eday
    )
__PROTO_END__
{
    if ( arg_eday < EDAY_SYSTEM_MIN ) {
	arg_timets_p->tv_sec = TIME_MIN;
	arg_timets_p->tv_nsec = 0;
    } else if ( arg_eday > EDAY_SYSTEM_MAX ) {
	arg_timets_p->tv_sec = TIME_MAX;
	arg_timets_p->tv_nsec = (long) 999999999;
    } else {
	arg_timets_p->tv_sec = ( arg_eday - EDAY_SYSTEM_EPOCH ) * 86400;
	arg_timets_p->tv_nsec = 0;
    }
    return;
}
