//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/time
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------
// reference	Explanatory Supplement to the Astronomical Almanac,
//		P. Kenneth Seidelmann, editor  [ISBN 0-935702-68-7]
//		http://shop.bn.com/bookSearch/isbnInquiry.asp?isbn=0935702687
//-----------------------------------------------------------------------------

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	etime_get_ymdhmsu_gregorian
// alias	etime_get_ymdhmsu
//
// purpose	Extract calendar and time of day elements in the Gregorian
//		calendar from etime_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour
//		6 (int *) where to store minute
//		7 (int *) where to store second
//		8 (long *) where to store macroseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymdhmsu_gregorian(t,yr,mo,da,hr,mn,se,ms) ({			\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd_gregorian(libh__etime,(yr),(mo),(da));			\
	etime_get_hmsu(libh__etime,(hr),(mn),(se),(ms));			\
})

#define etime_get_ymdhmsu(t,yr,mo,da,hr,mn,se,ms) ({				\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd(libh__etime,(yr),(mo),(da));				\
	etime_get_hmsu(libh__etime,(hr),(mn),(se),(ms));			\
})

//-----------------------------------------------------------------------------
// macro	etime_get_ymdhmsu_julian
//
// purpose	Extract calendar and time of day elements in the Gregorian
//		calendar from etime_t.
//
// arguments	1 (eday_t) Earth Day value
//		2 (long *) where to store year
//		3 (int *) where to store month
//		4 (int *) where to store day of month
//		5 (int *) where to store hour
//		6 (int *) where to store minute
//		7 (int *) where to store second
//		8 (long *) where to store macroseconds
//
// returns	(void)
//-----------------------------------------------------------------------------
#define etime_get_ymdhmsu_julian(t,yr,mo,da,hr,mn,se,ms) ({			\
	etime_t libh__etime;							\
	libh__etime = (t);							\
	etime_get_ymd_julian(libh__etime,(yr),(mo),(da));			\
	etime_get_hmsu(libh__etime,(hr),(mn),(se),(ms));			\
})

__FMACRO_END__
