//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/time
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------
// reference	Explanatory Supplement to the Astronomical Almanac,
//		P. Kenneth Seidelmann, editor  [ISBN 0-935702-68-7]
//		http://shop.bn.com/bookSearch/isbnInquiry.asp?isbn=0935702687
//-----------------------------------------------------------------------------

#include <ctype.h>

#include "time_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	etime_to_ecal_julian
//
// purpose	Convert an Earth Time value to Julian calendar elements.
//
// arguments	1 (etime_t) Earth Time value
//		2 (ecal_t *) where to store calendar elements.
//
// returns	(int) 0 : OK
//-----------------------------------------------------------------------------
int
etime_to_ecal_julian (
    etime_t	arg_etime
    ,
    ecal_t *	arg_ecal_p
    )
__PROTO_END__
{
    unsigned long	usecs	;

    eday_to_ecal_julian( etime_to_eday( arg_etime ), arg_ecal_p );

    arg_etime		= mod_ni( arg_etime, ETIME_C( 86400000000 ) );
    arg_ecal_p->hour	= arg_etime / ETIME_C( 3600000000 );
    usecs		= arg_etime % ETIME_C( 3600000000 );
    arg_ecal_p->min	= usecs / 60000000U;
    usecs		%= 60000000U;
    arg_ecal_p->sec	= usecs / 1000000U;
    arg_ecal_p->nsec	= ( usecs % 1000000U ) * 1000U;

    return 0;
}
