//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/time
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------
// reference	Explanatory Supplement to the Astronomical Almanac,
//		P. Kenneth Seidelmann, editor  [ISBN 0-935702-68-7]
//		http://shop.bn.com/bookSearch/isbnInquiry.asp?isbn=0935702687
//-----------------------------------------------------------------------------

#include <math.h>

#include "time_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	jd_to_timems
//
// purpose	Convert a Julian Date value as used in astronomy to a system
//		time value in milliseconds.
//
// arguments	1 (JD) Julian Date value
//
// returns	(timems_t) system time value in milliseconds
//-----------------------------------------------------------------------------
timems_t
jd_to_timems (
    JD		arg_jd
    )
__PROTO_END__
{
    long double		ld_timems	;

    ld_timems = (long double) arg_jd * 86400000.0L - 210866803199999.5L;
    return ld_timems > (long double) INT64_MAX ? INT64_MAX
	:  ld_timems < (long double) INT64_MIN ? INT64_MIN
	:  (timems_t) ld_timems;
}
