//-----------------------------------------------------------------------------
// Copyright  2006 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/time
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	str_gregorian_to_ecal
// macro	str_to_ecal
//
// purpose	Convert date and time in a string to type ecal_t for the
//		Gregorian calendar.
//
// arguments	1 (const char *) date and time string
//		2 (ecal_t *) where to store calendar and time data.
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	yyyy/mm/dd[_hh:mm:ss.nnnnnnnnnnnnnnnnnn]
//
// note		Any non-numeric character may be used as separators, but
//		such use should be limited to reasonable punctuation only
//		for future compatibility.
//
// note		Time parts may be specified but will be ignored.
//-----------------------------------------------------------------------------
#define str_gregorian_to_ecal(s,c) ({						\
	ecal_t * libh__cal;							\
	int libh__rc;								\
	libh__cal = (c);							\
	libh__rc = str_cal_to_ecal((s),libh__cal);				\
	if ( libh__rc == 0 ) libh__cal->type = ECAL_TYPE_GREGORIAN;		\
	libh__rc;								\
})

#define str_to_ecal(s,c) ({							\
	ecal_t * libh__cal;							\
	int libh__rc;								\
	libh__cal = (c);							\
	libh__rc = str_cal_to_ecal((s),libh__cal);				\
	if ( libh__rc == 0 ) libh__cal->type = ECAL_TYPE_GREGORIAN;		\
	libh__rc;								\
})

__FMACRO_END__
