//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/time
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------

#include "time_lib.h"

__PROTO_BEGIN__
//-----------------------------------------------------------------------------
// function	str_to_timespec
//
// purpose	Convert a time period expressed in a string with suffixes and
//		decimal fractions to timespec, stored where specified.
//
// arguments	1 (const char *) time string
//		2 (struct timespec *) timespec to store result in
//
// returns	(int) == 0 : OK
//		(int)  < 0 : error
//
// format	[#[.#][wdhms]]
//-----------------------------------------------------------------------------
int
str_to_timespec (
    const char *	arg_time_str
    ,
    struct timespec *	arg_timespec_p
    )
__PROTO_END__
{
    int_fast64_t		time_ns		;

    if ( ! arg_time_str ) return -1;
    if ( ! arg_timespec_p ) return -1;
    time_ns = str_to_nsecs( arg_time_str );
    if ( time_ns < 0 ) return (int) time_ns;
    timens_to_timespec( arg_timespec_p, time_ns );
    return 0;
}
