//-----------------------------------------------------------------------------
// Copyright  2004 - Philip Howard - All rights reserved
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//-----------------------------------------------------------------------------
// package	libh/time
// homepage	http://libh.slashusr.org/
//-----------------------------------------------------------------------------
// author	Philip Howard
// email	libh at ipal dot org
// homepage	http://phil.ipal.org/
//-----------------------------------------------------------------------------
// This file is best viewed using a fixed spaced font such as Courier
// and in a display at least 120 columns wide.
//-----------------------------------------------------------------------------
// reference	Explanatory Supplement to the Astronomical Almanac,
//		P. Kenneth Seidelmann, editor  [ISBN 0-935702-68-7]
//		http://shop.bn.com/bookSearch/isbnInquiry.asp?isbn=0935702687
//-----------------------------------------------------------------------------

__FMACRO_BEGIN__
//-----------------------------------------------------------------------------
// macro	ymdhmsm_gregorian_to_etime
//
// purpose	Convert a Gregorian calendar date, plus time in hours, minutes,
//		seconds, and milliseconds, to an Earth Time value.
//
// arguments	1 (long) Gregorian calendar year (0 = 1 BC, 1 = 1 AD)
//		2 (int) Gregorian calendar month (1-12)
//		3 (int) Gregorian calendar date (1-31)
//		4 (int) Hours part
//		5 (int) Minutes part
//		6 (int) Seconds part
//		7 (long) Milliseconds part
//
// returns	(etime_t) resultant Earth Time value (beginning of day)
//
// note		The calculation is for the proleptic Gregorian calendar.
//		There is no accomodation for selection between Gregorian
//		and Julian calendars.  Since different areas of the world
//		switched calendars at different dates, the calling program
//		will have to determine on its own what is the appropriate
//		calendar context and call ymd_gregorian_to_eday() or ymd_julian_to_eday().
//-----------------------------------------------------------------------------
#define ymdhmsm_gregorian_to_etime(yr,mo,da,hr,mi,se,ms) (ymd_gregorian_to_etime((yr),(mo),(da))+hmsm_to_usec((hr),(mi),(se),(ms)))

__FMACRO_END__
